package QBank::BankUtils;

require "qbank.ph";
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(%ARGS min max authorize bank_log);

###############################################################
# min($value1,$value2)
# Returns the minimum value
###############################################################
sub min
{ return $_[0]<=$_[1]?$_[0]:$_[1]; }

###############################################################
# max($value1,$value2)
# Not currently being used
# Returns the maximum value
###############################################################
sub max
{ return $_[0]>=$_[1]?$_[0]:$_[1]; }

##############################################################################
# authorize($auth_level,@command)
#
# Exclusively used within other subroutines for command authorization
# Auth_level must be one of the following:
#
#  bank  access granted to bank administrators only
#
#  acct  account-level access granted to account administrators
#     global access granted to bank administrators
#
#  user  users granted access to their own subaccount
#     account-level access granted to account administrators
#     global access granted to bank administrators
#
# Should be invoked as an ordinary subroutine call
#
# Returns 1 if authorized to perform the command, else 0
##############################################################################
sub authorize {
$auth_level = shift;
#%ARGS=%{$_[0]};

for ($auth_level) {

   /bank/ && do {
      return 1 if bank_auth();
      };

   /acct/ && do {
      return 1 if (bank_auth()||account_auth());
      };

   /user/ && do {
      return 1 if (bank_auth()||account_auth()||user_auth());
      };

   }

return 0;
}

=pod

=head1 authorize

=over 4

=item B<NAME>

authorize - check permissions

=item B<SYNOPSIS>

authorize(I<$auth_level>, I<@command>)

=item B<DESCRIPTION>

Exclusively used ithin other subroutines for command authentication
Auth_level must be one of the following:

=over 4

=item bank

 Access granted to bank administrators only

=item account

 Account-level access granted to account administrators.
 Global access granted to bank administrators.

=item user

 Users granted access to their own subaccount.
 Account-level access granted to account administrators.
 Global access granted to bank administrators

=back

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if authorized to perform the command, else 0.

=back

=cut


##############################################################################
# bank_auth AUTH=<user>
# Checks to see if the user is a bank admin
# Internal call used within authorize subroutine
# Returns 1 if user is a bank admin, else 0
##############################################################################
sub bank_auth {
#%ARGS=%{$_[0]};

return (grep(/^$ARGS{AUTH}$/,@::BANK_ADMINS) && 1 || 0);
}

=pod

=head1 bank_auth

=over 4

=item B<NAME>

bank_auth - checks whether a user is a bank admin

=item B<SYNOPSIS>

bank_auth B<AUTH>=I<user>

=item B<DESCRIPTION>

Checks whether a user is a bank administrator.
Internal call used within authorize subroutine.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if user is a bank admin, else 0.

=back

=cut

##############################################################################
# account_auth ACCOUNT=<account> AUTH=<user>
# Checks whether user is an account admin
# Internal call used within authorize subroutine
# Returns 1 if user is an account admin, else 0
##############################################################################
sub account_auth {
#%ARGS=%{$_[0]};

&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and user
name='$ARGS{AUTH}' and userstatus='admin'") || return 0;

return $QBank::DataBase::rows;
}

=pod

=head1 account_auth

=over 4

=item B<NAME>

account_auth - checks whether a user is an account administrator

=item B<SYNOPSIS>

account_auth B<ACCOUNT>=I<account> B<USER>=I<user>

=item B<DESCRIPTION>

Checks whether a user is an account administrator.
Internal call used within I<authorize> subroutine.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if user is an account admin, else 0.

=back

=cut


##############################################################################
# user_auth AUTH=<user> USER=<user>
# Checks to see if the user is running the command for himself or KITTY
# Internal call used within authorize subroutine
# Returns 1 if user is a bank admin, else 0
##############################################################################
sub user_auth {
#%ARGS=%{$_[0]};

return ("$ARGS{USER}" eq "$ARGS{AUTH}" || "$ARGS{USER}" eq "KITTY");
}

=pod

=head1 user_auth

=over 4

=item B<NAME>

user_auth - checks to see if user is running command on his own behalf or KITTY

=item B<SYNOPSIS>

user_auth B<AUTH>=I<user> B<USER>=I<user>

=item B<DESCRIPTION>

Checks whether a user information requested is for himself.
Internal call used within authorize subroutine.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if user is requesting information for himself, else 0.

=back

=cut


###############################################################
# bank_log($lvl,$text)
#
# This LOG facility functions as an encoder by logically ANDing
# the specified priority level with the following output keys:
#	0	DBGLOG	Debug (Toggled based on Value of DEBUG)
#	1	ERRLOG  Critical Failures, Fatal Errors
#	2	SECLOG	Authentication Failures
#	4	TXNLOG	Audit Trail, Transactions affecting user balances
#	8	STDERR	Warning Conditions, Syntax/Usage/Boundary Errors
#
# i.e. a priority of 24 will be logged to STDERR, the errlog,
# (and the dbglog if $DEBUG=1)
###############################################################
sub bank_log {
local($lvl,$text)=@_;

$DEBUG && do {
        open(DBGLOG,">>$LOG_DIR/$DBGLOG");
        print DBGLOG scalar(localtime),": $text\n";
        close DBGLOG;
	};
1 & $lvl && do {
	open(ERRLOG,">>$LOG_DIR/$ERRLOG");
	print ERRLOG scalar(localtime),": $text\n";
	close ERRLOG
	};
2 & $lvl && do {
	open(SECLOG,">>$LOG_DIR/$SECLOG");
	print SECLOG scalar(localtime),": $text\n";
	close SECLOG;
	};
4 & $lvl && do {
	open(TXNLOG,">>$LOG_DIR/$TXNLOG");
	print TXNLOG scalar(localtime),": $text\n";
	close TXNLOG;
	};
8 & $lvl && do { print STDERR "$text\n" };
return 1;	# Necessary for syntax/boundary checking condition statements
}


=pod

=head1 bank_log

=over 4

=item B<NAME>

bank_log

=item B<SYNOPSIS>

bank_log(I<$lvl>, I<$text>)

=item B<DESCRIPTION>

This LOG facility functions as an encoder by logically ANDing the
specivied priority level with the following output keys. 

=over 4

=item 0 => DBGLOG

Debug (Toggled based on Value of DEBUG)

=item 1 => ERRLOG

Critical Failures, Fatal Errors

=item 2 => SECLOG

Authentication Failures

=item 4 => TXNLOG

Audit Trail, Transactions affecting user balances.

=item 8 => STDERR

Warning Conditions, Syntax/Usage/Boundary Errors

=back

i.e. a priority of 24 will be logged to STDERR, the ERRLOG and the DBGLOG if $DEBUG=1 

=item B<OPTIONS>

=item B<RETURN VALUES>

=back

=cut

1;

