package QBank::ChargeRate;

#use strict;
#use vars qw($VERSION);
use Exporter;
use QBank::BankUtils;
use QBank::DataBase;
use QBank::Result;
@ISA=qw(Exporter);
@EXPORT = qw(%ARGS);

$VERSION = '3.0';

################################################################################
# QBank::ChargeRate::add [MACHINE=<machine>] [TYPE=<chargetype>] NAME=<chargename> RATE=<chargerate>
# Required Parameters: NAME, RATE
# Optional Parameters: MACHINE, TYPE
################################################################################
sub add {
my $pkg = shift;

# Authorize
unless (&authorize('bank'))
   { return &failure("$ARGS{AUTH} is not authorized to run $ARGS{COMMAND}"); }

# Usage
if ($ARGS{USAGE}||(!$ARGS{NAME}||!$ARGS{RATE}))
   { return &usage("CLASS=ChargeRate METHOD=add [MACHINE=<machine>] [TYPE=<chargetype>] NAME=<chargename> RATE=<chargerate>"); }

# Use defaults for optional parameters
$ARGS{MACHINE}="DEFAULT" unless defined $ARGS{MACHINE};
$ARGS{TYPE}="SIMPLE" unless defined $ARGS{TYPE};

# Check to make sure a duplicate entry does not exist
if (QBank::ChargeRate->exists())
   {return &failure("Chargerate MACHINE=$ARGS{MACHINE} TYPE=$ARGS{TYPE} NAME=$ARGS{NAME} already exists.");}

# Create chargerate
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('$ARGS{MACHINE}','$ARGS{TYPE}','$ARGS{NAME}',$ARGS{RATE})") || return &failure($QBank::DataBase::error);
$rows=$QBank::DataBase::rows;

# Log to txnlog
&db("insert into txnlog (txntype,machine,timestamp,authorization) values ('add_chargerate','$ARGS{MACHINE}','now','$me')") || return &failure($QBank::DataBase::error);

# Return
&bank_log(4,"@_");
return &success($rows,"Successfully created chargerate MACHINE=$ARGS{MACHINE} TYPE=$ARGS{TYPE} NAME=$ARGS{NAME} RATE=$ARGS{RATE}");
#$ARGS{STATUS}=1;
#$ARGS{MESSAGE}="Successfully created chargerate MACHINE=$ARGS{MACHINE} TYPE=$ARGS{TYPE} NAME=$ARGS{NAME} RATE=$ARGS{RATE}";
#return $rows;

}

=pod

=head1 ChargeRate::add

=over 4

=item B<NAME>

ChargeRate::add - define new chargerate

=item B<SYNOPSIS>

B<CLASS>=ChargeRate B<METHOD>=add B<MACHINE>=I<machine> [B<TYPE>=I<chargetype>]
[B<NAME>=I<chargename>] [B<RATE>=I<chargerate>]

=item B<DESCRIPTION>

Defines a new charge rate. The charge object stores machine-specific
charge rates such as processor billing multipliers, QOS charge rates, etc.

The following chargerates are predefined in a special DEFAULT machine to be
used when no other machine-specific chargerates can be located.

        MACHINE=DEFAULT TYPE=SIMPLE NAME=PROCS RATE=1
        MACHINE=DEFAULT TYPE=SIMPLE NAME=MEMORY RATE=0
        MACHINE=DEFAULT TYPE=SIMPLE NAME=DISK RATE=0
        MACHINE=DEFAULT TYPE=QOS NAME=0 RATE=1
        MACHINE=DEFAULT TYPE=QOS NAME=1 RATE=1
        MACHINE=DEFAULT TYPE=NODE NAME=DEFAULT RATE=1
        MACHINE=DEFAULT TYPE=TIME NAME=PRIME RATE=1
        MACHINE=DEFAULT TYPE=CLASS NAME=batch RATE=1
        MACHINE=DEFAULT TYPE=CLASS NAME=interactive RATE=1

=item B<OPTIONS>

MACHINE	is the machine for which the ChargeRate is to apply
	It defaults to DEFAULT if not specified

TYPE	is a type of charge factor such as QOS, NODETYPE, TIME, CLASS
	It defaults to SIMPLE if not specified

NAME	is a particular instance of the chargetype

RATE	is expressed as a positive decimal number (like 2 or 1.5)

TYPE is effectively a hash or associative array of names and rates,
i.e. %NODETYPE={ P2SC => 1, SMP => 2 } would be represented as:

        TYPE=NODETYPE NAME=P2SC RATE=1
        TYPE=NODETYPE NAME=SMP RATE=2

The SIMPLE type is a special charge factor in which each have a single rate

        TYPE=SIMPLE NAME=MEMORY RATE=1

=item B<RETURN VALUES>

Returns a success or failure message

=back

=cut


################################################################################
# ChargeRate::change [MACHINE=<machine>] [TYPE=<chargetype>] NAME=<chargename> RATE=<chargerate>
################################################################################
sub change {
my $pkg = shift;
#%ARGS=%{$_[0]};

# Authorize
unless (&authorize('bank'))
   { return &failure("$ARGS{AUTH} is not authorized to run $ARGS{COMMAND}"); }

# Usage
if ($ARGS{USAGE}||(!$ARGS{NAME}||!$ARGS{RATE}))
   { return &usage("CLASS=ChargeRate METHOD=change [MACHINE=<machine>] [TYPE=<chargetype>] NAME=<chargename> RATE=<chargerate>"); }

# Use defaults for optional parameters
$ARGS{MACHINE}="DEFAULT" unless defined $ARGS{MACHINE};
$ARGS{TYPE}="SIMPLE" unless defined $ARGS{TYPE};

# Create chargerate
&db("update chargerate set chargerate=$ARGS{RATE} where machinename='$ARGS{MACHINE}' and chargetype='$ARGS{TYPE}' and chargename='$ARGS{NAME}'") || return &failure($QBank::DataBase::error);
$rows=$QBank::DataBase::rows;

# Log to txnlog
&db("insert into txnlog (txntype,machine,timestamp,authorization) values ('change_chargerate','$ARGS{MACHINE}','now','$me')") || return &failure($QBank::DataBase::error);

# Return
#&bank_log(4,"@_");  Rendered Useless
return &success($rows,"Successfully changed $rows chargerate(s)");
}

=pod

=head1 ChargeRate::change

=over 4

=item B<NAME>

ChargeRate::change - change machine-specific billing rates

=item B<SYNOPSIS>

B<CLASS>=ChargeRate B<METHOD>=change [B<MACHINE>=I<machine>] [B<TYPE>=I<chargetype>] B<NAME>=I<chargename> B<RATE>=I<chargerate>

=item B<DESCRIPTION>

This command is used to change machine-specific billing rates
such as processor billing multipliers, QOS charge rates, etc.

=item B<OPTIONS>

MACHINE	is the machine for which the ChargeRate is to apply
	It defaults to DEFAULT if not specified

TYPE	is a type of charge factor such as QOS, NODETYPE, TIME, CLASS
	It defaults to SIMPLE if not specified
	The SIMPLE type is a special charge factor in which each have a
	single rate, i.e.  TYPE=SIMPLE NAME=MEMORY RATE=1

NAME	is a particular instance of the chargetype
	This is a required option.

RATE	is expressed as a positive decimal number (like 2 or 1.5)
	This is a required option.

=item B<RETURN VALUES>

Returns a success or failure message

=back

=cut


################################################################################
# ChargeRate::exists [MACHINE=<machine>] [TYPE=<chargetype>] NAME=<chargename>
################################################################################
sub exists {
my $pkg = shift;
# This method will accept additionally passed parameters 
# Append passed params to a local copy of ARGS to prevent side effects later on
local(%ARGS)=%ARGS;
if ($_[0]) {foreach (keys %{$_[0]}) { $ARGS{$_}=${$_[0]}{$_}; } }
#foreach (keys %ARGS) { print "during: $_ => $ARGS{$_}\n"; }

# Usage
if ($ARGS{USAGE}||!$ARGS{NAME})
   { return &usage("CLASS=ChargeRate METHOD=exists [MACHINE=<machine>] [TYPE=<chargetype>] NAME=<chargename>"); }

$ARGS{MACHINE}="DEFAULT" unless defined $ARGS{MACHINE};
$ARGS{TYPE}="SIMPLE" unless defined $ARGS{TYPE};

# Check whether entry exists
&db("select machinename from chargerate where machinename='$ARGS{MACHINE}' and chargetype='$ARGS{TYPE}' and chargename='$ARGS{NAME}'") || return &failure($QBank::DataBase::error);

return $QBank::DataBase::rows;
}

=pod

=head1 ChargeRate::exists

=over 4

=item B<NAME>

ChargeRate::exists - checks whether a chargerate exists

=item B<SYNOPSIS>

B<CLASS>=ChargeRate B<METHOD>=exists [B<MACHINE>=I<machine>] [B<TYPE>=I<chargetype>] B<NAME>=I<chargename>

=item B<DESCRIPTION>

Checks to see whether a chargerate exists

=item B<OPTIONS>

MACHINE	Specifies the MACHINE for which the chargerate would be valid.
	If not specified, DEFAULT will be used.

TYPE	Checks to see if chargerate of specified type exists, otherwise
	type SIMPLE will be used.

NAME	is the particular instance of the chargetype

=item B<RETURN VALUES>

Returns the number of matches found

=back

=cut

##############################################################
# ChargeRate::remove [MACHINE=<machine>] [TYPE=<chargetype>] [NAME=<chargename>]
##############################################################
sub remove {
my $pkg = shift;
#%ARGS=%{$_[0]};

unless (authorize('bank'))
        { return &failure("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||(!$ARGS{MACHINE}&&!$ARGS{TYPE}&&!$ARGS{NAME}))
        { return &usage("$ARGS{COMMAND} [MACHINE=<machine>] [TYPE=<chargetype>] [NAME=<chargename>]"); }

# Delete chargerate
$delete="delete from chargerate where 1=1";
if (defined $ARGS{MACHINE}) { $delete.=" and machinename='$ARGS{MACHINE}'"; }
if (defined $ARGS{TYPE}) { $delete.=" and chargetype='$ARGS{TYPE}'"; }
if (defined $ARGS{NAME}) { $delete.=" and chargename='$ARGS{NAME}'"; }
&db($delete) || return &failure($QBank::DataBase::error);
$rows = $QBank::DataBase::rows;

# Log the transaction
&db("insert into txnlog (txntype,machine,timestamp,authorization) values ('remove_chargerate','$ARGS{MACHINE}','now','$me')") || return &failure($QBank::DataBase::error);

&bank_log(4,"@_");

return &success($rows,"Successfully removed $rows chargerate(s)");
}

=pod

=head1 ChargeRate::remove

=over 4

=item B<NAME>

ChargeRate::remove - deletes a chargerate

=item B<SYNOPSIS>

B<CLASS>=ChargeRate B<METHOD>=remove B<MACHINE>=I<machine> [B<TYPE>=I<chargetype>] [B<NAME>=I<chargename>]

=item B<DESCRIPTION>

Removes the selected charge rate(s)
If MACHINE, TYPE and  NAME are all specified, at most that unique entry
will be removed. If MACHINE, TYPE and/or NAME are left unspecified, all
entries matching the specified attributes will be removed. You must specify
at least one of MACHINE, TYPE or NAME.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a success or failure message.

=back

=cut

################################################################################
# ChargeRate::get [MACHINE=<machine>] [TYPE=<chargetype>] [NAME=<chargename>] [RATE=<chargerate>]
################################################################################
sub get {
my $pkg = shift;
# This method will accept additionally passed parameters 
# Append passed params to a local copy of ARGS to prevent side effects later on
local(%ARGS)=%ARGS;
if ($_[0]) {foreach (keys %{$_[0]}) { $ARGS{$_}=${$_[0]}{$_}; } }

# Usage
if ($ARGS{USAGE})
   { return &usage("CLASS=ChargeRate METHOD=get [MACHINE=<machine>] [TYPE=<chargetype>] [NAME=<chargename>] [RATE=<chargerate>] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]"); }

if ($ARGS{CHECKPOINT}) { $X="x" } else { $X="" }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="machinename,chargetype,chargename,chargerate
" };

my $select="select $ARGS{SHOW} from ${X}chargerate where 1=1";

if ($ARGS{MACHINE}) {$select.=" and machinename='$ARGS{MACHINE}'"}
if ($ARGS{CHECKPOINT}) {$select.=" and checkpointkey=$ARGS{CHECKPOINT}"}
if ($ARGS{TYPE}) {$select.=" and chargetype='$ARGS{TYPE}'"}
if ($ARGS{NAME}) {$select.=" and chargename='$ARGS{NAME}'"}
if ($ARGS{RATE}) {$select.=" and chargerate='$ARGS{RATE}'"}

&db($select) || return &fail($QBank::DataBase::error);
return &success(join("\n",@QBank::DataBase::result));
}

=pod

=head1 ChargeRate::get

=over 4

=item B<NAME>

ChargeRate::get - display machine-specific billing rates

=item B<SYNOPSIS>

B<CLASS>=ChargeRate B<METHOD>=get [B<MACHINE>=I<machine>] [B<TYPE>=I<chargetype>] [B<NAME>=I<chargename>] [B<RATE>=I<chargerate>] [B<CHECKPOINT>=I<checkpointkey>] [B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

Dumps a line for each chargerate specified.
If MACHINE, TYPE and  NAME are all specified, at most that unique entry
will be displayed. If MACHINE, TYPE and/or NAME are left unspecified, all
entries matching the specified attributes will be displayed. You must specify
at least one of MACHINE, TYPE or NAME.

=item B<OPTIONS>

MACHINE	is the machine for which the ChargeRate is to apply

TYPE	is a type of charge factor such as QOS, NODETYPE, TIME, CLASS
	The SIMPLE type is a special charge factor in which each have a
	single rate, i.e.  TYPE=SIMPLE NAME=MEMORY RATE=1

NAME	is a particular instance of the chargetype

RATE	is expressed as a positive decimal number (like 2 or 1.5)

=item B<RETURN VALUES>

Returns a comma-delimited arrays of:
"machinename,chargetype,chargename,chargerate"
unless overridden by the B<SHOW> parameter.

=back

=cut
1;
