################################################################################
# 
#	qbank.pl
#
#	A collection of subroutines used by the allocations bank server
#
################################################################################
#                                                                              #
#                 COPYRIGHT (C) 1994, 1995, 1996, 1997, 1998                   #
#                  Pacific Northwest National Laboratory,                      #
#                       Battelle Memorial Institute.                           #
#                                                                              #
#                          All Rights Reserved                                 #
#                                                                              #
################################################################################
#                                                                              #
#                              DISCLAIMER                                      #
#                                                                              #
#   This material was prepared as an account of work sponsored by an agency    #
#   of the United States Government.  Neither the United States Government     #
#   nor the United States Department of Energy, nor Battelle, nor any of their #
#   employees, MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY LEGAL    #
#   LIABILITY OR RESPONSIBILITY FOR THE ACCURACY, COMPLETENESS, OR USEFULNESS  #
#   OF ANY INFORMATION, APPARATUS, PRODUCT, SOFTWARE, OR PROCESS DISCLOSED,    #
#   OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS.      #
#                                                                              #
################################################################################
#                                                                              #
#                              LIMITED USE                                     #
#                                                                              #
#   This software (including any documentation) is being made available to     #
#   you for your internal use only, solely for use in performance of work      #
#   directly for the U.S. Federal Government or work under contracts with      #
#   the U.S. Department of Energy or other U.S. Federal Government agencies.   #
#   This software is a version which has not yet been evaluated and cleared    #
#   for commercialization.  Adherence to this notice may be necessary for the  #
#   author, Battelle Memorial Institute, to successfully assert copyright in   #
#   and commercialize this software.  This software is not intended for        #
#   duplication or distribution to third parties without the permission of     #
#   the Manager of Software Products at Pacific Northwest National Laboratory, #
#   Richland, Washington, 99352.                                               #
#                                                                              #
################################################################################
#                                                                              #
#                              ACKNOWLEDGMENT                                  #
#                                                                              #
#   This software and its documentation were produced with Government support  #
#   under Contract Number DE-AC06-76RLO-1830 awarded by the United States      #
#   Department of Energy.  The Government retains a paid-up non-exclusive,     #
#   irrevocable worldwide license to reproduce, prepare derivative works,      #
#   perform publicly and display publicly by or for the Government, including  #
#   the right to distribute to other Government contractors.                   #
#                                                                              #
################################################################################

#use QBank;
#use QBank::Account;
#use QBank::Allocation;
#use QBank::Bank;
use QBank::ChargeRate;
#use QBank::CheckPoint;
#use QBank::DataBase;
#use QBank::Log;
#use QBank::Reservation;
#use QBank::Result;
#use QBank::SubAccount;
#use QBank::Txn;
#use QBank::User;
use Socket;

# $PREFIX is set by the program that requires this library
require "${PREFIX}/lib/qbank.ph";
require "${PREFIX}/lib/charge.pl";

$QAUTH="${PREFIX}/bin/qauth -s";
$AUTH=(getpwuid($<))[0];
$VERSION='2.10.0';

@VALIDCMDS=(
	"account_active",
	"account_admin",
	"account_balance",
	"account_exists",
	"add_account",
	"add_chargerate",
	"add_user",
	"add_subaccount",
	"bank_admin",
	"change_account",
	"change_allocation",
	"change_chargerate",
	"change_subaccount",
	"change_suballocation",
	"change_user",
	"create_tables",
	"dump_account",
	"get_accounts",
	"get_allocations",
	"get_checkpoints",
	"get_chargerates",
	"get_quotations",
	"get_reservations",
	"get_subaccounts",
	"get_suballocations",
	"get_subreservations",
	"get_txns",
	"get_users",
	"help",
	"list_accounts",
	"make_checkpoint",
	"make_deposit",
	"make_quotation",
	"make_refund",
	"make_reservation",
	"make_withdrawal",
	"make_transfer",
	"ping_bank",
	"reconcile_bank",
	"remove_account",
	"remove_allocation",
	"remove_checkpoint",
	"remove_chargerate",
	"remove_quotation",
	"remove_reservation",
	"remove_subaccount",
	"remove_user",
	"restore_checkpoint",
	"set_mask",
	"set_version",
	"show_mask",
	"sync_reservations",
	"update_allocations",
	"update_tables",
	"user_available",
	"user_balance",
	"user_enabled",
	"version"
	);


###############################################################
# version
# Prints the current version of QBank
###############################################################
sub version {

&db("select version from bank") || return &fail("version lookup failed -- run set_version: $dberror");

return &success($QUERYRESULT[0]);
}

=pod

=head1 version

=over 4

=item B<NAME>

version

=item B<SYNOPSIS>

version

=item B<DESCRIPTION>

Prints out the current version of QBank.

=item B<OPTIONS>

=item B<RETURN VALUES>

=back

=cut


##############################################################
# set_version [VERSION=<version>|CURRENT]
# Sets the QBank version number in the bank table.
# If VERSION is not specified, this subroutine will attempt to
# determine and set your version based on table configurations.
# CURRENT can be specified to force the bank version to be that of
# the distribution (useful in case of failure messages in update_tables).
# returns success or failure message
##############################################################
sub set_version {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
	{ return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
	{ return &usage("$ARGS{COMMAND} [VERSION=<version>]"); }

unless ($ARGS{VERSION} =~ /\d+\.\d+/ || $ARGS{VERSION} eq 'CURRENT')
	{ return &fail("Version format must be 'CURRENT' or of the form V.R[.M[.F]]
where V,R,M,F are whole numbers"); }

# Create Bank Version field if it does not already exist
unless (&db("select version from bank")) {
	&db("create table Bank (
		Version varchar(20)
	)") || return &fail($dberror);
	&db("insert into bank (version) values ('0.0')") || push @CREATEERRORS, $dberror;
	}


# Set the version
# If version is not specified, figure it out
unless (defined $ARGS{VERSION}) {
	if (&db("select accounttype from account")) { $ARGS{VERSION}='2.10'; }
	elsif (&db("select chargerate from chargerate")) { $ARGS{VERSION}='2.9'; }
	# Nobody should be less than 2.8
	else { $ARGS{VERSION}='2.8'; }
	}

&db("update bank set version='$ARGS{VERSION}'") || return &fail($dberror);

# Log the transaction
&bank_log(4,"@_");

return &success("Successfully set version to $ARGS{VERSION}");
}

=pod

=head1 set_version

=over 4

=item B<NAME>

set_version - sets the version

=item B<SYNOPSIS>

set_version [B<VERSION>=I<version|CURRENT>]

=item B<DESCRIPTION>

If B<VERSION> is not specified, this subroutine will attempt to
determine and set your version based on table configurations.
I<CURRENT> can be specified to force the bank version to be that of
the distribution (useful in case of failure messages in update_tables).

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating success or failure

=back

=cut


##############################################################################
# create_tables
# Returns success or failure message
##############################################################################
sub create_tables {
@CREATEERRORS=();

unless (&authorize('acct',@_))
	{ return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
	{ return &usage("$ARGS{COMMAND}"); }

# Create Bank Table and Set Current Version
&db("create table Bank (
	Version varchar(20)
)") || push @CREATEERRORS, $dberror;
&db("insert into bank (version) values ('$VERSION')") || push @CREATEERRORS, $dberror;

# Create Account Table
&db("create table Account (
  AccountName varchar(20) NOT NULL,
  AccountType varchar(20),
  AccountActive boolean,
  DefaultMachines varchar(40),
  ExchangeHost varchar(20),
  ExchangePort int,
  ExchangeAcct varchar(20),
  ExchangeUser varchar(20),
  ExchangeRate float,
  CreationDate timestamp,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index Account1 on Account (AccountName)") || push @CREATEERRORS, $dberror;

# Create SubAccount Table
&db("create table subAccount (
  AccountName   varchar(20) NOT NULL,
  UserName      varchar(20) NOT NULL,
  UserStatus    varchar(20) NOT NULL,
  DepositMask   int,
  CreationDate  timestamp,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index subAccount1 on subAccount (AccountName,UserName)") || push @CREATEERRORS, $dberror;

# Create Allocation Table
&db("create table Allocation (
  AllocKey      int         NOT NULL,
  AllocType     varchar(20) NOT NULL,
  AccountName   varchar(20) NOT NULL,
  ExpirationDate timestamp,
  ActivationDate timestamp,
  AllowMachines varchar(40),
  CreationDate  timestamp,
  Active        boolean     NOT NULL,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index Allocation1 on Allocation (AllocKey)") || push @CREATEERRORS, $dberror;

# Create SubAllocation Table
&db("create table subAllocation (
  AllocKey    int         NOT NULL,
  UserName    varchar(20) NOT NULL,
  Amount      int8 CHECK (Amount >= 0) NOT NULL,
  Overdraft   int8        DEFAULT 0,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index subAllocation1 on subAllocation (AllocKey,UserName)") || push @CREATEERRORS, $dberror;

# Create Reservation Table
&db("create table Reservation (
  JobID         varchar(40) NOT NULL,
  AccountName   varchar(20) NOT NULL,
  ExpirationDate timestamp     NOT NULL,
  Machine     varchar(40),
  CreationDate  timestamp        NOT NULL,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index Reservation1 on Reservation (JobID)") || push @CREATEERRORS, $dberror;

# Create SubReservation Table
&db("create table subReservation (
  JobID       varchar(40) NOT NULL,
  UserName    varchar(20) NOT NULL,
  Amount      int8        NOT NULL,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index subReservation1 on subReservation (JobID,UserName)") || push @CREATEERRORS, $dberror;

# Create Quotation Table
&db("create table Quotation (
  QuoteID         varchar(40) NOT NULL,
  Rate      float        NOT NULL,
  AccountName   varchar(20) NOT NULL,
  UserName   varchar(20),
  Machine     varchar(40),
  ExpirationDate timestamp     NOT NULL,
  CreationDate  timestamp        NOT NULL,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index Quotation1 on Quotation (QuoteID)") || push @CREATEERRORS, $dberror;

# Create ChargeRate Table and populate DEFAULT machine
&db("create table ChargeRate (
  MachineName varchar(20) NOT NULL,
  ChargeType varchar(20),
  ChargeName varchar(20),
  ChargeRate float,
  CheckpointKey   int
)") || push @CREATEERRORS, $dberror;
&db("create unique index ChargeRate1 on ChargeRate (MachineName,ChargeType,ChargeName)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','SIMPLE','PROCS',1)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','SIMPLE','MEMORY',0)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','SIMPLE','DISK',0)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','QOS','0',1)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','QOS','1',1)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','NODE','DEFAULT',1)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','TIME','PRIME',1)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','CLASS','batch',1)") || push @CREATEERRORS, $dberror;
&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','CLASS','interactive',1)") || push @CREATEERRORS, $dberror;

# Create TxnLog Table
&db("create table txnLog (
  TXNType     varchar(24) NOT NULL,
  TimeStamp   timestamp NOT NULL,
  AccountName varchar(20),
  UserName    varchar(20),
  ExpirationDate timestamp,
  ActivationDate timestamp,
  Key       varchar(40),
  Authorization varchar(20),
  Processors  int,
  Nodes       int,
  WallClock   int,
  Type     varchar(20),
  Amount      int8,
  Reason      varchar(40),
  Machine     varchar(20),
  Memory      int,
  Disk        int,
  Class       varchar(20),
  NodeType    varchar(10),
  QOS         varchar(10),
  Active      boolean
)") || push @CREATEERRORS, $dberror;

# Create UserInfo Table
&db("create table userinfo (
  UserName      varchar(20) NOT NULL,
  DefaultAccount   varchar(20),
  RealName      varchar(40),
  PhoneNumber   varchar(40),
  EMail         varchar(40),
  Description   varchar(40)
)") || push @CREATEERRORS, $dberror;
&db("create unique index userinfo1 on userinfo (UserName)") || push @CREATEERRORS, $dberror;

# Create CheckPoint Tables
&db("create table CheckPoint (
  CheckpointKey int NOT NULL,
  TimeStamp   timestamp NOT NULL,
  Reason      varchar(40)
)") || push @CREATEERRORS, $dberror;
&db("create unique index CheckPoint1 on CheckPoint (CheckpointKey)") || push @CREATEERRORS, $dberror;

# Create xAccount Table
&db("create table xAccount (
  AccountName varchar(20) NOT NULL,
  AccountType varchar(20),
  AccountActive  boolean,
  DefaultMachines varchar(40),
  ExchangeHost varchar(20),
  ExchangePort int,
  ExchangeAccount varchar(20),
  ExchangeUser varchar(20),
  ExchangeRate float,
  CreationDate timestamp,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index xAccount1 on xAccount (AccountName,CheckpointKey)") || push @CREATEERRORS, $dberror;

# Create xsubAccount Table
&db("create table xsubAccount (
  AccountName   varchar(20) NOT NULL,
  UserName      varchar(20) NOT NULL,
  UserStatus    varchar(20) NOT NULL,
  DepositMask   int,
  CreationDate  timestamp,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index xsubAccount1 on xsubAccount (AccountName,UserName,CheckpointKey)") || push @CREATEERRORS, $dberror;

# Create xAllocation Table
&db("create table xAllocation (
  AllocKey      int         NOT NULL,
  AllocType     varchar(20) NOT NULL,
  AccountName   varchar(20) NOT NULL,
  ExpirationDate timestamp,
  ActivationDate timestamp,
  AllowMachines varchar(40),
  CreationDate  timestamp,
  Active        boolean     NOT NULL,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index xAllocation1 on xAllocation (AllocKey,CheckpointKey)") || push @CREATEERRORS, $dberror;

# Create xsubAllocation Table
&db("create table xsubAllocation (
  AllocKey    int         NOT NULL,
  UserName    varchar(20) NOT NULL,
  Amount      int8 CHECK (Amount >= 0) NOT NULL,
  Overdraft   int8        DEFAULT 0,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index xsubAllocation1 on xsubAllocation (AllocKey,UserName,CheckpointKey)") || push @CREATEERRORS, $dberror;

# Create xReservation Table
&db("create table xReservation (
  JobID         varchar(20) NOT NULL,
  AccountName   varchar(20) NOT NULL,
  ExpirationDate timestamp     NOT NULL,
  Machine     varchar(40),
  CreationDate  timestamp        NOT NULL,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index xReservation1 on xReservation (JobID,CheckpointKey)") || push @CREATEERRORS, $dberror;

# Create xsubReservation Table
&db("create table xsubReservation (
  JobID       varchar(40) NOT NULL,
  UserName    varchar(20) NOT NULL,
  Amount      int8        NOT NULL,
  CheckpointKey int
)") || push @CREATEERRORS, $dberror;
&db("create unique index xsubReservation1 on xsubReservation (JobID,UserName,CheckpointKey)") || push @CREATEERRORS, $dberror;

# Create xChargeRate Table and populate DEFAULT machine
&db("create table xChargeRate (
  MachineName varchar(20) NOT NULL,
  ChargeType varchar(20),
  ChargeName varchar(20),
  ChargeRate float,
  CheckpointKey   int
)") || push @CREATEERRORS, $dberror;
&db("create unique index xChargeRate1 on xChargeRate (MachineName,ChargeType,ChargeName)") || push @CREATEERRORS, $dberror;

if (@CREATEERRORS) { return &fail(join('\n',@CREATEERRORS)); }
else { return &success("Successfully created database tables"); }
}

=pod

=head1 create_tables

=over 4

=item B<NAME>

create_tables - set up database tables

=item B<SYNOPSIS>

create_tables

=back

=cut


##############################################################################
# update_tables
# Returns success or failure message
##############################################################################
sub update_tables {
@UPDATEERRORS=();

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
	{ return &usage("$ARGS{COMMAND}"); }

# Determine current QBank version
&db("select version from bank") || return &fail("Unable to determine version. Run set_version first.");
$DBVERSION=$QUERYRESULT[0];

# Nobody should be less than 2.8.0

# Update to 2.9 if necessary
if (vlt($DBVERSION,'2.9')) {

	# Create ChargeRate Table and populate DEFAULT machine
	&db("create table ChargeRate (
	  MachineName varchar(20) NOT NULL,
	  ChargeType varchar(20),
	  ChargeName varchar(20),
	  ChargeRate float,
	  CheckpointKey   int
	)") || push @UPDATEERRORS, $dberror;
	&db("create unique index ChargeRate1 on ChargeRate (MachineName,ChargeType,ChargeName)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','SIMPLE','PROCS',1)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','SIMPLE','MEMORY',0)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','SIMPLE','DISK',0)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','QOS','0',1)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','QOS','1',1)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','NODE','DEFAULT',1)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','TIME','PRIME',1)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','CLASS','batch',1)") || push @UPDATEERRORS, $dberror;
	&db("insert into chargerate (machinename,chargetype,chargename,chargerate) values ('DEFAULT','CLASS','interactive',1)") || push @UPDATEERRORS, $dberror;

	# Create xChargeRate Table and populate DEFAULT machine
	&db("create table xChargeRate (
	  MachineName varchar(20) NOT NULL,
	  ChargeType varchar(20),
	  ChargeName varchar(20),
	  ChargeRate float,
	  CheckpointKey   int
	)") || push @UPDATEERRORS, $dberror;
	&db("create unique index xChargeRate1 on xChargeRate (MachineName,ChargeType,ChargeName)") || push @UPDATEERRORS, $dberror;

	# Add Memory, Disk and Class fields to txnlog
	&db("alter table txnlog add column Memory int") || push @UPDATEERRORS, $dberror;
	&db("alter table txnlog add column Disk int") || push @UPDATEERRORS, $dberror;
	&db("alter table txnlog add column Class varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table txnlog add column Nodes int") || push @UPDATEERRORS, $dberror;

	# Bail out if error detected, otherwise update database version
	if (@UPDATEERRORS) {
		return &fail(join('\n',"The following errors were detected while updating your database tables to 2.9:",@UPDATEERRORS));
		}
	else {
		&db("update bank set version='2.9'") || return &fail($dberror);
		$DBVERSION='2.9';
		}
	}

# Update to 2.10 if necessary
if (vlt($DBVERSION,'2.10')) {

	# Add Exchange and Traceback mechanisms to account
	&db("alter table account add column AccountType varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table account add column ExchangeHost varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table account add column ExchangePort int") || push @UPDATEERRORS, $dberror;
	&db("alter table account add column ExchangeAccount varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table account add column ExchangeUser varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table account add column ExchangeRate float") || push @UPDATEERRORS, $dberror;
	&db("update account set accounttype='local'") || push @UPDATEERRORS, $dberror;
	&db("update account set exchangerate=1") || push @UPDATEERRORS, $dberror;

	# Add Exchange and Traceback mechanisms to xaccount
	&db("alter table xaccount add column AccountType varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table xaccount add column ExchangeHost varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table xaccount add column ExchangePort int") || push @UPDATEERRORS, $dberror;
	&db("alter table xaccount add column ExchangeAccount varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table xaccount add column ExchangeUser varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table xaccount add column ExchangeRate float") || push @UPDATEERRORS, $dberror;

	# Change jobtype to type in txnlog
	&db("alter table txnlog add column type varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("update txnlog set type=jobtype") || push @UPDATEERRORS, $dberror;

	# Create Quotation Table
	&db("create table Quotation (
  		QuoteID         varchar(40) NOT NULL,
  		Rate      float        NOT NULL,
  		AccountName   varchar(20) NOT NULL,
  		Machine     varchar(40),
  		ExpirationDate timestamp     NOT NULL,
  		CreationDate  timestamp        NOT NULL,
  		CheckpointKey int
	)") || push @CREATEERRORS, $dberror;
	&db("create unique index Quotation1 on Quotation (QuoteID)") || push @CREATEERRORS, $dberror;

	# Add AllocType field to Allocation
	&db("alter table allocation add column alloctype varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("alter table xallocation add column alloctype varchar(20)") || push @UPDATEERRORS, $dberror;
	&db("update allocation set alloctype='debit'") || push @UPDATEERRORS, $dberror;
	&db("update xallocation set alloctype='debit'") || push @UPDATEERRORS, $dberror;

	# Add overdraft field to suballocation
	&db("alter table suballocation add column Overdraft int8 DEFAULT 0") || push @UPDATEERRORS, $dberror;
	&db("alter table xsuballocation add column Overdraft int8 DEFAULT 0") || push @UPDATEERRORS, $dberror;
	&db("update suballocation set overdraft=0") || push @UPDATEERRORS, $dberror;
	&db("update xsuballocation set overdraft=0") || push @UPDATEERRORS, $dberror;

	# Add reserved subaccount ANY to all accounts
	&db("select accountname from account") || push @UPDATEERRORS, $dberror;
	foreach $account (@QUERYRESULT) {
		&db("insert into subaccount (accountname,username,userstatus,depositmask,creationdate) values ('$acccount','ANY','reserved','0','now')") || push @UPDATEERRORS, $dberror;
		}

	# Bank Version Field created in set_version subroutine in 2.10

	# Bail out if error detected, otherwise update database version
	if (@UPDATEERRORS) {
		return &fail(join('\n',"The following errors were detected while updating your database tables to 2.10:",@UPDATEERRORS));
		}
	else {
		&db("update bank set version='2.10'") || return &fail($dberror);
		$DBVERSION='2.10';
		}
	}

	# Update Bank Version field to current version if no failures detected
	&db("update bank set version='$VERSION'") || return &fail($dberror);

	return &success("Successfully updated database tables to support QBank $VERSION");
}

=pod 

=head1 update_tables

=over 4

=item B<NAME>

update_tables - update previous release db tables to current style.

=item B<SYNOPSIS>

update_tables

=back 

=cut


##############################################################################
# account_exists ACCOUNT=<account>
# Returns 1 if account exists, else 0
##############################################################################
sub account_exists {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
	{ return &usage("$ARGS{COMMAND} ACCOUNT=<account>"); }

&db("select accountname from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);

return &success($ROWS);
}

=pod

=head1 account_exists

=over 4

=item B<NAME>

account_exists - check if account exists

=item B<SYNOPSIS>

account_exists B<ACCOUNT>=I<account>

=item B<DESCRIPTION>

Used in jobfilters or other subroutines to verify an account exists

=item B<RETURN VALUE>

Returns 1 if account exists.  Otherwise returns 0.

=back

=cut


##############################################################################
# list_accounts
# Preserved to provide a list of accounts for parsing 
# without the authorization constraints built into get_accounts
# Returns a list of accounts
##############################################################################
sub list_accounts {
local(%ARGS);
foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

&db("select accountname from account") || return &fail($dberror);

return &success(join("\n",sort @QUERYRESULT));
}

=pod

=head1 list_accounts

=over 4

=item B<NAME>

list_accounts - list accounts

=item B<SYNOPSIS>

list_accounts

=item B<DESCRIPTION>

Preserved to provide a list of accounts for parsing without the
authorization constraints built into B<get_accounts> 

=item B<RETURN VALUES>

Returns a space separated list of accounts

=back 

=cut


##############################################################################
# account_active ACCOUNT=<account>
# Returns 1 if active, 0 if inactive, NULL if account does not exist
##############################################################################
sub account_active {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
	{ return &usage("$ARGS{COMMAND} ACCOUNT=<account>"); }

&db("select accountactive from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
($active)=@QUERYRESULT;

return &success($active);
}

=pod

=head1 account_active

=over 4

=item B<NAME>

account_active - check if account is active

=item B<SYNOPSIS>

account_active B<ACCOUNT>=I<account>

=item B<DESCRIPTION>

Determines if the given I<account> is accive or not.

=item B<RETURN VALUE>

Returns 1 if active, 0 if inactive or B<NULL> if account doesn't exist.

=back

=cut


##############################################################
# add_account ACCOUNT=<account> [ACTIVE=1|0] [MACHINES=<mach1:mach2:...>|ANY]
# [TYPE=<accounttype>] [EXCHANGEHOST=<hostname>] [EXCHANGEPORT=<port>]
# [EXCHANGEACCT=<account>]  [EXCHANGEUSER=<user>] [EXCHANGERATE=<rate>]
# Account is made active by default unless specified to be inactive.
# Default allowmachines is ANY unless otherwise specified.
# Account type may be one of local, meta, forwarding or seesaw
# and defaults to local.
# A local account makes debits only to the account itself for local users.
# A meta account is used by offsite users and debits the traceback account
# (if passed within the arguments of the transaction) then debits itself.
# A forwarding account debits the traceback account (if passed), then makes a
# debit to the exchange account, and upon success, debits itself.
# A seesaw account debits the traceback account (if passed), makes a debit to
# the exchange account (taking the exchange rate into account) and finally
# debits itself (doubly if the exchange credit fails).
# Bear in mind that any of these accounts may consist of debit or credit
# based allocations.
# Exchange rate is a positive real number and defaults to 1.
# Special subaccounts KITTY, RESERVE and ANY are created automatically.
# Returns success or failure message
##############################################################
sub add_account {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
	{ return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
	{ return &usage("$ARGS{COMMAND} ACCOUNT=<account> [ACTIVE=1|0] [MACHINES=<mach1:mach2:...>|ANY] [TYPE=<accounttype>] [EXCHANGEHOST=<hostname>] [EXCHANGEPORT=<port>] [EXCHANGEACCT=<account>]  [EXCHANGEUSER=<user>] [EXCHANGERATE=<rate>]"); }

if (&result(&account_exists("ACCOUNT=$ARGS{ACCOUNT}")))
	{ return &deny("Account $ARGS{ACCOUNT} already exists"); }

# Create account

# Account is made active by default unless specified to be inactive
if ($ARGS{ACTIVE} eq "0") { $active='f' } else { $active='t' }

# Exchange rate defaults to 1 unless specified to be otherwise
if ($ARGS{EXCHANGERATE}) { $xrate=$ARGS{EXCHANGERATE}; } else { $xrate=1;}

# Account Type defaults to "local" unless otherwise specified
if (! defined $ARGS{TYPE}) { $accttype="local";}
elsif ($ARGS{TYPE} =~ /local|meta|forwarding|seesaw/) { $accttype=$ARGS{TYPE}; }
else { return &deny("Invalid TYPE -- must be one of local, meta, forwarding or seesaw"); }

# Make sure Exchange Port is a valid number
$ARGS{EXCHANGEPORT}*=1;

# Default allowmachines is ANY unless otherwise specified
if ($ARGS{MACHINES}) {
	unless ($ARGS{MACHINES} =~ /^[\w:]+$/)
        	{ return &deny("Machine Argument must be a colon-delimited list of alphanumeric [a-zA-Z_0-9]\nnames or the special keyword ANY."); }
	$machines=$ARGS{MACHINES};
		}
else {	$machines='ANY'; }

&db("insert into account (accountname,accounttype,accountactive,creationdate,defaultmachines,exchangehost,exchangeport,exchangeacct,exchangeuser,exchangerate) values ('$ARGS{ACCOUNT}','$accttype','$active','now',':${machines}:','$ARGS{EXCHANGEHOST}',$ARGS{EXCHANGEPORT},'$ARGS{EXCHANGEACCT}','$ARGS{EXCHANGEUSER}',$xrate)") || return &fail($dberror);

# Create reserved subaccounts KITTY and RESERVE
&db("insert into subaccount (accountname,username,userstatus,depositmask,creationdate) values ('$ARGS{ACCOUNT}','KITTY','reserved','100','now')") || return &fail($dberror);
&db("insert into subaccount (accountname,username,userstatus,depositmask,creationdate) values ('$ARGS{ACCOUNT}','RESERVE','reserved','0','now')") || return &fail($dberror);
&db("insert into subaccount (accountname,username,userstatus,depositmask,creationdate) values ('$ARGS{ACCOUNT}','ANY','reserved','0','now')") || return &fail($dberror);

# Log to txnlog
&db("insert into txnlog (txntype,accountname,timestamp,authorization,active,machine,type) values ('add_account','$ARGS{ACCOUNT}','now','$me','$active',':${machines}:','$accttype')") || return &fail($dberror);

&bank_log(4,"@_");
return &success("Successfully created account $ARGS{ACCOUNT}");
}

=pod

=head1 add_account

=over 4

=item B<NAME>

add_account - create new account

=item B<SYNOPSIS>

add_account B<ACCOUNT>=I<account> [B<ACTIVE>=I<1|0>]
[B<MACHINES>=I<mach1:mach2:...|ANY>] [B<TYPE>=I<accounttype>]
[B<EXCHANGEHOST>=I<hostname>] [B<EXCHANGEPORT>=I<port>]
[B<EXCHANGEACCT>=I<account>] [B<EXCHANGEUSER>=I<user>]
[B<EXCHANGERATE>=I<rate>]

=item B<DESCRIPTION>

Creates a new account with the specified name. The new account is
created with three reserved subaccounts: B<KITTY>, B<ANY>, and B<RESERVE>.

The KITTY subaccount is a common pool from which all users within the account
may withdraw equally. Withdrawals will only be taken from the KITTY after the
user's own subaccount funds are depleted.

The ANY subaccount is a pool from which any requestor may draw, regardless
of whether they have a user subaccount or not. This subaccount is to support
environments where the requesting user may not be known beforehand such as
in a metascheduling context.

The RESERVE subaccount is a pool which cannot be withdrawn from. This
subaccount can be used by administrators to hold funds in reserve, or
preallocate them but not activate them. The account administrator can readily
transfer funds to and from the RESERVE, KITTY and user subaccounts.
Funds may be reserved and periodically dispensed as a way of ensuring
that funds endure throughout a project cycle.

The default depositmask is B<KITTY=100>, and the account is active by default.
The B<AllowMachines> field is set to I<ANY> by default
B<AccountType> may be one of local, meta, forwarding or seesaw
and defaults to local.
B<ExchangeRate> is a positive real number and defaults to 1.

=item B<OPTIONS>

B<TYPE> is the account type and may be one of local, meta, forwarding
or seesaw. If left blank, it defaults to a standard local account.

A local account makes debits only to the account itself for local users.
A meta account is used by offsite users and debits the traceback account
(if passed within the arguments of the transaction) then debits itself.
A forwarding account debits the traceback account (if passed), then makes a
debit to the exchange account, and upon success, debits itself.
A seesaw account debits the traceback account (if passed), makes a debit to
the exchange account (taking the exchange rate into account) and finally
debits itself (doubly if the exchange credit fails).
Bear in mind that any of these accounts may consist of debit or credit
based allocations.

The B<EXCHANGEHOST> is the hostname of the QBank server which must be contacted
to perform remote debits or credits for seesaw and forwarding accounts.

The B<EXCHANGEPORT> is the port that the remote QBank server is listening on.

The B<EXCHANGEACCT> is used by the forwarding and seesaw accounts to make a
debit or credit corresponding to a local debit.

The B<EXCHANGEUSER> is used by the forwarding and seesaw accounts to make a
debit or credit to the appropriate foreign subaccount. For seesaw accounts,
if this value is left NULL, then the remote depositmaske will be used.

The B<EXCHANGERATE> is used to convert the cycle units of this site to that of
the other in the case of a seesaw exchange or a traceback debit.
It should be a positive real number between 0 and 1 and defaults to 1.
A debit of N local units will translate to a debit or credit to the
remote account of N*EXCHANGERATE units.

=item B<RETURN VALUES>

Returns a success or failure message

=back

=cut


##############################################################
# remove_account ACCOUNT=<account>
# Returns success or failure message
##############################################################
sub remove_account {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
	{ return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
	{ return &usage("$ARGS{COMMAND} ACCOUNT=<account>"); }

# Calls remove_allocation to remove and log allocations and suballocations
&db("select allockey from allocation where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
foreach $allockey (@ALLOCATIONS=@QUERYRESULT) {
	&remove_allocation("KEY=$allockey","REASON=\"Removal of Account $ARGS{ACCOUNT}\"","AUTH=$me");
	}

# Delete associated reservations and subreservations
&db("select jobid from reservation where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
foreach $jobid (@RESERVATIONS=@QUERYRESULT) {
	&db("delete from subreservation where jobid='$jobid'") ||return &fail($dberror);
	&db("delete from reservation where jobid='$jobid'") ||return &fail($dberror);
	}

# Delete associated defaultaccount assignments
&db("update userinfo set defaultaccount='' where defaultaccount='$ARGS{ACCOUNT}'") || return &fail($dberror);

# Delete account and associated subaccounts
&db("delete from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$rows = $ROWS;
&db("delete from subaccount where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);

# Log the transaction - amount is total credits deleted by account removal
&db("insert into txnlog (txntype,accountname,timestamp,authorization) values ('remove_account','$ARGS{ACCOUNT}','now','$me')") || return &fail($dberror);

&bank_log(4,"@_");
if ($rows == 1) { return &success("Successfully removed account $ARGS{ACCOUNT}") }
else	{ return &success("Account $ARGS{ACCOUNT} does not exist") }
}

=pod

=head1 remove_account

=over 4

=item B<NAME>

remove_account - removes an account

=item B<SYNOPSIS>

remove_account B<ACCOUNT>=I<account>

=item B<DESCRIPTION>

Deletes the account, as well as all of the associated subaccounts,
allocations and reservations. Calls B<remove_allocation> and passes in a
reason.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a success or failure message.

=back

=cut


##############################################################
# change_account ACCOUNT=<accountname> [ACTIVE=1|0]
# [MACHINES=<mach1:mach2:...>|ANY] [TYPE=<accounttype>]
# [EXCHANGEHOST=<hostname>] [EXCHANGEPORT=<port>]
# [EXCHANGEACCT=<account>] [EXCHANGEUSER=<user>] [EXCHANGERATE=<rate>]
# This command can be used to activate/deactivate an account or change other
# account attributes such as the machine list, exchange rate, etc.
# MACHINES is specified as a colon-delimited list of alphanumeric machine names
# or the special keyword ANY to grant debit access from all machines.
# MACHINES represents the default machinelist that will be associated with
# new allocations if allowmachines is not explicitly specified. It is does
# not change the current machines allowed by the allocations in the account.
# TYPE may be one of local, meta, forwarding or seesaw and defaults to local
# if left blank.
# Exchange rate must be expressed as a positive real number.
# Returns success or failure message
##############################################################
sub change_account {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
$changes=0;

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<accountname> [ACTIVE=1|0] [MACHINES=<mach1:mach2:...>|ANY] [TYPE=<accounttype>] [EXCHANGEHOST=<hostname>] [EXCHANGEPORT=<port>] [EXCHANGEACCT=<account>] [EXCHANGEUSER=<user>] [EXCHANGERATE=<rate>]"); }

if ($ARGS{MACHINES}) {
	unless ($ARGS{MACHINES} =~ /^[\w:]+$/)
        	{ return &deny("Machine Argument must be a colon-delimited list of alphanumeric [a-zA-Z_0-9]\nnames or the special keyword ANY."); }
	&db("update account set defaultmachines=':$ARGS{MACHINES}:' where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$changes+=1;
	}

# Verify account exists
unless (&result(&account_exists("ACCOUNT=$ARGS{ACCOUNT}")))
        { return &deny("$ARGS{ACCOUNT} is not a valid account"); }

if ($ARGS{ACTIVE} eq "1") {
&db("update account set accountactive='t' where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$active='t';
$changes+=1;
	}

elsif ($ARGS{ACTIVE} eq "0") {
&db("update account set accountactive='f' where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$active='f';
$changes+=1;
	}

else { &db("select accountactive from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
($active)=@QUERYRESULT;
	}

if ($ARGS{TYPE}) {
	unless ($ARGS{TYPE} =~ /local|meta|seesaw|forwarding/)
        	{ return &deny("Account Type must be one of local, meta or seesaw."); }
	&db("update account set accounttype='$ARGS{TYPE}' where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$changes+=1;
	}

if ($ARGS{EXCHANGEHOST}) {
	&db("update account set exchangehost='$ARGS{EXCHANGEHOST}' where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$changes+=1;
	}

if ($ARGS{EXCHANGEPORT}) {
	&db("update account set exchangeport=$ARGS{EXCHANGEPORT} where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$changes+=1;
	}

if ($ARGS{EXCHANGEACCT}) {
	&db("update account set exchangeacct='$ARGS{EXCHANGEACCT}' where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$changes+=1;
	}

# EXCHANGEUSER needs to be able to be undefined (so will use depositmask)
if (defined $ARGS{EXCHANGEUSER}) {
	&db("update account set exchangeuser='$ARGS{EXCHANGEUSER}' where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$changes+=1;
	}

if ($ARGS{EXCHANGERATE} ne "" && ARGS{EXCHANGERATE} >= 0) {
	&db("update account set exchangerate=$ARGS{EXCHANGERATE} where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
$changes+=1;
	}

&db("insert into txnlog (txntype,accountname,timestamp,authorization,active,machine) values ('change_account','$ARGS{ACCOUNT}','now','$me','$active',':$ARGS{MACHINES}:')") || return &fail($dberror);

&bank_log(4,"@_");
return &success("Successfully made $changes changes to account $ARGS{ACCOUNT}");
}

=pod

=head1 change_account

=over 4

=item B<NAME>

change_account - change account status or machine list

=item B<SYNOPSIS>

change_account B<ACCOUNT>=I<account> [B<ACTIVE>=I<1|0>]
[B<MACHINES>=I<mach1:mach2:...:|:ANY>] [B<TYPE>=I<accounttype>]
[B<EXCHANGEHOST>=I<hostname>] [B<EXCHANGEPORT>=I<port>]
[B<EXCHANGEACCT>=I<account>] [B<EXCHANGEUSER>=I<user>]
[B<EXCHANGERATE>=I<rate>]

=item B<DESCRIPTION>

This command can be used to activate/deactivate an account or change
the list of machines from which debits may be made.

Activating an account makes it available for reservations and
withdrawals so that users may run jobs against the
account. Deactivation prevents make_reservation and make_withdrawal
from running. Deposits, transfers, queries can still be performed on
the account.

Setting the allowed machines list defines from what systems jobs may
charge against the account. B<MACHINES> is specified as a colon-separated
list of machines or the special keyword I<:ANY:> to grant debit access
from all machines.

B<MACHINES> represents the default machinelist that will be associated
with new allocations if allowmachines is not explicitly specified. It
does not change the current machines allowed by the allocations in the
account.

=item B<OPTIONS>

B<TYPE> is the account type and may be one of local, meta, forwarding
or seesaw. If left blank, it defaults to a standard local account.

A local account makes debits only to the account itself for local users.
A meta account is used by offsite users and debits the traceback account
(if passed within the arguments of the transaction) then debits itself.
A forwarding account debits the traceback account (if passed), then makes a
debit to the exchange account, and upon success, debits itself.
A seesaw account debits the traceback account (if passed), makes a credit to
the exchange account (taking the exchange rate into account) and finally
debits itself (doubly if the exchange credit fails).
Bear in mind that any of these accounts may consist of debit or credit
based allocations.

The B<EXCHANGEHOST> is the hostname of the QBank server which must be contacted
to perform remote debits or credits for seesaw and forwarding accounts.

The B<EXCHANGEPORT> is the port that the remote QBank server is listening on.

The B<EXCHANGEACCT> is used by the forwarding and seesaw accounts to make a
debit or credit corresponding to a local debit.

The B<EXCHANGEUSER> is used by the forwarding and seesaw accounts to make a
debit or credit to the appropriate foreign subaccount. For seesaw accounts,
if this value is left NULL, then the remote depositmaske will be used.

The B<EXCHANGERATE> is used to convert the cycle units of this site to that of
the other in the case of a seesaw exchange or a traceback debit.
It should be a positive real number between 0 and 1 and defaults to 1.
A debit of N local units will translate to a debit or credit to the
remote account of N*EXCHANGERATE units.

=item B<RETURN VALUES>

Returns success or failure message.

=back

=cut


################################################################################
# dump_account ACCOUNT=<account>
# Primarily used for debugging purposes
################################################################################
sub dump_account {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local (@dump=("\n"));

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<account>"); }

unless (&result(&account_exists("ACCOUNT=$ARGS{ACCOUNT}")))
        { return &deny("$ARGS{ACCOUNT} is not a valid account"); }

&db("select * from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
($acct)=@QUERYRESULT;
@accountinfo=split(/\|/,$acct);
$headers=$sth->{NAME};
foreach $field (@$headers) {
        push(@dump,sprintf "%-15s %s\n","$field:",shift @accountinfo);
        }

push(@dump,"\nsubaccounts:\n");
&db("select * from subaccount where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
foreach $subacct (@SUBACCOUNTS=@QUERYRESULT) {
        @subacctinfo=split(/\|/,$subacct);
        $headers=$sth->{NAME};
        foreach $field (@$headers) {
                push(@dump,sprintf "\t%-15s %s\n","$field:",shift @subacctinfo);
                }
        push(@dump,"\n");
        }

push(@dump,"\nallocations:\n");
&db("select * from allocation where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
@ALLOCATIONS=@QUERYRESULT;
$allocheaders=$sth->{NAME};
foreach $allocation (@ALLOCATIONS) {
        ($allockey,$deposit,$expires)=@allocinfo=split(/\|/,$allocation);
        foreach $allocfield (@$allocheaders) {
                push(@dump,sprintf "\t%-15s %s\n","$allocfield:",shift @allocinfo);
                }
        push(@dump,"\tsuballocations:\n");
        &db("select * from suballocation where allockey=$allockey") || return &fail($dberror);
        foreach $suballocation (@SUBALLOCATIONS=@QUERYRESULT) {
                @suballocinfo=split(/\|/,$suballocation);
                $suballocheaders=$sth->{NAME};
                foreach $suballocfield (@$suballocheaders) {
                        push(@dump,sprintf "\t\t%-15s %s\n","$suballocfield:",shift @suballocinfo);
                        }
                }
        push(@dump,"\n");
        }

push(@dump,"\nreservations:\n");
&db("select * from reservation where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
@RESERVATIONS=@QUERYRESULT;
foreach $reservation (@RESERVATIONS) {
        ($jobid,$expires)=@resinfo=split(/\|/,$reservation);
        $resheaders=$sth->{NAME};
        foreach $resfield (@$resheaders) {
                push(@dump,sprintf "\t%-15s %s\n","$resfield:",shift @resinfo);
                }
        push(@dump,"\tsubreservations:\n");
        &db("select * from subreservation where jobid='$jobid'") || return &fail($dberror);
        foreach $subreservation (@SUBRESERVATION=@QUERYRESULT) {
                @subresinfo=split(/\|/,$subreservation);
                $subresheaders=$sth->{NAME};
                foreach $subresfield (@$subresheaders) {
                        push(@dump,sprintf "\t\t%-15s %s\n","$subresfield:",shift @subresinfo);
                        }
                }
        push(@dump,"\n");
        }

push(@dump,"\nquotations:\n");
&db("select * from quotations where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
foreach $quote (@SUBACCOUNTS=@QUERYRESULT) {
        @quoteinfo=split(/\|/,$quote);
        $headers=$sth->{NAME};
        foreach $field (@$headers) {
                push(@dump,sprintf "\t%-15s %s\n","$field:",shift @quoteinfo);
                }
        push(@dump,"\n");
        }

return &success("@dump");
}

=pod

=head1 dump_account

=over 4 

=item B<NAME>

dump_account - 

=item B<SYNOPSIS>

dump_account  B<ACCOUNT>=I<account>

=item B<DESCRIPTION>

=item B<OPTIONS>

=item B<RETURN VALUES>

=back

=cut


################################################################################
# get_accounts [ACCOUNT=<account>] [ACTIVE=1|0] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]
# Returns an array of "accountname,accountactive,defaultmachines,creationdate,accounttype,exchangehost,exchangeport,exchangeacct,exchangeuser,exchangerate"
################################################################################
sub get_accounts {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]"); }

if ($ARGS{CHECKPOINT}) { $X="x" } else { $X="" }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="accountname,accountactive,defaultmachines,creationdate,accounttype,exchangehost,exchangeport,exchangeacct,exchangeuser,exchangerate" };

$select="select $ARGS{SHOW} from ${X}account where 1=1";

if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{CHECKPOINT}) {$select.=" and checkpointkey=$ARGS{CHECKPOINT}"}
if ($ARGS{ACTIVE} eq "1") {$select.=" and accountactive='t'"}
if ($ARGS{ACTIVE} eq "0") {$select.=" and accountactive='f'"}

&db($select) || return &fail($dberror);
return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_accounts

=over 4

=item B<NAME>

get_accounts - display account information.

=item B<SYNOPSIS>

get_accounts [B<ACCOUNT>=I<account>] [B<ACTIVE>=I<1|0>] [B<CHECKPOINT>=I<ckptkey>]
[B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

Dumps a line for each account specified. One can selectively display
active or inactive accounts or those with a specified machine
list. Primarily used within administrative tools to provide account
information Information accessible depends on one's administrative
priveledges. Bank admins can access global subaccount information,
account admins can access subaccount information for their
administrative accounts, and users can access subaccount information
about the subaccounts to which they pertain.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns pipe-delimited arrays of:
"accountname, accountactive, allowmachines, creationdate, accounttype,
exchangehost, exchangeport, exchangeacct, exchangeuser, exchangerate"
unless overridden by the B<SHOW> parameter.

=back

=cut


################################################################################
# get_subaccounts [ACCOUNT=<account>] [USER=<user>] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]
# Returns an array of "accountname,username,userstatus,depositmask"
################################################################################
sub get_subaccounts {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [USER=<user>] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]"); }

if ($ARGS{CHECKPOINT}) { $X="x" } else { $X="" }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="accountname,username,userstatus,depositmask" };

$select="select $ARGS{SHOW} from ${X}subaccount where 1=1";

if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{USER}) {$select.=" and username='$ARGS{USER}'"}
if ($ARGS{CHECKPOINT}) {$select.=" and checkpointkey=$ARGS{CHECKPOINT}"}

&db($select) || return &fail($dberror);
return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_subaccounts

=over 4

=item B<NAME>

get_subaccounts - display subaccount information

=item B<SYNOPSIS>

get_subaccounts [B<ACCOUNT>=I<account>] [B<USER>=I<user>]
[B<CHECKPOINT>=I<ckptkey>] [B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

Dumps a line for each subaccount specified Primarily used within
administrative tools to provide account information Information
accessible depends on one's administrative priveledges. Bank admins
can access global subaccount information, account admins can access
subaccount information for their administrative accounts, and users
can access subaccount information about the subaccounts to which they
pertain.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns pipe-delimited arrays of "accountname, username, userstatus,
depositmask" unless overridden by the B<SHOW> parameter.

=back

=cut


################################################################################
# get_suballocations [ACCOUNT=<account>] [USER=<user>] [KEY=<allockey>] [ACTIVE=1|0] [MACHINES=<mach1:mach2:...>|ANY] [TYPE=debit|credit] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]
# If MACHINES specified will look for a match (for <mach1:mach2:...> or ANY)
# in the allocation allowmachines string. If you want ALL suballocations,
# leave MACHINES unspecified.
# All types will be returned if TYPE is left unspecified.
# Returns an array of "accountname,username,amount,allockey,alloctype,expirationdate,activatindate,active,allowmachines,overdraft"
################################################################################
sub get_suballocations {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [USER=<user>] [KEY=<allockey>] [ACTIVE=1|0] [MACHINES=<mach1:mach2:...>|ANY] [TYPE=debit|credit] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]"); }

if ($ARGS{CHECKPOINT}) { $X="x" } else { $X="" }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="accountname,username,amount,${X}allocation.allockey,alloctype,expirationdate,activationdate,active,allowmachines,overdraft" };

$select="select $ARGS{SHOW} from ${X}allocation,${X}suballocation where ${X}allocation.allockey=${X}suballocation.allockey";

if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{USER}) {$select.=" and username='$ARGS{USER}'"}
if ($ARGS{KEY}) {$select.=" and ${X}allocation.allockey=$ARGS{KEY}"}
if ($ARGS{ACTIVE} eq "1") {$select.=" and active='t'"}
if ($ARGS{ACTIVE} eq "0") {$select.=" and active='f'"}
if ($ARGS{MACHINES}) {$select.=" and (allowmachines~':$ARGS{MACHINES}:' or allowmachines=':ANY:')"}
if ($ARGS{TYPE}) {$select.=" and alloctype='$ARGS{TYPE}')"}
if ($ARGS{CHECKPOINT}) {$select.=" and ${X}allocation.checkpointkey=$ARGS{CHECKPOINT} and ${X}suballocation.checkpointkey=$ARGS{CHECKPOINT}"}

&db($select) || return &fail($dberror);
return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_suballocations

=over 4

=item B<NAME>

get_suballocations - display suballocation information

=item B<SYNOPSIS>

get_suballocations [B<ACCOUNT>=I<account>] [B<USER>=I<user>]
[B<KEY>=I<allockey>] [B<ACTIVEB>=I<1|0>] [B<MACHINES>=I<mach1:mach2:...|ANY>]
[B<CHECKPOINT>=I<ckptkey>] [B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

Dumps a line for each suballocation specified Primarily used within
administrative tools to provide account information Information
accessible depends on one's administrative priveledges. Specifying
B<C<ACTIVE=1>> will return only currently active suballocations
(excluding suballocations which have not yet started or which have
already expired). The default is to return all suballocations (Active
or Inactive).

=item B<OPTIONS>

B<ALLOCTYPE> can be one of debit or credit.

=item B<RETURN VALUES>

Returns pipe-delimited arrays of "accountname, username, amount,
allockey, alloctype, expirationdate, activationdate, active, overdraft"
unless overridden by the I<SHOW> parameter.

=back

=cut


################################################################################
# get_subreservations [ACCOUNT=<account>] [USER=<user>] [JOBID=<jobid>] 
#                     [EXPIRED=1|0] [MACHINE=<machine>] [CHECKPOINT=<ckptkey>]
#                     [SHOW=<attributelist>]
# Returns an array of "accountname,username,amount,jobid,expirationdate,machine"
################################################################################
sub get_subreservations {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [USER=<user>] [JOBID=<jobid>] [EXPIRED=1|0] [MACHINE=<machine>] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]"); }

if ($ARGS{CHECKPOINT}) { $X="x" } else { $X="" }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="accountname,username,amount,${X}reservation.jobid,expirationdate,machine" };

$select="select $ARGS{SHOW} from ${X}reservation,${X}subreservation where ${X}reservation.jobid=${X}subreservation.jobid";

if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{USER}) {$select.=" and username='$ARGS{USER}'"}
if ($ARGS{JOBID}) {$select.=" and ${X}reservation.jobid='$ARGS{JOBID}'"}
if ($ARGS{EXPIRED} eq "1") {$select.=" and expirationdate<'now'"}
if ($ARGS{EXPIRED} eq "0") {$select.=" and expirationdate>='now'"}
if ($ARGS{MACHINE}) {$select.=" and machine='$ARGS{MACHINE}'"}
if ($ARGS{CHECKPOINT}) {$select.=" and ${X}reservation.checkpointkey=$ARGS{CHECKPOINT} and ${X}subreservation.checkpointkey=$ARGS{CHECKPOINT}"}

&db($select) || return &fail($dberror);
return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_subreservations

=over 4

=item B<NAME>

get_subreservations - displays subreservation information

=item B<SYNOPSIS>

get_subreservations [B<ACCOUNT>=I<account>] [B<USER>=I<user>]
[B<JOBID>=I<jobid>] [B<EXPIRED>=I<1|0>] [B<MACHINE>=I<machine>]
[B<CHECKPOINT>=I<ckptkey>] [B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

Dumps a line for each subreservation specified Primarily used within
administrative tools to provide account information Information
accessible depends on one's administrative priveledges.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns pipe-delimited arrays of "accountname, username, amount,
jobid, expirationdate" unless overridden by the I<SHOW> parameter.

=back

=cut


##############################################################
# get_users [USER=<user>] [SHOW=<attributelist>]
# returns an array of "username,defaultaccount,realname,phonenumber,email,description"
##############################################################
sub get_users {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [USER=<user>] [SHOW=<attributelist>]"); }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="username,defaultaccount,realname,phonenumber,email,description" };

$select="select $ARGS{SHOW} from userinfo";
if ($ARGS{USER}) {$select.=" where username='$ARGS{USER}'"}

&db($select) || return &fail($dberror);

return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_users

=over 4

=item B<NAME>

get_users - display user information

=item B<SYNOPSIS>

get_users [B<USER>=I<user>] [B<SHOW>=I<atribute list>]

=item B<DESCRIPTION>

Dumps a line for each user specified 

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns pipe-delimited arrays of "username, defaultaccount, realname,
phonenumber, email, description" unless overridden by the I<SHOW>
parameter.

=back

=cut


##############################################################
# add_user USER=<user> [DEFAULT=<defaultaccount>] [NAME="<realname>"] [PHONE="<phonenumber>"] [EMAIL="<emailaddress>"] [DESCRIPTION="<description>"]
# Creates a user and initially populates associated user information.
# This subroutine will not update userinfo if user already exists.
# returns success or failure message
##############################################################
sub add_user {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{USER})
        { return &usage("$ARGS{COMMAND} USER=<user> [DEFAULT=<defaultaccount>] [NAME=\"<realname>\"] [PHONE=\"<phonenumber>\"] [EMAIL=\"<emailaddress>\"] [DESCRIPTION=\"<description>\"]"); }

# Fail if user already exists
&db("select username from userinfo where username='$ARGS{USER}'") || return &fail($dberror);
if ($ROWS)
	{return &deny("User $ARGS{USER} already exists.");}

# Add the user
&db("insert into userinfo (username,defaultaccount,realname,phonenumber,email,description) values ('$ARGS{USER}','$ARGS{DEFAULT}','$ARGS{NAME}','$ARGS{PHONE}','$ARGS{EMAIL}','$ARGS{DESCRIPTION}')") || return &fail($dberror);

# Log the transaction
&bank_log(4,"@_");

return &success("Successfully created user $ARGS{USER}");
}

=pod

=head1 add_user

=over 4

=item B<NAME>

add_user - create user entry

=item B<SYNOPSIS>

add_user B<USER>=I<user> [B<DEFAULT>=I<default account>] [B<NAME>=I<"realname">]
[B<PHONE>=I<"phonenumber">] [B<EMAIL>=I<"emailaddress">]
[B<DESCRIPTION>=I<"description">]

=item B<DESCRIPTION>

Creates a user and initially populates associated user
information. This subroutine will not update userinfo if user already
exists.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a success or failure message.

=back

=cut


##############################################################
# add_subaccount USER=<user> ACCOUNT=<account> [DEFAULT=1|0] [STATUS=enabled|disabled|admin]
# If the user does not already exist, the user is created.
# If a default account does not already exist for the user, it 
# makes ACCOUNT the default account unless overidden with DEFAULT=0.
# The subaccount is enabled by default unless overridden by STATUS.
# returns success or failure message
##############################################################
sub add_subaccount {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT}||!$ARGS{USER})
        { return &usage("$ARGS{COMMAND} USER=<user> ACCOUNT=<account> [DEFAULT=1|0] [STATUS=enabled|disabled|admin]"); }

if ($ARGS{STATUS} eq "disabled") { $userstatus='disabled' }
elsif ($ARGS{STATUS} eq "admin") { $userstatus='admin' }
else { $userstatus='enabled' }

# Verify account exists
&db("select accountname from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
unless ($ROWS)
	{return &deny("$ARGS{ACCOUNT} is not a valid account");}

# Verify user exists
&db("select username from userinfo where username='$ARGS{USER}'") || return &fail($dberror);
unless ($ROWS)
	{return &deny("$ARGS{USER} is not a valid user"); }

# Add the subaccount
&db("insert into subaccount (accountname,username,userstatus,depositmask,creationdate) values ('$ARGS{ACCOUNT}','$ARGS{USER}','$userstatus','0','now')") || return &fail($dberror);

# If no default account exists, make this the default account
&db("select defaultaccount from userinfo where username='$ARGS{USER}'") || return &fail($dberror);
if ($ARGS{DEFAULT} eq "1" || ($QUERYRESULT[0] eq '' && $ARGS{DEFAULT} ne "0")) {
	&db("update userinfo set defaultaccount='$ARGS{ACCOUNT}' where username='$ARGS{USER}'") || return &fail($dberror);
	}

# Log the transaction
&db("insert into txnlog (txntype,accountname,username,timestamp,authorization) values ('add_subaccount','$ARGS{ACCOUNT}','$ARGS{USER}','now','$me')") || return &fail($dberror);
&bank_log(4,"@_");

return &success("Successfully added user $ARGS{USER} to account $ARGS{ACCOUNT}");
}

=pod

=head1 add_subaccount

=over 4

=item B<NAME>

add_subaccount - add a user to an account

=item B<SYNOPSIS>

add_subaccount B<ACCOUNT>=I<account> B<USER>=I<user> [B<DEFAULT>=I<1|0>]
[B<STATUS>=I<enabled|disabled|admin>]

=item B<DESCRIPTION>

This command maps a user to an account, creating a user subaccount. If
the user does not already exist, the user is defined in
B<userinfo>. If a default account does not already exist for the user,
it makes I<account> the default account unless overidden with
B<DEFAULT=I<0>>. The subaccount is enabled by default unless
overridden by B<STATUS>.

=item B<OPTIONS>

=item B<RETURN VALUES>

returns 1 if successful, else 0

=back

=cut


##############################################################
# remove_user USER=<user>
# The user is removed from the userinfo table iff
# they have no subaccounts remaining. 
# Returns success or failure message
##############################################################
sub remove_user {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{USER})
        { return &usage("$ARGS{COMMAND} USER=<user>"); }

# Fail if user does not exist in userinfo
&db("select username from userinfo where username='$ARGS{USER}'") || return &fail($dberror);
if ($ROWS==0) { return &fail("User $ARGS{USER} was not found in userinfo"); }

# Fail if user is a member of any accounts
&db("select accountname from subaccount where username='$ARGS{USER}'") || return &fail($dberror);
if ($ROWS>0) { return &deny("User $ARGS{USER} is still a member of account(s) @QUERYRESULT"); }

# Remove user
&db("delete from userinfo where username='$ARGS{USER}'") || return &fail($dberror);

&bank_log(4,"@_");
if ($ROWS) { return &success("Successfully deleted user $ARGS{USER} from userinfo"); }
else { return &deny("No users were deleted"); }
}

=pod

=head1 remove_user

=over 4

=item B<NAME>

remove_user - removes suer from userinfo table

=item B<SYNOPSIS>

remove_user B<USER>=I<user>

=item B<DESCRIPTION>

The user is removed from the userinfo table iff they have no
subaccounts remaining.

=item B<OPTIONS>

=item B<RETURN VALUES>

returns a success or failure message.

=back

=cut


##############################################################
# remove_subaccount ACCOUNT=<account> USER=<user>
# Removes a user subaccount from the specified account
# The user's balance will be transferred to the RESERVE subaccount
# Should probably be modified to add depositmask portion to KITTY
# Returns message indicating success or failure
##############################################################
sub remove_subaccount {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT}||!$ARGS{USER})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<account> USER=<user>"); }

if ($ARGS{USER} eq 'KITTY' || $ARGS{USER} eq 'ANY' || $ARGS{USER} eq 'RESERVE')
	{ return &deny("$ARGS{USER} is a reserved subaccount and may not be removed."); }

$sum=$amount=0;
&db("select allocation.allockey,amount,alloctype,overdraft from allocation,suballocation where allocation.allockey=suballocation.allockey and accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}'") || return &fail($dberror);
foreach $suballocation (@SUBALLOCATIONS=@QUERYRESULT) {
	($allockey,$amount,$type,$overdraft)=split(/\|/,$suballocation);
	# Transfer user balance to RESERVE (should work for all allocation types)
	&db("update suballocation set amount=amount+$amount,overddraft=overdraft+$overdraft where allockey=$allockey and username='RESERVE'") || return &fail($dberror);
	unless ($ROWS) { 	# Create suballocation if nonexistent
		&db("insert into suballocation (allockey,username,amount) values ($allockey,'RESERVE',$amount)") || return &fail($dberror);
		}
	$sum=$sum+$amount-$overdraft;

	# Remove suballocation
	&db("delete from suballocation where allockey=$allockey and username='$ARGS{USER}'") || return &fail($dberror);
	}

# Remove subaccount
&db("delete from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}'") || return &fail($dberror);

# Clear defaultaccount if set to this account
&db("update userinfo set defaultaccount='' where username='$ARGS{USER}' and defaultaccount='$ARGS{ACCOUNT}'") || return &fail($dberror);

# The amount logged represents how much was transferred to the RESERVE account
$sum*=1;			# Make sure amount is not null
&db("insert into txnlog (txntype,accountname,username,amount,timestamp,authorization) values ('remove_subaccount','$ARGS{ACCOUNT}','$ARGS{USER}',$sum,'now','$me')") || return &fail($dberror);
&bank_log(4,"@_");

if ($ROWS) { return &success("Successfully removed user $ARGS{USER} from account $ARGS{ACCOUNT}"); }
else { return &deny("No users were removed from account $ARGS{ACCOUNT}"); }
}

=pod

=head1 remove_subaccount

=over 4

=item B<NAME>

remove_subaccount - removes a user from an account

=item B<SYNOPSIS>

remove_subaccount B<ACCOUNT>=I<account> B<USER>=I<user>

=item B<DESCRIPTION>

Removes a user subaccount from the specified account and transfers
their remaining balance to the RESERVE subaccount.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a success or failure message

=back

=cut


##############################################################
# change_user USER=<user> [DEFAULT=<defaultaccount>] [NAME="<realname>"] [PHONE="<phonenumber>"] [EMAIL="<emailaddress>"] [DESCRIPTION="<description>"]
# Changes the specified userinfo attributes for a user
# First verifies that user exists.
# Returns message indicating success or failure
##############################################################
sub change_user {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{USER})
        { return &usage("$ARGS{COMMAND} USER=<user> [DEFAULT=<defaultaccount>] [NAME=\"<realname>\"] [PHONE=\"<phonenumber>\"] [EMAIL=\"<emailaddress>\"] [DESCRIPTION=\"<description>\"]"); }

# Verify user exists
&db("select username from userinfo where username='$ARGS{USER}'") || return &fail($dberror);
unless ($ROWS)
	{return &deny("$ARGS{USER} does not exist.");}

if ($ARGS{DEFAULT}) { &db("update userinfo set defaultaccount='$ARGS{DEFAULT}' where username='$ARGS{USER}'") || return &fail($dberror); }
if ($ARGS{NAME}) { &db("update userinfo set realname='$ARGS{NAME}' where username='$ARGS{USER}'") || return &fail($dberror); }
if ($ARGS{PHONE}) { &db("update userinfo set phonenumber='$ARGS{PHONE}' where username='$ARGS{USER}'") || return &fail($dberror); }
if ($ARGS{EMAIL}) { &db("update userinfo set email='$ARGS{EMAIL}' where username='$ARGS{USER}'") || return &fail($dberror); }
if ($ARGS{DESCRIPTION}) { &db("update userinfo set description='$ARGS{DESCRIPTION}' where username='$ARGS{USER}'") || return &fail($dberror); }

if ($ROWS) { return &success("Successfully changed user $ARGS{USER}"); }
else { return &deny("No changes were made to $ARGS{USER}"); }
}

=pod

=head1 change_user

=over 4

=item B<NAME>

change_user - changes the specified userinfo attributes.

=item B<SYNOPSIS>

change_user B<USER>=I<user> [B<DEFAULT>=I<defaultaccount>] [B<NAME>=I<"realname">]
[B<PHONE>=I"phonenumber">] [B<EMAIL>=I<"emailaddress">]
[B<DESCRIPTION>=I<"description">]

=item B<DESCRIPTION>

Changes the specified userinfo attributes for a user.  First verifies
that the user exists.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating success or failure.

=back 

=cut


##############################################################
# change_subaccount USER=<user> ACCOUNT=<account> STATUS=enabled|disabled|admin
# Changes properties of a subaccount
# Currently only function is to change the status for a user
# Returns message indicating success or failure
##############################################################
sub change_subaccount {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
$changes=0;

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT}||!$ARGS{USER}||!$ARGS{STATUS})
        { return &usage("$ARGS{COMMAND} USER=<user> ACCOUNT=<account> STATUS=enabled|disabled|admin"); }

unless (grep($_ eq $ARGS{STATUS},@STATUS=(admin,enabled,disabled))) {
	return &deny("$ARGS{STATUS} is not a valid user status")
	}

&db("update subaccount set userstatus='$ARGS{STATUS}' where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and userstatus!='reserved'") || return &fail($dberror);
$changes+=$ROWS;

&bank_log(4,"@_");
return &success("Successfully made $changes changes to subaccount $ARGS{ACCOUNT}:$ARGS{USER}");
}

=pod

=head1 change_subaccount

=over 4

=item B<NAME>

change_subaccount - changes subaccount attributes such as user status.

=item B<SYNOPSIS>

change_subaccount B<USER>=I<user> B<ACCOUNT>=I<account>
B<STATUS>=I<enabled|disabled|admin>

=item B<DESCRIPTION>

Currently only function is to change the status for a subaccount
userstatus may be set to one of the following values:

=over 4

=item B<admin>:

authorized to perform account_level administrative functions

=item B<enabled>:

enables a subaccount for withdrawals and reservations

=item B<disabled>:

withdrawals and reservations cannot be made

=back

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating success or failure.

=back

=cut


##############################################################################
# user_enabled ACCOUNT=<account> USER=<user>
# Returns 1 if user is enabled, else 0
##############################################################################
sub user_enabled {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and userstatus='enabled'") || return 0;

return &success($ROWS);
}

=pod

=head1 user_enabled

=over 4

=item B<NAME>

user_enabled - checks user status

=item B<SYNOPSIS>

user_enabled B<ACCOUNT>=I<account> B<USER>=I<user>

=item B<DESCRIPTION>

Checks whether a user is enabled or not.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if enabled else 0.

=back

=cut


##############################################################################
# account_admin ACCOUNT=<account> USER=<user>
# Internal call used within authorize subroutine
# Returns 1 if user is an account admin, else 0
##############################################################################
sub account_admin {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and userstatus='admin'") || return 0;

return &success($ROWS);
}

=pod

=head1 account_admin

=over 4

=item B<NAME>

account_admin - internal call used within I<authorize> subroutine.

=item B<SYNOPSIS>

account_admin B<ACCOUNT>=I<account> B<USER>=I<user>

=item B<DESCRIPTION>

Checks whether a user is an account administrator.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if user is an account admin, else 0.

=back

=cut


##############################################################################
# bank_admin USER=<user>
# Internal call used within authorize subroutine
# Returns 1 if user is a bank admin, else 0
##############################################################################
sub bank_admin {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

return &success(grep(/^$ARGS{USER}$/,@BANK_ADMINS) && 1 || 0);
}

=pod

=head1 bank_admin

=over 4

=item B<NAME>

bank_admin - internal call used within I<authorize> subroutine

=item B<SYNOPSIS>

bank_admin B<USER>=I<user>

=item B<DESCRIPTION>

Checks whether a user is a bank administrator

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if user is a bank admin, else 0.

=back

=cut


##############################################################################
# authorize($auth_level,@command)
# Exclusively used within other subroutines for command authorization
# Auth_level must be one of the following:
#
#	bank	access granted to bank administrators only
#
#	acct	account-level access granted to account administrators
#		global access granted to bank administrators
#
#	user	users granted access to their own subaccount
#		account-level access granted to account administrators
#		global access granted to bank administrators
#
# Returns 1 if authorized to perform the command, else 0
##############################################################################
sub authorize {
$auth_level = shift(@_);
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

for ($auth_level) {

	/bank/ && do {
		return 0 unless (&result(&bank_admin("USER=$me")));
		};

	/acct/ && do {
		return 0 unless (&result(&bank_admin("USER=$me"))||&result(&account_admin("ACCOUNT=$ARGS{ACCOUNT}","USER=$me")));
		};

	/user/ && do {
		return 0 unless (&result(&bank_admin("USER=$me"))||&result(&account_admin("ACCOUNT=$ARGS{ACCOUNT}","USER=$me"))||"$ARGS{USER}" eq "$me"||"$ARGS{USER}" eq "KITTY");
		};

	}

return 1;
}

=pod

=head1 authorize

=over 4

=item B<NAME>

authorize - check permissions

=item B<SYNOPSIS>

authorize(I<$auth_level>, I<@command>

=item B<DESCRIPTION>

Exclusively used ithin other subroutines for command authentication
Auth_level must be one of the following:

=over 4

=item bank 

 Access granted to bank administrators only

=item account

 Account-level access granted to account administrators.
 Global access granted to bank administrators.

=item user 

 Users granted access to their own subaccount. 
 Account-level access granted to account administrators. 
 Global access granted to bank administrators

=back 

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if authorized to perform the command, else 0.

=back

=cut


##############################################################
# show_mask ACCOUNT=<account>
# Returns the deposit mask
##############################################################
sub show_mask {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local(@depositmask=());

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<account>"); }

&db("select username,depositmask from subaccount where accountname='$ARGS{ACCOUNT}' and depositmask>0") || return &fail($dberror);
foreach (@SUBACCOUNTS=@QUERYRESULT) {
	($user,$mask)=split('\|',$_);
	push(@depositmask,"$user=$mask");
	}

return &success(join(' ',@depositmask));
}

=pod

=head1 show_mask

=over 4

=item B<NAME>

show_mask - shows deposit mask.

=item B<SYNOPSIS>

show_mask B<ACCOUNT>=I<account>

=item B<DESCRIPTION>

Shows the deposit mask for an account in the form:

      user1=percent1 user2=percent2 user3=percent3

i.e.

      scottmo=25 monkeyboy=30 KITTY=25 RESERVE=20

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns the deposit mask in name-value pairs.

=back

=cut


##############################################################
# set_mask ACCOUNT=<account> <user1>=<percent> ...
# Ensures allocation distribution equals 100 and users are account members
# Returns new mask if successful
##############################################################
sub set_mask {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($user,$percent,$sum=0,%mask=());

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<account> <user1>=<percent> ...\n\tMask distribution must total 100"); }

# Parse mask out of command, make sure the distribution=100% and
# the mask percentages are non-negative integers
foreach (@mask=grep(!/COMMAND|ACCOUNT|AUTH/,@_)) {
	($user,$percent)=split(/=/);	
	next unless (&result(&account_member("ACCOUNT=$ARGS{ACCOUNT}","USER=$user")));
	if ($percent<0) { return &deny("mask percentages cannot be negative"); }
	if (int($percent)!=$percent) { return &deny("mask percentages must be integers"); }
	$sum+=$percent;
	$mask{$user}=$percent; # Populate mask hash
	}
unless ($sum==100) { return &deny("Distribution must total 100%"); }

# Clear old deposit mask in subaccounts
&db("update subaccount set depositmask='0' where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);

# Push new deposit mask to subaccounts
@mask=();
foreach $user (keys %mask) {
	&db("update subaccount set depositmask='$mask{$user}' where accountname='$ARGS{ACCOUNT}' and username='$user'") || return &fail($dberror);
	push(@mask,"$user=$mask{$user}");
	}

return &success(join(' ',@mask));
}

=pod

=head1 set_mask

=over 4

=item B<NAME>

set_mask - sets the deposit mask.

=item B<SYNOPSIS>

set_mask B<ACCOUNT>=I<account> I<user1>=I<percent1> ... 

=item B<DESCRIPTION>

Sets the deposit mask for an account. This mask will be applied at
time of deposit to distribute allocations among the subaccounts
(users). For example, for a deposit mask of 'I<scottmo=20 monkeyboy=30
KITTY=50>', a deposit of 1000 nodeseconds would allocate 200 to
I<scottmo>, 300 to I<monkeyboy> and 500 to the I<KITTY> pool. Each invocation
of the command must specify the entire deposit mask This subroutine
checks to ensure that the overall percentage totals 100, and that the
individual percentages are non-negative integers.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns the new deposit mask.

=back

=cut


##############################################################
# make_deposit ACCOUNT=<account> [USER=<user>] AMOUNT=<nodeseconds> 
#              [KEY=<allockey>] [TYPE=debit|credit]
#					[EXPIRES=<MM/DD[/YY][_hh:mm:ss]>]
#					[STARTS=<MM/DD[/YY][_hh:mm:ss]>]
#              [MACHINES=<mach1:mach2:...>|ANY] [REASON="<reason>"]
# Makes a deposit to USER subaccount if specified, otherwise uses depositmask.
# Allowmachines is set to the account's defaultmachines value unless specified
# by a colon delimited list of machines or the special keyword ANY
# if EXPIRES attribute is left out, zero or null then expirationdate='infinity'
# if STARTS attribute is left out then activationdate='-infinity'
# You cannot specify KEY with (EXPIRES, STARTS, MACHINES or TYPE)
# TYPE must be one of debit or credit and defaults to debit if unspecified.
# If the TYPE is specified to be credit, and no matching existing credit
# allocation can be found, then AMOUNT is used to set the credit limit. 
# If a matching credit allocation is found, then the allocation balance will
# be increased by AMOUNT and the credit limit will remain unchanged.
# A credit allocation may be used to establish an overdraft buffer or used
# as a credit account on a use first, pay later basis.
# Returns a message indicating success or failure
##############################################################
sub make_deposit {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($mask,$largestkey=0,@keys,@allocation);

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||(!$ARGS{ACCOUNT}&&!$ARGS{KEY})||!$ARGS{AMOUNT})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<account> [USER=<user>] AMOUNT=<nodeseconds> [KEY=<allockey>] [TYPE=debit|credit] [EXPIRES=<MM/DD[/YY][_hh:mm:ss]>] [STARTS=<MM/DD[/YY][_hh:mm:ss]>] [MACHINES=<mach1:mach2:...>|ANY] [REASON=\"<reason>\"]"); }

# Cannot specify KEY and (EXPIRES, STARTS, MACHINES or TYPE)
if ($ARGS{KEY}&&($ARGS{EXPIRES}||$ARGS{STARTS}||$ARGS{MACHINES}||$ARGS{TYPE}))
	{ return &usage("$ARGS{COMMAND}: You cannot specify KEY with EXPIRES, STARTS, MACHINES or TYPE"); }

# Verify account exists
&db("select accountname from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
unless ($ROWS)
	{return &deny("$ARGS{ACCOUNT} is not a valid account");}

if ($ARGS{AMOUNT}<0)
	{ return &deny("Deposit amount cannot be negative"); }

# Parse and format expiration time to an acceptable database format
if ($ARGS{EXPIRES}) {
	$_=$ARGS{EXPIRES};	# Parse time expression
	($month,$day,$slash,$year,$underscore,$time,$hour,$minute,$second)=/(\d\d)\/(\d\d)(\/(\d\d))?(_((\d\d):(\d\d):(\d\d)))?/;
	# Generate usage if incorrect
unless ($month)	{ return &usage("$ARGS{COMMAND} ACCOUNT=<account> [USER=<user>] AMOUNT=<nodeseconds> [EXPIRES=<MM/DD[/YY][_hh:mm:ss]>] [STARTS=<MM/DD[/YY][_hh:mm:ss]>] [REASON=\"<reason>\"]"); }
	# Format time to a database acceptable format
	$etime="$month/$day/".($year?$year:(localtime)[5]).($time?" $time":"");
	}
else	{ $etime='infinity'; }

# Parse and format activation time to an acceptable database format
if ($ARGS{STARTS}) {
	$_=$ARGS{STARTS};	# Parse time expression
	($month,$day,$slash,$year,$underscore,$time,$hour,$minute,$second)=/(\d\d)\/(\d\d)(\/(\d\d))?(_((\d\d):(\d\d):(\d\d)))?/;
	# Generate usage if incorrect
unless ($month)	{ return &usage("$ARGS{COMMAND} ACCOUNT=<account> [USER=<user>] AMOUNT=<nodeseconds> [EXPIRES=<MM/DD[/YY][_hh:mm:ss]>] [STARTS=<MM/DD[/YY][_hh:mm:ss]>] [REASON=\"<reason>\"]"); }
	# Format time to a database acceptable format
	$atime="$month/$day/".($year?$year:(localtime)[5]).($time?" $time":"");
	}
else	{ $atime='-infinity'; }

# If MACHINES is specified, check it for valid syntax
if ($ARGS{MACHINES}) {
	unless ($ARGS{MACHINES} =~ /^[\w:]+$/)
       		{ return &deny("Machine Argument must be a colon-delimited list of alphanumeric [a-zA-Z_0-9]\nnames or the special keyword ANY."); }
	$machines=":$ARGS{MACHINES}:";
	}
else { # If MACHINES is not specified, use the account's defaultmachines
	&db("select defaultmachines from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
	# If defaultmachines is unpopulated, fallback to ANY
	if ($ROWS) { ($machines)=@QUERYRESULT[0]; }
	else { $machines=':ANY:'; }
	}

# Set TYPE to debit if unspecified
if (defined $ARGS{TYPE})
{
	$type=$ARGS{TYPE};
	$ARGS{REASON}.=" Overdraft amount $overdraft";
}
else { $type='debit'; }
	 
$key=$debitkey=$creditkey=0;
# Look to see if an allocation with the right key or (expirationdate, activationdate, alloctype and allowmachines) exists
if ($ARGS{KEY}) {
	&db("select allockey,alloctype from allocation where accountname='$ARGS{ACCOUNT}' and allockey=$ARGS{KEY}") || return &fail($dberror);
	($key,$type)=$QUERYRESULT[0]; $keycount=$ROWS;
	if (!$ROWS) { return &fail("Allocation Key $ARGS{KEY} is not valid"); }
	if ($type eq 'credit') { $creditkey=$key; }
	else { $debitkey=$key; }
	}
else {
	&db("select allockey from allocation where accountname='$ARGS{ACCOUNT}' and activationdate='$atime' and expirationdate='$etime' and allowmachines='$machines' and alloctype='debit'") || return &fail($dberror);
	$debitkey=$QUERYRESULT[0];
	&db("select allockey from allocation where accountname='$ARGS{ACCOUNT}' and activationdate='$atime' and expirationdate='$etime' and allowmachines='$machines' and alloctype='credit'") || return &fail($dberror);
	$creditkey=$QUERYRESULT[0];
	}

# Here are the cases (assuming the allocations match the specified flags):
#
# Deposit			 				Some 				Some 				Some
# Specifies		No 				Credit			Debit				Debit and Credit
# Type:			Allocations		Allocations		Allocations		Allocations
# =========		===========		===========		===========		===========
#
# Debit			Create Debit	Create Debit	Put all			Put all
#					and put all		and put all		in Debit			in Debit
#
# Credit			Create Credit	Put in Credit	Create Credit	Put in Credit
#					and put all		and overflow	and put all		and overflow
#
# Unspecified	Create Credit	Put in Credit	Put all			Put in Credit
#					and put all		and overflow	in Debit			til filled,
#																				then into Debit

# We need a new debit allocation if debit is specified and no debit key exists
# We need a new credit allocation if credit specified and no credit key exists
# We need a new debit allocation if unspecified and no allocations exist
if (($ARGS{TYPE} eq 'debit' && !$debitkey) || ($ARGS{TYPE} eq 'credit' && !$creditkey) || (!defined $ARGS{TYPE} && !$debitkey && !$creditkey)) {
	# Create a unique allocation key
	&db("select max(allockey) from allocation") || return &fail($dberror);
	$key=$QUERYRESULT[0];
	@KEYS=(++$key);
	if ($ARGS{TYPE} eq 'credit') { $creditkey=$key; }
	else { $debitkey=$key; }

	# Create new allocation
	&db("insert into allocation (accountname,allockey,alloctype,expirationdate,activationdate,creationdate,active,allowmachines) values ('$ARGS{ACCOUNT}',$key,'$type','$etime','$atime','now','t','$machines')") || return &fail($dberror);
	# Unset the active flag if unstarted or already expired
	&db("update allocation set active='f' where allockey=$key and (activationdate>'now' or expirationdate<'now')");
	}

if ($ARGS{USER}) {@MASK="$ARGS{USER}|100"}
else {
	&db("select username,depositmask from subaccount where accountname='$ARGS{ACCOUNT}' and depositmask>0") || return &fail($dberror);
	@MASK=@QUERYRESULT;
	}

# Verify users (whether specified explicitly or in depositmask) are members of the account and that the sum of the depositmasks is 100
$masktotal=0;
foreach (@MASK) {
	($user,$mask)=split(/\|/,$_);
	$masktotal+=$mask;
	&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$user'") || return &fail($dberror);
	unless ($ROWS)
		{return &fail("$user is not a member of account $ARGS{ACCOUNT} -- Possibly a stale depositmask");}
	}
if ($masktotal != 100) { return &deny("Deposit mask does not total 100%"); }

$creditamt=$debitamt=$deptotal=0;
# We need to first deposit to credit allocation if TYPE was specified as credit
# or unspecified and creditkey was found or created.
# Then deposit to debit allocation if TYPE was specified as debit
# or unspecified and debitkey was found or created.
if ($ARGS{TYPE} eq 'credit') { $creditamt=$ARGS{AMOUNT}; }
elsif ($ARGS{TYPE} eq 'debit') { $debitamt=$ARGS{AMOUNT}; }
elsif ($creditkey && !$debitkey) { $creditamt=$ARGS{AMOUNT}; }
elsif ($debitkey && !$creditkey) { $debitamt=$ARGS{AMOUNT}; }
elsif ($debitkey && $creditkey) {
	# First fill up the user's credit suballocation as far as it will go
	&db("select amount,overdraft from suballocation where allockey=$creditkey and username='$user'") || return &fail($dberror);
	($credit,$overdraft)=split(/\|/,$QUERYRESULT[0]);
	$creditamt=$ARGS{AMOUNT}<($overdraft-$credit)?$ARGS{AMOUNT}:($overdraft-$credit);
	# The rest will go into a debit suballocation
	$debitamt=$ARGS{AMOUNT}-$creditamt;
	}
elsif (!$debitkey && !$creditkey) { return &fail("Unexpected -- No allocation key found or created for deposit"); }

# Make appropriate deposits to the credits allocations
if ($creditamt) {
	foreach (@MASK) {
		($user,$mask)=split('\|',$_);
		# /100% for mask multiplication and use int((2*n+1)/2) to round
		$amount=int((2*($creditamt*$mask/100)+1)/2);

		&db("update suballocation set amount=amount+$amount where allockey=$creditkey and username='$user'") || return &fail($dberror);
		unless ($ROWS) { # Create suballocation if nonexistent
			# Set overdraft if suballocation does not already exist
			&db("insert into suballocation (allockey,username,amount,overdraft) values ($creditkey,'$user',$amount,$amount)") || return &fail($dberror);
			}
		$deptotal+=$amount;
		# Log the subdeposit in the txnlog
		&db("insert into txnlog (txntype,accountname,username,key,amount,expirationdate,activationdate,timestamp,authorization,reason) values ('make_subdeposit','$ARGS{ACCOUNT}','$user','$creditkey',$amount,'$etime','$atime','now','$me','$ARGS{REASON}')") || return &fail($dberror);
		}
	# Look up appropriate values for correct logging
	&db("select active from allocation where allockey=$creditkey") || return &fail($dberror);
	($active)=$QUERYRESULT[0];

	&db("insert into txnlog (txntype,accountname,key,amount,expirationdate,activationdate,timestamp,authorization,reason,active,machine) values ('make_deposit','$ARGS{ACCOUNT}','$creditkey',$deptotal,'$etime','$atime','now','$me','$ARGS{REASON}','$active','$machines')") || return &fail($dberror);
	}

# Then make appropriate deposits to debit allocations
if ($debitamt) {
	foreach (@MASK) {
		($user,$mask)=split('\|',$_);
		# /100% for mask multiplication and use int((2*n+1)/2) to round
		$amount=int((2*($debitamt*$mask/100)+1)/2);

		&db("update suballocation set amount=amount+$amount where allockey=$debitkey and username='$user'") || return &fail($dberror);
		unless ($ROWS) { # Create suballocation if nonexistent
			&db("insert into suballocation (allockey,username,amount,overdraft) values ($debitkey,'$user',$amount,0)") || return &fail($dberror);
			}
		$deptotal+=$amount;
		# Log the subdeposit in the txnlog
		&db("insert into txnlog (txntype,accountname,username,key,amount,expirationdate,activationdate,timestamp,authorization,reason) values ('make_subdeposit','$ARGS{ACCOUNT}','$user','$debitkey',$amount,'$etime','$atime','now','$me','$ARGS{REASON}')") || return &fail($dberror);
		}
	# Look up appropriate values for correct logging
	&db("select active from allocation where allockey=$debitkey") || return &fail($dberror);
	($active)=$QUERYRESULT[0];

	&db("insert into txnlog (txntype,accountname,key,amount,expirationdate,activationdate,timestamp,authorization,reason,active,machine) values ('make_deposit','$ARGS{ACCOUNT}','$debitkey',$deptotal,'$etime','$atime','now','$me','$ARGS{REASON}','$active','$machines')") || return &fail($dberror);
	}

&bank_log(4,"@_");
return &success("Successfully deposited $deptotal credits");
}

=pod

=head1 make_deposit

=over 4

=item B<NAME>

make_deposit - deposit funds in an account.

=item B<SYNOPSIS>

make_deposit B<ACCOUNT>=I<account> [B<USER>=I<user>] B<AMOUNT>=I<nodeseconds>
[B<KEY>=I<allockey>] [B<EXPIRES>=I<MM/DD[/YY][_hh:mm:ss]>]
[B<STARTS>=I<MM/DD[/YY][_hh:mm:ss]>] [B<MACHINES>=I<mach1:mach2:...:|:ANY>]
[B<REASON>=I<"reason">] [B<TYPE>=debit|credit]

=item B<DESCRIPTION>

Makes a deposit to the specified account or user subaccount. If a
particular subaccount is not specified as the recipient the deposit
mask will determine how the new allocation is distributed to the
subaccounts.

Allowmachines is set to the account's defaultmachines value unless
B<MACHINES> is specified by a colon delimited list of machines or the
special keyword I<ANY>

If the B<EXPIRES> attribute is left out, zero or null then expirationdate
will default to B<'infinity'>. The deposit amount cannot be negative.

If B<STARTS> attribute is left out then B<activationdate='-infinity'> You
may use B<KEY> to directly deposit into a preexisting allocation so long
as B<STARTS> or B<EXPIRES> are not specified.

B<KEY> may be specified to indicate the precise allocation that you want to
deposit into.  You cannot specify B<KEY> with B<EXPIRES>, B<STARTS>, 
B<MACHINES>, B<TYPE> or B<OVERDRAFT>.

B<TYPE> indicates whether the allocation is a credit or debit based allocation.
It must be one of debit or credit and defaults to debit if unspecified.
If the B<TYPE> is specified to be credit, and no matching existing credit
allocation can be found, then B<AMOUNT> is used to set the credit limit. 
If a matching credit allocation is found, then the allocation balance will
be increased by B<AMOUNT> and the credit limit will remain unchanged.
A credit allocation may be used to establish an overdraft buffer or used
as a credit account on a use first, pay later basis.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating success of failure.

=back

=cut


##############################################################
# change_allocation KEY=<allockey> [EXPIRES=<MM/DD[/YY][_hh:mm:ss]>] 
#                   [STARTS=<MM/DD[/YY][_hh:mm:ss]>] [TYPE=debit|credit]
#                   [MACHINES=<mach1:mach2:...>|ANY]
# Changes the expirationdate, activationdate, allocation type and/or
# allowmachines of an allocation
# MACHINES must be a colon-delimited list of alphanumeric [a-zA-Z_0-9]
# names or the special keyword ANY to grant debit access from all machines.
# TYPE can be one of credit or debit.
# Warning: Changing the allocation type between debit and credit may produce
# undesirable effects and should only be used by persons with knowledge
# of QBank's internal handling of credit allocations.
# Returns message indicating success or failure
##############################################################
sub change_allocation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
$changes=0;

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{KEY})
        { return &usage("$ARGS{COMMAND} KEY=<allockey> [EXPIRES=<MM/DD[/YY][_hh:mm:ss]>] [STARTS=<MM/DD[/YY][_hh:mm:ss]>] [MACHINES=<mach1:mach2:...>|ANY] [TYPE=debit|credit]"); }

&db("select accountname from allocation where allockey=$ARGS{KEY}") || return &fail($dberror);
($account)=@QUERYRESULT;

# Parse and format expiration time to an acceptable database format
if ($ARGS{EXPIRES}) {
	$_=$ARGS{EXPIRES};	# Parse time expression
	($month,$day,$slash,$year,$underscore,$time,$hour,$minute,$second)=/(\d\d)\/(\d\d)(\/(\d\d))?(_((\d\d):(\d\d):(\d\d)))?/;
	# Generate usage if incorrect
unless ($month) { return &usage("$ARGS{COMMAND} KEY=<allockey> EXPIRES=<MM/DD[/YY][_hh:mm:ss]>"); }
	# Format time to a database acceptable format
	$etime="$month/$day/".($year?$year:(localtime)[5]).($time?" $time":"");
	&db("update allocation set expirationdate='$etime' where allockey=$ARGS{KEY}") || return &fail($dberror);
	$changes+=$ROWS;
	}

# Parse and format activation time to an acceptable database format
if ($ARGS{STARTS}) {
	$_=$ARGS{STARTS};	# Parse time expression
	($month,$day,$slash,$year,$underscore,$time,$hour,$minute,$second)=/(\d\d)\/(\d\d)(\/(\d\d))?(_((\d\d):(\d\d):(\d\d)))?/;
	# Generate usage if incorrect
unless ($month) { return &usage("$ARGS{COMMAND} KEY=<allockey> EXPIRES=<MM/DD[/YY][_hh:mm:ss]>"); }
	# Format time to a database acceptable format
	$atime="$month/$day/".($year?$year:(localtime)[5]).($time?" $time":"");
	&db("update allocation set activationdate='$atime' where allockey=$ARGS{KEY}") || return &fail($dberror);
	$changes+=$ROWS;
	}

if ($ARGS{MACHINES}) {
	unless ($ARGS{MACHINES} =~ /^[\w:]+$/)
       		{ return &deny("Machine Argument must be a colon-delimited list of alphanumeric [a-zA-Z_0-9]\nnames or the special keyword ANY."); }
	&db("update allocation set allowmachines=':$ARGS{MACHINES}:' where allockey='$ARGS{KEY}'") || return &fail($dberror);
	$changes+=$ROWS;
	}

if ($ARGS{TYPE}) {
	unless ($ARGS{TYPE} =~ /debit|credit/)
		{ return &deny("Type Argument must be either debit or credit."); }
	&db("update allocation set alloctype='$ARGS{TYPE}' where allockey='$ARGS{KEY}'") || return &fail($dberror);
	$changes+=$ROWS;
	}

&db("select activationdate,expirationdate from allocation where allockey=$ARGS{KEY}") || return &fail($dberror);
($atime,$etime)=split(/\|/,$QUERYRESULT[0]);

&db("insert into txnlog (txntype,key,accountname,expirationdate,activationdate,machine,timestamp,authorization,type) values ('change_allocation','$ARGS{KEY}','$account','$etime','$atime',':$ARGS{MACHINES}:','now','$me','$ARGS{TYPE}')") || return &fail($dberror);

# Expire/Unexpire Allocations as appropriate
&update_allocations;

&bank_log(4,"@_");
return &success("Successfully made $changes changes to allocation $ARGS{KEY}");
}

=pod

=head1 change_allocation

=over 4

=item B<NAME>

change_allocation - changes allocation dates.

=item B<SYNOPSIS>

change_allocation B<KEY>=I<allockey> [B<EXPIRES>=I<MM/DD[/YY][_hh:mm:ss]>]
[B<STARTS>=I<MM/DD[/YY][_hh:mm:ss]>] [B<MACHINES>=I<mach1:mach2:...:|:ANY>]
[B<TYPE>=debit|credit]

=item B<DESCRIPTION>

Changes the expirationdate, activationdate, list of allowed machines or
the type for an allocation

B<MACHINES> must be a colon-delimited list of alphanumeric [a-zA-Z_0-9]
names or the special keyword :ANY: and must begin and end with a
colon.

B<TYPE> can be one of credit or debit.
Warning: Changing the allocation type between debit and credit may produce
undesirable effects and should only be used by persons with knowledge
of QBank's internal handling of credit allocations.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating success or failure.

=back

=cut

##############################################################
# remove_allocation KEY=<allockey> | ACTIVE=0 [REASON="<reason>"]
# Logs how many cycles were lost when the allocation was removed
# Expiration philosophy is to remove them only on fiscal boundaries
# Logs whether or not the allocation was expired when it was removed
# Returns message indicating balance removed
##############################################################
sub remove_allocation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||(!$ARGS{KEY}&&($ARGS{ACTIVE} ne "0")))
        { return &usage("$ARGS{COMMAND} KEY=<allockey> | ACTIVE=0 [REASON=\"<reason>\"]"); }

if ($ARGS{KEY}) {
        &db("select accountname,allockey,active from allocation where allockey=$ARGS{KEY}") || return &fail($dberror);
        }
elsif ($ARGS{ACTIVE} eq "0") {
	&update_allocations;	# Check for newly expired allocations
        &db("select accountname,allockey,active,alloctype from allocation where active='f'") || return &fail($dberror);
        }
else { return &usage("$ARGS{COMMAND} KEY=<allockey> | ACTIVE=0"); }

$sum=0;
foreach $allocation (@ALLOCATIONS=@QUERYRESULT) {
        ($account,$allockey,$active,$type)=split(/\|/,$allocation);

	# Log the suballocation_removal
	&db("select username,amount,overdraft from suballocation where allockey=$allockey") || return &fail($dberror);
	$balance=0;
	foreach (@SUBALLOCATION=@QUERYRESULT)
		{
		($user,$amount,$overdraft)=split(/\|/,$_);
		# We want to record the amount owed if this is a credit allocation
		$removeamt=$amount-$overdraft;
		&db("insert into txnlog (txntype,key,accountname,username,amount,timestamp,authorization,active,reason,type) values ('remove_suballocation','$allockey','$account','$user',$removeamt,'now','$me','$active','$ARGS{REASON}','$type')") || return &fail($dberror);
		$balance+=$removeamt;
		}
	$sum+=$balance;

	# Delete allocation and associated suballocations
	&db("delete from suballocation where allockey=$allockey") || return &fail($dberror);
	&db("delete from allocation where allockey=$allockey") || return &fail($dberror);

	# Log the txn
	&db("insert into txnlog (txntype,key,accountname,amount,timestamp,authorization,active,reason,type) values ('remove_allocation','$allockey','$account',$balance,'now','$me','$active','$ARGS{REASON}','$type')") || return &fail($dberror);
	}

return &success("Successfully removed allocation(s) with a balance of $balance credits");
}

=pod

=head1 remove_allocation

=over 4

=item B<NAME>

remove_allocation - removes allocations.

=item B<SYNOPSIS>

remove_allocation B<KEY>=I<allockey> | B<ACTIVE>=0 [B<REASON>=I<"reason">]

=item B<DESCRIPTION>

Logs how many cycles were lost when the allocation was
removed. Expiration philosophy is to remove them only on fiscal
boundaries. Logs whether or not the allocation was expired when it was
removed. Removes only the specified allocation if allockey specified
Removes all inactive allocations if B<ACTIVE=I<0>> is specified.
Obtains and records balance deleted for the transaction records.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating balance removed.

=back

=cut


##############################################################
# update_allocations
# sets/unsets active flag and logs any allocations which have since (un)expired
# Returns the number of allocations credits expired
##############################################################
sub update_allocations {
local($sum=0,$balance,@EXPIRATIONS,@UNEXPIRATIONS);

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND}"); }

# Deactivate Allocation if expired or unstarted
&db("select allockey,accountname,expirationdate,activationdate,active,alloctype from allocation where (activationdate>'now' or expirationdate<'now') and active='t'") || return &fail($dberror);
@EXPIRATIONS=@QUERYRESULT;

foreach (@EXPIRATIONS) {
	($allockey,$account,$expires,$starts,$active,$type)=split('\|',$_);
	&db("select username,amount,overdraft from suballocation where allockey=$allockey") || return &fail($dberror);
	@EXPIREUSERS=@QUERYRESULT;
	$balance=0;
	foreach (@EXPIREUSERS)
		{
		($user,$amount,$overdraft)=split(/\|/,$_);
		# We want to record the amount owed if this is a credit allocation
		$deactivateamt=$amount-$overdraft;
		&db("insert into txnlog (txntype,key,accountname,username,amount,expirationdate,activationdate,active,timestamp,authorization,reason) values ('deactivate_suballocation','$allockey','$account','$user',$deactivateamt,'$expires','$starts','f','now','$me','Allocation Expired or UnStarted')") || return &fail($dberror);
		$balance+=$deactivateamt;
		}
	$sum+=$balance;

	&db("update allocation set active='f' where allockey=$allockey") || return &fail($dberror);
	&db("insert into txnlog (txntype,key,accountname,amount,expirationdate,activationdate,active,timestamp,authorization,reason) values ('deactivate_allocation','$allockey','$account','$balance','$expires','$starts','f','now','$me','Allocation Expired or UnStarted')") || return &fail($dberror);
	}

# Activate Allocation if unexpired or started
&db("select allockey,accountname,expirationdate,activationdate,active,alloctype from allocation where (expirationdate>'now' and activationdate<'now') and active='f'") || return &fail($dberror);
@UNEXPIRATIONS=@QUERYRESULT;

foreach (@UNEXPIRATIONS) {
	($allockey,$account,$expires,$starts,$active,$type)=split('\|',$_);
	&db("select username,amount,overdraft from suballocation where allockey=$allockey") || return &fail($dberror);
	@UNEXPIREUSERS=@QUERYRESULT;
	$balance=0;
	foreach (@UNEXPIREUSERS)
		{
		($user,$amount,$overdraft)=split(/\|/,$_);
		# We want to record the amount owed if this is a credit allocation
		$activateamt=$amount-$overdraft;
		&db("insert into txnlog (txntype,key,accountname,username,amount,expirationdate,activationdate,active,timestamp,authorization,reason) values ('activate_suballocation','$allockey','$account','$user',$deactivateamt,'$expires','$starts','t','now','$me','Allocation Started or UnExpired')") || return &fail($dberror);
		$balance+=$deactivateamt;
		}
	$sum-=$balance;

	&db("update allocation set active='t' where allockey=$allockey") || return &fail($dberror);
	&db("insert into txnlog (txntype,key,accountname,amount,expirationdate,activationdate,active,timestamp,authorization,reason) values ('activate_allocation','$allockey','$account','$balance','$expires','$starts','t','now','$me','Allocation Started or UnExpired')") || return &fail($dberror);
	}

&bank_log(4,"@_");
return &success("Expired $sum credits");
}

=pod

=head1 update_allocations

=over 4

=item B<NAME>

update_allocations - updates allocation status.

=item B<SYNOPSIS>

update_allocations

=item B<DESCRIPTION>

Updates the expiration and activation status for all allocations. This
subroutine is called by other subroutines such as make_withdrawal,
remove_allocation, change_allocation. Logs how many cycles were lost
when the allocation was (de)activated.
Takes into account credit allocations.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns amount expired (nodeseconds)

=back

=cut


##############################################################
# get_allocations [ACCOUNT=<account>] [KEY=<allockey>] [ACTIVE=1|0] 
#                 [MACHINES=<mach1:mach2:...>|ANY] [CHECKPOINT=<ckptkey>] 
#                 [SHOW=<attributelist>] [TYPE=debit|credit]
# If MACHINES specified will look for a match (for <mach1:mach2:...> or ANY)
# in the allowmachines string. If you want ALL allocations,
# leave MACHINES unspecified.
# returns a pipe-delimited arrays of:
#  "accountname,allockey,amount,expirationdate,activationdate,active,allowmachines,alloctype"
##############################################################
sub get_allocations {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local(@ALLOCATIONS=(),@ALLOCINFO=());

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [KEY=<allockey>] [ACTIVE=1|0] [MACHINES=<mach1:mach2:...>|ANY] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>] [TYPE=debit|credit]"); }

if ($ARGS{CHECKPOINT}) { $X="x" } else { $X="" }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="accountname,allockey,amount,expirationdate,activationdate,active,allowmachines,alloctype" };
$ARGS{SHOW}=~s/amount/sum(amount)/;			 # Needed because of group by
$ARGS{SHOW}=~s/allockey/${X}allocation.allockey/; 	 # Needed because of join

$select="select $ARGS{SHOW} from ${X}allocation,${X}suballocation where ${X}allocation.allockey=${X}suballocation.allockey";
if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{KEY}) {$select.=" and ${X}allocation.allockey=$ARGS{KEY}"}
if ($ARGS{ACTIVE} eq "1") {$select.=" and active='t'"}
if ($ARGS{ACTIVE} eq "0") {$select.=" and active='f'"}
if ($ARGS{MACHINES}) {$select.=" and (allowmachines~':$ARGS{MACHINES}:' or allowmachines=':ANY:')"}
if ($ARGS{TYPE}) {$select.=" and alloctype='$ARGS{TYPE}'"}
if ($ARGS{CHECKPOINT}) {$select.=" and ${X}allocation.checkpointkey=$ARGS{CHECKPOINT} and ${X}suballocation.checkpointkey=$ARGS{CHECKPOINT}"}

@GROUP_BY=grep(!/sum\(amount\)/&&!/${X}allocation.allockey/,split(/,/,$ARGS{SHOW}));
$select.=join(','," group by ${X}allocation.allockey",@GROUP_BY);

&db($select) || return &fail($dberror);

return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_allocations

=over 4

=item B<NAME>

get_allocations - Display allocation information

=item B<SYNOPSIS>

get_allocations [B<ACCOUNT>=I<account>] [B<KEY>=I<allockey>] [B<ACTIVE>=I<1|0>]
[B<MACHINES>=I<mach1:mach2:...|ANY>] [B<CHECKPOINT>=I<ckptkey>]
[B<SHOW>=I<attributelist>] [B<TYPE>=debit|credit]

=item B<DESCRIPTION>

Dumps a line for each allocation specified Primarily used within
administrative tools to provide account information Information
accessible depends on one's administrative priveledges. Bank admins
can access global allocation information, account admins can access
allocation information for their administrative accounts, and users
can access allocation information about the subaccounts to which they
pertain. Specifying B<ACTIVE=I<1>> will return only currently active
allocations (excluding Allocations which have not yet started or which
have already expired). The default is to return all Allocations
(Active or Inactive).

If B<MACHINES> is specified, a match will be searched from
I<mach1:mach2:...|ANY>

=item B<OPTIONS>

B<TYPE> can be either debit or credit.

=item B<RETURN VALUES>

Returns pipe-delimited arrays of "accountname, allockey, balance,
expirationdate, activationdate, active, allowmachines" unless
overridden by the SHOW parameter.

=back

=cut


##############################################################
# make_quotation [USER=<user>] [ACCOUNT=<account>] [WCTIME=<wallclockseconds>] [PROCS=<numprocessors>] [MACHINE=<machine>] [NODES=<numnodes>] [MEMORY=<memory>] [DISK=<disk>] [CLASS=<class>] [QOS=<qos>] [NODETYPE=<nodetype>] [JOBTYPE=<jobtype>] [REASON="<reason>"]
# Makes a cost quote - verifies user has sufficient balance
# Make_quotation actually guarantees a nodehour rate based on the
# projected wallclock time. The quote will expire after
# WCTIME+GRACETIME seconds and a job will have to complete
# before that time in order for QBank to honor the quote.
# This is to support a metascheduling environment where it may be useful
# to determine how much it will "cost" to run a particular job with
# the specified requirements. Make_quotation returns a quote_id which
# could be specified with the subsequent reservation and withdrawal to
# secure a prorated charge based on the quotation parameters.
# Although the quote mechanism can be used to guarantee a rate, there is
# substantial potential for abuse. If a user makes the quotation using a cheap
# QOS, MACHINE or any other attribute, there is nothing preventing them from
# laying claim to the guaranteed quotation rate while obtaining a more
# favorable QOS, NODETYPE etc. Since charge attributes are arbitrary
# (site specific), there is no easy way to make sure the quotation is only
# secured for a job with the same specified attributes.
# I would recommend using quotations to get an appraisal for what it would
# cost at the time of the quote but have the scheduler not use the QUOTEID
# to lock in the rate, but allow the actual charge to vary from the quoted
# rate due to dynamic charge rates such as BACKLOG or PRIMETIME charges.
# Returns cost of specified job and the quotation identifier
##############################################################
sub make_quotation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($nodeseconds,$useravail,$kittyavail,$useramt,$kittyamt);

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [USER=<user>] [ACCOUNT=<account>] [WCTIME=<seconds>] [PROCS=<numprocessors>] [MACHINE=<machine>] [NODES=<numnodes>] [MEMORY=<memory>] [DISK=<disk>] [CLASS=<class>] [QOS=<qos>] [NODETYPE=<nodetype>] [JOBTYPE=<jobtype>] [REASON=\"<reason>\"]"); }

# Use UNKNOWN user if no USER specified
unless ($ARGS{USER}) { $ARGS{USER}='UNKNOWN'; }

# Try to use default account if no ACCOUNT specified
unless ($ARGS{ACCOUNT}) {
        &db("select defaultaccount from userinfo where username='$ARGS{USER}'") || return &fail($dberror);
        if ($QUERYRESULT[0] ne '') { $ARGS{ACCOUNT}=$QUERYRESULT[0]; }
        else { return &deny("No Default Account can be found for $ARGS{USER}") }
        }

# Verify account active
&db("select accountactive from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
unless ($QUERYRESULT[0])
	{return &deny("$ARGS{ACCOUNT} is not an active account.");}

# Verify user enabled for account
&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and (userstatus='enabled' or userstatus='admin')") || return &fail($dberror);
$enabled=$ROWS;
#unless ($ROWS)
#	{return &deny("$ARGS{USER} is not enabled to charge to account $ARGS{ACCOUNT}");}

# Set machine to system default if unspecified
if ($ARGS{MACHINE}) { $machine=$ARGS{MACHINE}; }
else { $machine=$MACHINE; }

# Factor in charge multipliers when computing quotation amount
# This is done by calling the external charge subroutine from charge.ph
$quoteamt=charge();
if ($quoteamt eq "") { return &deny($QBank::Result::ARGS{MESSAGE}); }

# Fail if reservamt is negative
if ($quoteamt<0) { return &deny("Reservation Amount CANNOT be Negative"); }

#
# Obtain Available balances for user and KITTY
# This includes available credit balances
#

# User portion
if ($ARGS{USER} ne 'KITTY' || $ARGS{USER} ne 'ANY' || $ARGS{USER} ne 'UNKNOWN' || $ARGS{USER} ne 'RESERVE') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($userbalance)=@QUERYRESULT;

	&db("select sum(amount) from subreservation,reservation where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and subreservation.jobid=reservation.jobid and expirationdate>'now' and machine='$machine'") || return &fail($dberror);
	($userpending)=@QUERYRESULT;

	$useravail=1*($userbalance-$userpending);
	}

# KITTY portion
if ( $enabled || $ARGS{USER} eq 'KITTY') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='KITTY' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($kittybalance)=@QUERYRESULT;

	&db("select sum(amount) from subreservation,reservation where accountname='$ARGS{ACCOUNT}' and username='KITTY' and subreservation.jobid=reservation.jobid and expirationdate>'now' and machine='$machine'") || return &fail($dberror);
($kittypending)=@QUERYRESULT;

	$kittyavail=1*($kittybalance-$kittypending);
	}

# ANY portion
if ($ARGS{USER} ne 'RESERVE') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='ANY' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($anybalance)=@QUERYRESULT;

	&db("select sum(amount) from subreservation,reservation where accountname='$ARGS{ACCOUNT}' and username='ANY' and subreservation.jobid=reservation.jobid and expirationdate>'now' and machine='$machine'") || return &fail($dberror);
($anypending)=@QUERYRESULT;

	$anyavail=1*($anybalance-$anypending);
	}

# Bail out if user has insufficient funds
if (($useravail+$kittyavail+$anyavail)<$quoteamt)
	{ return &fail("$ARGS{USER} does not have sufficient funds for job $ARGS{JOBID}"); }

# Create unique quoteid by appending account, user and epoch time
$QUOTEID="$ARGS{ACCOUNT}-$ARGS{USER}-".time();

# The grace time is how long the submitter has to secure the quote
# and get his job running. The quote will expire after
# WALLCLOCKLIMIT+GRACETIME seconds and a job will have to complete
# before that time in order for QBank to honor the quote.
$GRACETIME=86400; # 1 day grace time

# Format time to a database acceptable format
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time+$ARGS{WCTIME}+$GRACETIME);
$etime=sprintf("%02d/%02d/%02d %02d:%02d:%02d",($mon+1),$mday,$year%100,$hour,$min,$sec);

# Calculate chargerate to store in quotation in order to prorate actual debit
$CHARGERATE=$quoteamt/($ARGS{PROCS}*$ARGS{WCTIME});

# Create the quotation with the quoteid as the unique identifier
&db("insert into quotation (quoteid,rate,accountname,username,expirationdate,creationdate,machine) values ('$QUOTEID',$CHARGERATE,'$ARGS{ACCOUNT}','$ARGS{USER}','$etime','now','$machine')") || return &fail($dberror);

# Log the overall quotation in the txnlog
$NAMES="txntype,accountname,username,key,amount,processors,wallclock,timestamp,authorization,nodetype,qos,machine,class,type,expirationdate,reason";
$VALUES="'make_quotation','$ARGS{ACCOUNT}','$ARGS{USER}','$QUOTEID',$quoteamt,$ARGS{PROCS},$ARGS{WCTIME},'now','$me','$ARGS{NODETYPE}','$ARGS{QOS}','$machine','$ARGS{CLASS}','$ARGS{JOBTYPE}','$etime','$ARGS{REASON}'";
if ($ARGS{MEMORY}) { $NAMES.=",memory"; $VALUES.=",$ARGS{MEMORY}"; }
if ($ARGS{DISK}) { $NAMES.=",disk"; $VALUES.=",$ARGS{DISK}"; }
if ($ARGS{NODES}) { $NAMES.=",nodes"; $VALUES.=",$ARGS{NODES}"; }
&db("insert into txnlog ($NAMES) values ($VALUES)") || return &fail($dberror);
&bank_log(4,"@_");

return &success("$quoteamt|$QUOTEID");
}

=pod

=head1 make_quotation

=over 4

=item B<NAME>

make_quotation - makes a charge quote

=item B<SYNOPSIS>

make_quotation [B<USER>=I<user>] [B<ACCOUNT>=I<account>] [B<WCTIME>=I<seconds>]
[B<PROCS>=I<numprocessors>] [B<MACHINE>=I<machine>]
[B<MEMORY>=I<memory>] [B<NODES>=I<numnodes>] [B<DISK>=I<disk>]
[B<CLASS>=I<class>] [B<QOS>=I<qos>] [B<NODETYPE>=I<nodetype>]
[B<JOBTYPE>=I<jobtype>] [B<REASON>="I<reason>"]

=item B<DESCRIPTION>

Generates a quote for how much would be charged to run under the specified
set of circumstances - currently will verify user has sufficient
balance. The account must be active. 
If B<QOS> or B<NODETYPE> are specified, the bank will do a
lookup on B<%QOSTYPES> and B<%NODETYPES> respectively to obtain the
appropriate charge multipiers for computing the quote amount. If
the specified B<QOS> or B<NODETYPE> are not defined, the quote
will fail. If MACHINE is specified, the bank will make sure the
withdrawal request was issued on behalf of a machine allowed to debit
to the account. Tries to use default account if no B<ACCOUNT> specified.
The quote will be logged to the transactionlog for verification purposes.
If B<USER> is not specified, B<USER> will be mapped to I<UNKNOWN>
and make_quotation will verify that enough credits exist in the
ANY subaccount to fulfill a corresponding withdrawal.

This is to support a metascheduling environment where it may be useful
to determine how much it will "cost" to run a particular job with
the specified requirements. Make_quotation returns a quote_id which
could be specified with the subsequent reservation and withdrawal to
secure a prorated charge based on the quotation parameters.
Although the quote mechanism can be used to guarantee a rate, there is
substantial potential for abuse. If a user makes the quotation using a cheap
QOS, MACHINE or any other attribute, there is nothing preventing them from
laying claim to the guaranteed quotation rate while obtaining a more
favorable QOS, NODETYPE etc. Since charge attributes are arbitrary
(site specific), there is no easy way to make sure the quotation is only
secured for a job with the same specified attributes.
I would recommend using quotations to get an appraisal for what it would
cost at the time of the quote but have the scheduler not use the QUOTEID
to lock in the rate, but allow the actual charge to vary from the quoted
rate due to dynamic charge rates such as BACKLOG or PRIMETIME charges.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a job charge rate quote and a quotation identifier

=back

=cut


##############################################################
# remove_quotation QUOTEID=<quoteid> | EXPIRED=1
# Removes only the specified quotation if quotid specified
# Removes all expired quotations if EXPIRED=1 is specified
# Returns message indicating amount of nodeseconds restored
##############################################################
sub remove_quotation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($count=0);

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||(!$ARGS{QUOTEID}&&!$ARGS{EXPIRED}))
        { return &usage("$ARGS{COMMAND} JOBID=<jobid> | EXPIRED=1"); }

# Post to reconcile log if the database server is not responding
unless ($dbh->ping) {
	&post_txn("@_");
	return &success("0\nWarning: $DBI::errstr\nTransaction has been logged to the reconcile log");
	}

if ($ARGS{QUOTEID}) {
	&db("select accountname,quoteid from quotation where quoteid='$ARGS{QUOTEID}'") || return &fail($dberror);
	}
elsif ($ARGS{EXPIRED} eq "1") {
	&db("select accountname,quoteid,rate from quotation where expirationdate<'now'") || return &fail($dberror);
	}
else { return &usage("$ARGS{COMMAND} QUOTEID=<quoteid> | EXPIRED=1"); }

foreach $quotation (@QUOTATIONS=@QUERYRESULT) {
	($account,$quoteid,$rate)=split(/\|/,$quotation);

	$count+=1;

	&db("delete from quotation where quoteid='$quoteid'") || return &fail($dberror);

	# Log in txnlog iff a reservation was removed
	if ($ROWS) {
		&db("insert into txnlog (txntype,accountname,key,amount,timestamp,authorization) values ('remove_quotation','$account','$quoteid',$rate,'now','$me')") || return &fail($dberror);
		}
	}

return &success("Successfully removed $count quotations");
}

=pod

=head1 remove_quotation

=over 4

=item B<NAME>

remove_quotation - removes quotations

=item B<SYNOPSIS>

remove_quotation B<QUOTEID>=I<quoteid>|B<EXPIRED>=I<1> 

=item B<DESCRIPTION>

Removes only the specified quotation if B<QUOTEID> is specified. Removes
all expired quotations if B<EXPIRED=I<1>> is specified.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns message indicating number of quotations removed.

=back

=cut


##############################################################
# get_quotations [ACCOUNT=<account>] [USER=<user>] [QUOTEID=<quoteid>]
# [EXPIRED=1|0] [MACHINE=<machine>] [CHECKPOINT=<ckptkey>]
# [SHOW=<attributelist>]
# returns pipe-delimited arrays of "accountname,username,quoteid,rate,expirationdate,machine"
##############################################################
sub get_quotations {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local(@QUOTATIONS=(),@QUOTEINFO=());

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [USER=<user>] [QUOTEID=<quoteid>] [EXPIRED=1|0] [MACHINE=<machine>] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]"); }

if ($ARGS{CHECKPOINT}) { $X="x" } else { $X="" }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="accountname,username,quoteid,rate,expirationdate,machine" };

$select="select $ARGS{SHOW} from ${X}quotation where 1=1";
if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{USER}) {$select.=" and username='$ARGS{USER}'"}
if ($ARGS{QUOTEID}) {$select.=" and quoteid='$ARGS{QUOTEID}'"}
if ($ARGS{EXPIRED} eq "1") {$select.=" and expirationdate<'now'"}
if ($ARGS{EXPIRED} eq "0") {$select.=" and expirationdate>='now'"}
if ($ARGS{MACHINE}) {$select.=" and machine='$ARGS{MACHINE}'"}
if ($ARGS{CHECKPOINT}) {$select.=" and ${X}reservation.checkpointkey=$ARGS{CHECKPOINT} and ${X}subreservation.checkpointkey=$ARGS{CHECKPOINT}"}

&db($select) || return &fail($dberror);

return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_quotations

=over 4

=item B<NAME>

get_quotations - gets quotation information.

=item B<SYNOPSIS>

get_quotations [B<ACCOUNT>=I<account>] [B<USER>=I<user>]
[B<QUOTEID>=I<quoteid>] [B<EXPIRED>=I<1|0>] [B<MACHINE>=I<machine>]
[B<CHECKPOINT>=I<ckptkey>] [B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

Dumps a line for each quotation specified.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns pipe-delimited array of "accountname, username, quoteid, rate,
expirationdate, machine"

=back

=cut


##############################################################
# make_reservation [ACCOUNT=<account>] [USER=<user>] [WCTIME=<wallclockseconds> PROCS=<numprocessors>] JOBID=<jobid> [EXPIRES=<MM/DD[/YY][_hh:mm:ss]>] [MACHINE=<machine>] [NODES=<nodenum>] [MEMORY=<memory>] [DISK=<disk>] [CLASS=<class>] [QOS=<qos>] [NODETYPE=<nodetype>] [JOBTYPE=<jobtype>] [REASON="<reason>"] [QUOTEID=<quoteid>] [TRACEBACKHOST=<host>] [TRACEBACKPORT=<port>] [TRACEBACKACCT=<account>]  [TRACEBACKUSER=<user>] [AMOUNT=<amount>]
# Makes a pending withdrawal - must verify user has sufficient balance
# Logs transaction to flatfile if database server is not responding
# Tries to use default account if no ACCOUNT specified
# If USER is not specified, will try to find funds in ANY subaccount.
# If QUOTEID is specified, will use the quoted rate if not expired
# If traceback information is supplied and target account is a not of type
# local, a traceback reservation will first be attempted.
# At some point, if the local reservation fails we should try
# to remove the traceback and forwarding reservations if they were made.
# It is expected that AMOUNT will not normally be explicitly specified
# except in the case of forwarding exchange accounts. If AMOUNT is
# not specified, QBank will invoke the charge function to calculate the
# amount based on the specified parameters (PROCS, WCTIME, QOS, etc.)
# Maps USER to UNKNOWN if no USER specified.
# Will debit maximally from USER, KITTY, then ANY where possible
# in order to satisfy withdrawal request -- in that order.
# Returns amount reserved
##############################################################
sub make_reservation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($nodeseconds,$useravail,$kittyavail,$useramt,$kittyamt);
if ($ARGS{PROCCOUNT}) { $ARGS{PROCS}=$ARGS{PROCCOUNT}; } # Pre-2.9 compatibility
if ($ARGS{WCLIMIT}) { $ARGS{WCTIME}=$ARGS{WCLIMIT}; } # Pre-2.9 compatibility
@RESERVEMESSAGES=();

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||(!$ARGS{AMOUNT}&&(!$ARGS{WCTIME}||!$ARGS{PROCS}))||!$ARGS{JOBID})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [USER=<user>] [WCTIME=<seconds> PROCS=<numprocessors>] JOBID=<jobid> [EXPIRES=<MM/DD[/YY][_hh:mm:ss]>] [MACHINE=<machine>] [NODES=<nodenum>] [MEMORY=<memory>] [DISK=<disk>] [CLASS=<class>] [QOS=<qos>] [NODETYPE=<nodetype>] [JOBTYPE=<jobtype>] [REASON=\"<reason>\"] [QUOTEID=<quoteid>]  [TRACEBACKHOST=<host>] [TRACEBACKPORT=<port>]  [TRACEBACKACCT=<account>] [TRACEBACKUSER=<user>] [AMOUNT=<amount>]"); }

# Post to reconcile log if the database server is not responding
unless ($dbh->ping) {
	&post_txn("@_");
	return &success("0\nWarning: $DBI::errstr\nTransaction has been logged to the reconcile log");
	}

# Use UNKNOWN if no USER specified
unless ($ARGS{USER}) { $ARGS{USER}='UNKNOWN'; }

# Try to use default account if no ACCOUNT specified
unless ($ARGS{ACCOUNT}) {
        &db("select defaultaccount from userinfo where username='$ARGS{USER}'") || return &fail($dberror);
        if ($QUERYRESULT[0] ne '') { $ARGS{ACCOUNT}=$QUERYRESULT[0]; }
        else { return &deny("No Default Account can be found for $ARGS{USER}") }
        }

# Verify account active
&db("select accountactive from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
unless ($QUERYRESULT[0])
	{return &deny("$ARGS{ACCOUNT} is not an active account.");}

# Verify user enabled for account
&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and (userstatus='enabled' or userstatus='admin')") || return &fail($dberror);
$enabled=$ROWS;
#unless ($ROWS)
#	{return &deny("$ARGS{USER} is not enabled to charge to account $ARGS{ACCOUNT}");}

# Format expiration time to an acceptable database format
if ($ARGS{EXPIRES}) {
	$_=$ARGS{EXPIRES};	# Parse time expression
	($month,$day,$slash,$year,$underscore,$time,$hour,$minute,$second)=/(\d\d)\/(\d\d)(\/(\d\d))?(_((\d\d):(\d\d):(\d\d)))?/;
	# Generate usage if incorrect
   unless ($month) { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] USER=<user> WCTIME=<seconds> PROCS=<numprocessors> JOBID=<jobid> [EXPIRES=<MM/DD[/YY][_hh:mm:ss]>] [MACHINE=<machine>] [NODES=<nodenum>] [MEMORY=<memory>] [DISK=<disk>] [CLASS=<class>] [QOS=<qos>] [NODETYPE=<nodetype>] [JOBTYPE=<jobtype>] [REASON=\"<reason>\"]"); }
	# Format time to a database acceptable format
	$etime="$month/$day/".($year?$year:(localtime)[5]).($time?" $time":"");
	}
else	{ # If no expirationdate supplied, expire after wallclocklimit
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time+$ARGS{WCTIME});
	$etime=sprintf("%02d/%02d/%02d %02d:%02d:%02d",($mon+1),$mday,$year%100,$hour,$min,$sec);
	}

# Set machine to system default if unspecified
if ($ARGS{MACHINE}) { $machine=$ARGS{MACHINE}; }
else { $machine=$MACHINE; }

# If QUOTEID is specified, will use the quoted rate if not expired
if ($ARGS{QUOTEID}) {
	&db("select rate from quotation where quoteid='$ARGS{QUOTEID}' and expirationdate>='now'") || return &fail($dberror);
	if ($ROWS) { $CHARGERATE=$QUERYRESULT[0]; }
}
if ($CHARGERATE) {
	$reservamt=$ARGS{PROCS}*$ARGS{WCTIME}*$CHARGERATE;
	$ARGS{REASON}.=" Using QUOTEID $ARGS{QUOTEID} at rate $CHARGERATE";
}
# Else if AMOUNT is specified use it else calculate it with charge function
elsif ($ARGS{AMOUNT}) { $reservamt=$ARGS{AMOUNT}; }
# Otherwise, factor in charge multipliers when computing withdrawal amount
# This is done by calling the external charge subroutine from charge.ph
else { $reservamt=charge(); }

# Fail if reservamt is NULL
if ($reservamt eq "") { return &deny($QBank::Result::ARGS{MESSAGE}); }

# Fail if reservamt is negative
if ($reservamt<0) { return &deny("Reservation Amount CANNOT be Negative");}

# Check for account type to see if remote exchange is necessary
&db("select accounttype,exchangehost,exchangeport,exchangeacct,exchangeuser,exchangerate from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
($accttype,$xhost,$xport,$xacct,$xuser,$xrate)=split(/\|/,$QUERYRESULT[0]);

# Debit traceback account if type is not local and TRACEBACK arguments specified
if (($accttype eq "meta" || $accttype eq "forwarding" || $accttype eq "seesaw") && $ARGS{TRACEBACKACCT} && $ARGS{TRACEBACKHOST} && $ARGS{TRACEBACKPORT})
{
	unless ($ARGS{TRACEBACKUSER}) { $ARGS{TRACEBACKUSER}=$ARGS{USER}; }
	@QUERYRESULT=&qbank($ARGS{TRACEBACKHOST},$ARGS{TRACEBACKPORT},"make_reservation ACCOUNT=$ARGS{TRACEBACKACCT} USER=$ARGS{TRACEBACKUSER} AMOUNT=" . $reservamt*$xrate . " REASON=\"Traceback reservation from $ARGS{ACCOUNT}\@${machine}\" PROCS=$ARGS{PROCS} WCTIME=$ARGS{WCTIME} JOBID=$ARGS{JOBID} MACHINE=$ARGS{ACCOUNT}\@${machine}");
	$tracebackstatus=$STATUS;
	if (!$tracebackstatus) {
		return &deny("Reservation aborted: Failed reserving Traceback subaccount $ARGS{TRACEBACKUSER}\@$ARGS{TRACEBACKACCT} at $ARGS{TRACEBACKHOST}:$ARGS{TRACEBACKPORT} for ".($reservamt*$xrate).". Reason: @QUERYRESULT");
	}
	else {
		push(@RESERVEMESSAGES,"\nTraceback account @QUERYRESULT");
		&db("insert into txnlog (txntype,accountname,username,amount,timestamp,authorization,machine,key,processors,wallclock,reason) values ('remote_reservation','$ARGS{TRACEBACKACCT}','$ARGS{TRACEBACKUSER}',".($reservamt*$xrate).",'now','$me','$machine','$ARGS{JOBID}',$ARGS{PROCS},$ARGS{WCTIME},'Remote reservation to $ARGS{TRACEBACKACCT} at $ARGS{TRACEBACKHOST}:$ARGS{TRACEBACKPORT}')") || return &fail($dberror);
		}
	}

# Debit exchange account if account type is forwarding
if ($accttype eq "forwarding")
{
	# If an exchangeuser is not defined then use USER
	unless ($xuser) { $xuser=$ARGS{USER}; }
	@QUERYRESULT=&qbank($xhost,$xport,"make_reservation USER=$xuser ACCOUNT=$xacct AMOUNT=$reservamt REASON=\"Forwarding reservation from $ARGS{ACCOUNT}\@${machine}\" PROCS=$ARGS{PROCS} WCTIME=$ARGS{WCTIME} JOBID=$ARGS{JOBID} MACHINE=$machine");
	$forwardingstatus=$STATUS;
	if (!$forwardingstatus) {
		# We should try to remove the traceback reservation if one was made
		if ($tracebackstatus) {
			&qbank($ARGS{TRACEBACKHOST},$ARGS{TRACEBACKPORT},"remove_reservation JOBID=$ARGS{JOBID}");
		}
		return &deny("Reservation aborted: Failed reserving Forwarding subaccount $xuser\@$xacct at $xhost:$xport for $reservamt. Reason: @QUERYRESULT");
	}
	else {
		push(@RESERVEMESSAGES,"\nForwarding account @QUERYRESULT");
		&db("insert into txnlog (txntype,accountname,username,amount,timestamp,authorization,machine,key,processors,wallclock,reason) values ('remote_withdrawal','$ARGS{ACCOUNT}','$xuser',$reservamt,'now','$me','$machine','$ARGS{JOBID}',$ARGS{PROCS},$ARGS{WCTIME},'Reserved $xacct at $xhost:$xport')") || return &fail($dberror);
		}
	}

# Lock subreservations before other reservation attempts check for sufficient
# funds. If a failure occurs anywhere between here and the COMMIT, the fail
# routine will perform a rollback. At some point, upon failure we should try
# to remove the traceback and forwarding reservations if they were made.
$WORK=1;
&db("lock table subreservation");

# Remove old reservation by same name if it exists
# Important for this operation to be repeatable successfully for retries
# If it is going to reissue the reservation, it must have updated expiration

&db("select jobid from reservation where jobid='$ARGS{JOBID}'") || return &fail($dberror);
if ($ROWS) {
	&db("select accountname,jobid from reservation where jobid='$ARGS{JOBID}'") || return &fail($dberror);
	foreach $reservation (@RESERVATIONS=@QUERYRESULT) {
		($account,$jobid)=split(/\|/,$reservation);

		&db("select sum(amount) from subreservation where jobid='$jobid'") || return &fail($dberror);
		($balance)=@QUERYRESULT;
		$balance*=1;
		$sum+=$balance;

		&db("delete from subreservation where jobid='$jobid'") || return &fail($dberror);
		&db("delete from reservation where jobid='$jobid'") || return &fail($dberror);

		# Log in txnlog iff a reservation was removed
		if ($ROWS) {
			&db("insert into txnlog (txntype,accountname,key,amount,timestamp,authorization) values ('remove_reservation','$account','$jobid',$balance,'now','$me')") || return &fail($dberror);
			}
		}
	}

#
# Obtain Available balances for user and KITTY
# This includes available credit balances
#

# User portion
if ($ARGS{USER} ne 'KITTY' || $ARGS{USER} ne 'ANY' || $ARGS{USER} ne 'UNKNOWN' || $ARGS{USER} ne 'RESERVE') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($userbalance)=@QUERYRESULT;

	&db("select sum(amount) from subreservation,reservation where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and subreservation.jobid=reservation.jobid and expirationdate>'now' and machine='$machine'") || return &fail($dberror);
	($userpending)=@QUERYRESULT;

	$useravail=1*($userbalance-$userpending);
	}

# KITTY portion
if ( $enabled || $ARGS{USER} eq 'KITTY') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='KITTY' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($kittybalance)=@QUERYRESULT;

	&db("select sum(amount) from subreservation,reservation where accountname='$ARGS{ACCOUNT}' and username='KITTY' and subreservation.jobid=reservation.jobid and expirationdate>'now' and machine='$machine'") || return &fail($dberror);
($kittypending)=@QUERYRESULT;

	$kittyavail=1*($kittybalance-$kittypending);
	}

# ANY portion
if ($ARGS{USER} ne 'RESERVE') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='ANY' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($anybalance)=@QUERYRESULT;

	&db("select sum(amount) from subreservation,reservation where accountname='$ARGS{ACCOUNT}' and username='ANY' and subreservation.jobid=reservation.jobid and expirationdate>'now' and machine='$machine'") || return &fail($dberror);
($anypending)=@QUERYRESULT;

	$anyavail=1*($anybalance-$anypending);
	}

# Bail out if user has insufficient funds
if (($useravail+$kittyavail+$anyavail)<$reservamt)
	{ return &fail("$ARGS{USER} does not have sufficient funds for job $ARGS{JOBID}"); }

# Debit as much as possible from user, KITTY then ANY
$useramt=$kittyamt=$anyamt=0;
$remaining=$reservamt;

if ($remaining > 0) {
   if ($remaining<$userbalance) { $useramt=$remaining }
   else { $useramt=$userbalance }
   $remaining-=$useramt;
   }

if ($remaining > 0) {
   if ($remaining<$kittybalance) { $kittyamt=$remaining }
   else { $kittyamt=$kittybalance }
   $remaining-=$kittyamt;
   }

if ($remaining > 0) {
   if ($remaining<$anybalance) { $anyamt=$remaining }
   else { $anyamt=$anybalance }
   $remaining-=$anyamt;
   }

if ($remaining != 0) { return &fail("Something went wrong: Reservation amount was not correctly divided among subaccounts"); }

# Create the reservation with the jobid as the unique identifier
&db("insert into reservation (accountname,jobid,expirationdate,creationdate,machine) values ('$ARGS{ACCOUNT}','$ARGS{JOBID}','$etime','now','$machine')") || return &fail($dberror);

# Create subreservations for user and KITTY as needed
if ($useramt) {
	&db("insert into subreservation (jobid,username,amount) values ('$ARGS{JOBID}','$ARGS{USER}',$useramt)");
	# Log the user subreservation in the txnlog
	&db("insert into txnlog (txntype,accountname,username,key,amount,expirationdate,timestamp,authorization,nodetype,qos,machine) values ('make_subreservation','$ARGS{ACCOUNT}','$ARGS{USER}','$ARGS{JOBID}',$useramt,'$etime','now','$me','$ARGS{NODETYPE}','$ARGS{QOS}','$machine')") || return &fail($dberror);
	}
if ($kittyamt) {
	&db("insert into subreservation (jobid,username,amount) values ('$ARGS{JOBID}','KITTY',$kittyamt)");
	# Log the KITTY subreservation in the txnlog
	&db("insert into txnlog (txntype,accountname,username,key,amount,expirationdate,timestamp,authorization,nodetype,qos,machine) values ('make_subreservation','$ARGS{ACCOUNT}','KITTY','$ARGS{JOBID}',$kittyamt,'$etime','now','$me','$ARGS{NODETYPE}','$ARGS{QOS}','$machine')") || return &fail($dberror);
	}
if ($anyamt) {
	&db("insert into subreservation (jobid,username,amount) values ('$ARGS{JOBID}','ANY',$anyamt)");
	# Log the ANY subreservation in the txnlog
	&db("insert into txnlog (txntype,accountname,username,key,amount,expirationdate,timestamp,authorization,nodetype,qos,machine) values ('make_subreservation','$ARGS{ACCOUNT}','ANY','$ARGS{JOBID}',$anyamt,'$etime','now','$me','$ARGS{NODETYPE}','$ARGS{QOS}','$machine')") || return &fail($dberror);
	}
		
# Log the overall reservation in the txnlog
$NAMES="txntype,accountname,username,key,amount,processors,wallclock,timestamp,authorization,nodetype,qos,machine,class,type,reason";
$VALUES="'make_reservation','$ARGS{ACCOUNT}','$ARGS{USER}','$ARGS{JOBID}',$reservamt,$ARGS{PROCS},$ARGS{WCTIME},'now','$me','$ARGS{NODETYPE}','$ARGS{QOS}','$machine','$ARGS{CLASS}','$ARGS{JOBTYPE}','$ARGS{REASON}'";
if ($ARGS{MEMORY}) { $NAMES.=",memory"; $VALUES.=",$ARGS{MEMORY}"; }
if ($ARGS{DISK}) { $NAMES.=",disk"; $VALUES.=",$ARGS{DISK}"; }
if ($ARGS{NODES}) { $NAMES.=",nodes"; $VALUES.=",$ARGS{NODES}"; }
&db("insert into txnlog ($NAMES) values ($VALUES)") || return &fail($dberror);
&bank_log(4,"@_");

# Commit the transaction
undef $WORK;
$dbh->commit;

return &success("Successfully reserved $reservamt credits@RESERVEMESSAGES");
}

=pod

=head1 make_reservation

=over 4

=item B<NAME>

make_reservation - makes pending withdrawl

=item B<SYNOPSIS>

make_reservation [B<ACCOUNT>=I<account>] [B<USER>=I<user>] [B<WCTIME>=I<seconds> B<PROCS>=I<numprocessors>] B<JOBID>=I<jobid> [B<EXPIRES>=I<MM/DD[/YY][_hh:mm:ss]>] [B<MACHINE>=I<machine>] [B<MEMORY>=I<memory>] [B<DISK>=I<disk>] [B<CLASS>=I<class>] [B<QOS>=I<qos>] [B<NODETYPE>=I<nodetype>] [B<JOBTYPE>=I<jobtype>] [B<REASON>="I<reason>"] [B<QUOTEID>=I<quoteid>] [B<TRACEBACKHOST>=I<host>] [B<TRACEBACKPORT>=I<port>] [B<TRACEBACKACCT>=I<account>] [[B<TRACEBACKUSER>=I<user>] [B<AMOUNT>=I<amount>]

=item B<DESCRIPTION>

Makes a pending withdrawal - must verify user has sufficient
balance. The account must be active. Reservation amount cannot be
negative.

Logs transaction to flatfile if database server is not
responding. Remove old reservation by same name if it exists. This is
important for this operation to be repeatable successfully for
retries. If the reservation has to be reissued, it must have updated
expirationdate. This procedure must make subreservations against the
user subaccount first with any overflow reserved against the B<KITTY>
pool. Maps B<USER> to UNKNOWN if no B<USER> specified.
Will reserve maximally from B<USER>, I<KITTY>, then I<ANY> where possible
in order to satisfy withdrawal request -- in that order.

=item B<OPTIONS>

Either B<AMOUNT> must be specified or enough qualifiers to appropriately
determine a charge (currently B<WCTIME> and B<PROCS> are required.)
It is expected that B<AMOUNT> will not normally be explicitly specified
except in the case of I<forwarding> exchange accounts. If B<AMOUNT> is
not specified, QBank will invoke the charge function to calculate the
amount based on the specified parameters (PROCS, WCTIME, QOS, etc.)

If B<USER> is not specified, B<USER> will be mapped to I<UNKNOWN> and
make_reservation will try to reserve funds under the I<ANY> subaccount.

If B<QOS> or B<NODETYPE> are specified, the bank will do a
lookup on B<%QOSTYPES> and B<%NODETYPES> respectively to obtain the
appropriate charge multipiers for computing the reservation amount. If
the specified B<QOS> or B<NODETYPE> are not defined, the reservation
will fail.

If B<MACHINE> is specified, the bank will make sure the withdrawal request
was issued on behalf of a machine allowed to debit to the account.

If B<QUOTEID> is specified, the rate guaranteed by the quote will be applied
and prorated according to job procs and wallclock time used, so long as the
job has completed before the expirationdate of the quote.
If the quote has expired, the regular charging algorithm will be used.

Tries to use default account if no B<ACCOUNT> specified.

Reservation will automatically expire at specified expirationdate or
after B<WCTIME> has elapsed if B<EXPIRES> is left unspecified.

TRACEBACKACCT	specifies the account local to the submitting user that is
					used to enforce his site's local policy and track remote usage.
					This is only used if the specified account is not of type local.

TRACEBACKUSER	used in conjunction with TRACEBACKACCT to specify the subaccount
					used to enforce the submitting site's local policy and provide
					accounting information.

TRACEBACKHOST 	The QBank server host managing the submitting user's traceback
					account.

TRACEBACKPORT 	The QBank server port managing the submitting user's traceback
					account.

=item B<RETURN VALUES>

Returns a message indicating amount reserved.

=back

=cut


##############################################################
# remove_reservation JOBID=<jobid> | EXPIRED=1
# Logs transaction to flatfile if database server is not responding
# Logs how many nodeseconds were restored when the reservation was removed
# Removes only the specified reservation if jobid specified
# Removes all expired reservations if EXPIRED=1 is specified
# Returns message indicating amount of nodeseconds restored
##############################################################
sub remove_reservation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($sum=0);

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||(!$ARGS{JOBID}&&!$ARGS{EXPIRED}))
        { return &usage("$ARGS{COMMAND} JOBID=<jobid> | EXPIRED=1"); }

# Post to reconcile log if the database server is not responding
unless ($dbh->ping) {
	&post_txn("@_");
	return &success("0\nWarning: $DBI::errstr\nTransaction has been logged to the reconcile log");
	}

if ($ARGS{JOBID}) {
	&db("select accountname,jobid from reservation where jobid='$ARGS{JOBID}'") || return &fail($dberror);
	}
elsif ($ARGS{EXPIRED} eq "1") {
	&db("select accountname,jobid from reservation where expirationdate<'now'") || return &fail($dberror);
	}
else { return &usage("$ARGS{COMMAND} JOBID=<jobid> | EXPIRED=1"); }

foreach $reservation (@RESERVATIONS=@QUERYRESULT) {
	($account,$jobid)=split(/\|/,$reservation);

	&db("select sum(amount) from subreservation where jobid='$jobid'") || return &fail($dberror);
	($balance)=@QUERYRESULT;
	$balance*=1;
	$sum+=$balance;

	&db("delete from subreservation where jobid='$jobid'") || return &fail($dberror);
	&db("delete from reservation where jobid='$jobid'") || return &fail($dberror);

	# Log in txnlog iff a reservation was removed
	if ($ROWS) {
		&db("insert into txnlog (txntype,accountname,key,amount,timestamp,authorization) values ('remove_reservation','$account','$jobid',$balance,'now','$me')") || return &fail($dberror);
		}
	}

&bank_log(4,"@_ AMOUNT=$sum");
return &success("Successfully removed reservations totalling $sum credits");
}

=pod

=head1 remove_reservation

=over 4

=item B<NAME>

remove_reservation - removes reservations

=item B<SYNOPSIS>

remove_reservation B<JOBID>=I<jobid>|B<EXPIRED>=I<1> 

=item B<DESCRIPTION>

Removes only the specified reservation if B<JOBID> specified. Removes
all expired reservations if B<EXPIRED=I<1>> is specified Logs
transaction to flatfile if database server is not responding

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns message indicating amount of nodeseconds removed.

=back

=cut



##############################################################
# sync_reservations TYPE=maui [MACHINE=<MACHINE>] LIST=<JOBID>[:<JOBID>]...
# A mechanism by which the scheduler sends a list of all jobs actually
# running in the batch system (for which it should have reservations),
# and the bank returns the subset of those jobs which do not have current
# reservations. The scheduler should attempt to make reservations for the
# list of running jobs without reservations.
# Currently nothing is done automatically by the bank with the list of
# jobs for which it has current reservations, but which are not actually
# running in the batch queue. For now, it seems more useful to leave stale
# reservations around so any problem can be identified and corrected and
# then manually cleaned up with expire_reservations.
# Returns 1 if synced, else a list of jobs that the bank has no reservation for
##############################################################
sub sync_reservations {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local(@RESERVATIONS=(),@UNRESERVED=());

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{TYPE})
        { return &usage("$ARGS{COMMAND} TYPE=maui LIST=<JOBID>[:<JOBID>]..."); }

# Get a list of current bank reservations
$select="select jobid from reservation where expirationdate>'now'";
# Compare only against specified machine if MACHINE was NOT NULL in the request
if ($ARGS{MACHINE}) { $select.=" and machine='$ARGS{MACHINE}'"; }
&db($select) || return &fail($dberror);
@RESERVATIONS=@QUERYRESULT;

&bank_log(0,"reservations=".join(' ',@RESERVATIONS)."\n");
&bank_log(0,"jobs=".join(' ',split(/:/,$ARGS{LIST}))."\n");

foreach $job (split(/:/,$ARGS{LIST})) {
	grep(/$job/,@RESERVATIONS) || push(@UNRESERVED,$job);
	}

&bank_log(4,"@_");
unless (@UNRESERVED) { return &success(1); }
else { return &success(join(':',@UNRESERVED)); }
}

=pod

=head1 sync_reservations

=over 4

=item B<NAME>

sync_reservations - sync between scheduler and QBank.

=item B<SYNOPSIS>

sync_reservations B<TYPE>=I<maui> [B<MACHINE>=I<machine>] B<LIST>=I<jobid[:jobid]...>

=item B<DESCRIPTION>

A mechanism by which the scheduler sends a list of all jobs actually
running in the batch system (for which it should have reservations),
and the bank returns the subset of those jobs which do not have
current reservations. The scheduler should attempt to make
reservations for the list of running jobs without
reservations. Currently nothing is done automatically by the bank with
the list of jobs for which it has current reservations, but which are
not actually running in the batch queue. For now, it seems more useful
to leave stale reservations around so any problem can be identified
and corrected and then manually cleaned up with remove_reservation.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if synced, else a list of jobs for which the bank has no
reservation

=back

=cut


##############################################################
# get_reservations [ACCOUNT=<account>] [JOBID=<jobid>] [EXPIRED=1|0]
#                  [MACHINE=<machine>] [CHECKPOINT=<ckptkey>]
#						 [SHOW=<attributelist>]
# returns pipe-delimited arrays of "accountname,jobid,amount,expirationdate,machine"
##############################################################
sub get_reservations {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local(@RESERVATIONS=(),@RESINFO=());

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [JOBID=<jobid>] [EXPIRED=1|0] [MACHINE=<machine>] [CHECKPOINT=<ckptkey>] [SHOW=<attributelist>]"); }

if ($ARGS{CHECKPOINT}) { $X="x" } else { $X="" }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="accountname,jobid,amount,expirationdate,machine" };
$ARGS{SHOW}=~s/amount/sum(amount)/;			 # Needed because of group by
$ARGS{SHOW}=~s/jobid/${X}reservation.jobid/; 	 # Needed because of join

$select="select $ARGS{SHOW} from ${X}reservation,${X}subreservation where ${X}reservation.jobid=${X}subreservation.jobid";
if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{KEY}) {$select.=" and jobid='$ARGS{JOBID}'"}
if ($ARGS{JOBID}) {$select.=" and ${X}reservation.jobid='$ARGS{JOBID}'"}
if ($ARGS{EXPIRED} eq "1") {$select.=" and expirationdate<'now'"}
if ($ARGS{EXPIRED} eq "0") {$select.=" and expirationdate>='now'"}
if ($ARGS{MACHINE}) {$select.=" and machine='$ARGS{MACHINE}'"}
if ($ARGS{CHECKPOINT}) {$select.=" and ${X}reservation.checkpointkey=$ARGS{CHECKPOINT} and ${X}subreservation.checkpointkey=$ARGS{CHECKPOINT}"}

@GROUP_BY=grep(!/sum\(amount\)/&&!/${X}reservation.jobid/,split(/,/,$ARGS{SHOW}));
$select.=join(','," group by ${X}reservation.jobid",@GROUP_BY);

&db($select) || return &fail($dberror);

return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_reservations

=over 4

=item B<NAME>

get_reservations - gets reservation information.

=item B<SYNOPSIS>

get_reservations [B<ACCOUNT>=I<account>] [B<JOBID>=I<jobid>]
[B<EXPIRED>=I<1|0>] [B<MACHINE>=I<machine>] [B<CHECKPOINT>=I<ckptkey>]
[B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

Dumps a line for each reservation specified Primarily used within
administrative tools to provide account information Information
accessible depends on one's administrative priveledges.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns pipe-delimited array of "accountname, jobid, balance,
expirationdate, machine"

=back

=cut


##############################################################
# make_withdrawal [ACCOUNT=<account>] [USER=<user>] [WCTIME=<wallclockseconds>] [PROCS=<numprocessors>] [JOBID=<jobid>] [MACHINE=<machine>] [NODES=<numnodes>] [MEMORY=<memory>] [DISK=<disk>] [CLASS=<class>] [QOS=<qos>] [NODETYPE=<nodetype>] [JOBTYPE=<jobtype>] [REASON="<reason>"] [AMOUNT=<amount>] [QUOTEID=<quoteid>] [TRACEBACKHOST=<host>] [TRACEBACKPORT=<port>] [TRACEBACKACCT=<account>] [TRACEBACKUSER=<user>]
# This is a high-level subroutine which debits the specified account
# taking into account the KITTY and ANY pools.
# Logs transaction to flatfile if database server is not responding
# Tries to use default account if no ACCOUNT specified
# If QUOTEID is specified, will use the quoted rate if not expired
# An attempt is made to remove the associated reservation
# Not logging nodetype,jobtype,qos,memory,disk and class info to sub_withdrawals
# At some point I want to flag the reservation as a failure if it failed
# at any of its traceback or exchange transactions even if succeeded locally.
# If traceback information is supplied and target account is a not of type
# local, a traceback reservation will first be attempted.
# Maps USER to UNKNOWN if no USER specified.
# Will debit maximally from USER, KITTY, then ANY in that order
# in order to satisfy withdrawal request.
# It is expected that AMOUNT will not normally be explicitly specified
# except in the case of manual administrative adjustments. If AMOUNT is
# not specified, QBank will invoke the charge function to calculate the
# amount based on the specified parameters (PROCS, WCTIME, QOS, etc.)
# A warning is sent if unsuccessful
##############################################################
sub make_withdrawal {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
if ($ARGS{PROCCOUNT}) { $ARGS{PROCS}=$ARGS{PROCCOUNT}; } # Pre-2.9 compatibility
@WITHDRAWMESSAGES=();

&bank_log(0,"Entering @_");	# For debug purposes

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [USER=<user>] [WCTIME=<wallclockseconds>] [PROCS=<numprocessors>] [JOBID=<jobid>] [MACHINE=<machine>] [NODES=<numnodes>] [MEMORY=<memory>] [DISK=<disk>] [CLASS=<class>] [QOS=<qos>] [NODETYPE=<nodetype>] [JOBTYPE=<jobtype>] [REASON=\"<reason>\"] [AMOUNT=<amount>] [QUOTEID=<quoteid>] [TRACEBACKHOST=<host>] [TRACEBACKPORT=<port>] [TRACEBACKACCT=<account>] [TRACEBACKUSER=<user>]"); }

# Post to reconcile log if the database server is not responding
unless ($dbh->ping) {
	&post_txn("@_");
	return &success("0\nWarning: $DBI::errstr\nTransaction has been logged to the reconcile log");
	}

# Try to use default account if no ACCOUNT specified
unless ($ARGS{ACCOUNT}) {
        &db("select defaultaccount from userinfo where username='$ARGS{USER}'") || return &fail($dberror);
        if ($QUERYRESULT[0] ne '') { $ARGS{ACCOUNT}=$QUERYRESULT[0]; }
        else { return &deny("No Default Account can be found for $ARGS{USER}") }
        }

# Use UNKNOWN user if no USER specified
unless ($ARGS{USER}) { $ARGS{USER}='UNKNOWN'; }

# Verify account active
&db("select accountactive from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
unless ($QUERYRESULT[0])
	{return &deny("$ARGS{ACCOUNT} is not an active account.");}

# Verify user enabled for account
&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and (userstatus!='disabled')") || return &fail($dberror);
$enabled=$ROWS;
#unless ($ROWS)
#	{return &deny("$ARGS{USER} is not enabled to charge to account $ARGS{ACCOUNT}");}

# Set to default machine if unspecified
if ($ARGS{MACHINE}) { $machine=$ARGS{MACHINE}; }
else { $machine=$MACHINE; }

# If QUOTEID is specified, will use the quoted rate if not expired
if ($ARGS{QUOTEID}) {
	&db("select rate from quotation where quoteid='$ARGS{QUOTEID}' and expirationdate>='now'") || return &fail($dberror);
	if ($ROWS) { $CHARGERATE=$QUERYRESULT[0]; }
}
if ($CHARGERATE) {
	$withdrawamt=$ARGS{PROCS}*$ARGS{WCTIME}*$CHARGERATE;
	$ARGS{REASON}.=" Using QUOTEID $ARGS{QUOTEID} at rate $CHARGERATE";
}
# Else if AMOUNT is specified use it else calculate it with charge function
elsif ($ARGS{AMOUNT}) { $withdrawamt=$ARGS{AMOUNT}; }
# Otherwise, factor in charge multipliers when computing withdrawal amount
# This is done by calling the external charge subroutine from charge.ph
else { $withdrawamt=charge(); }

# Fail if withdrawamt is NULL
if ($withdrawamt eq "") { return &deny($QBank::Result::ARGS{MESSAGE}); }

# Fail if withdrawamt is negative
if ($withdrawamt<0) { return &deny("Withdraw Amount CANNOT be Negative");}

# Check for account type to see if remote exchange is necessary
&db("select accounttype,exchangehost,exchangeport,exchangeacct,exchangeuser,exchangerate from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
($accttype,$xhost,$xport,$xacct,$xuser,$xrate)=split(/\|/,$QUERYRESULT[0]);

# Debit traceback account if type is not local and TRACEBACK arguments specified
if (($accttype eq "meta" || $accttype eq "forwarding" || $accttype eq "seesaw") && $ARGS{TRACEBACKACCT} && $ARGS{TRACEBACKHOST} && $ARGS{TRACEBACKPORT})
{
	# If an TRACEBACKUSER is not defined then use USER
	unless ($ARGS{TRACEBACKUSER}) { $ARGS{TRACEBACKUSER}=$ARGS{USER}; }
	@QUERYRESULT=&qbank($ARGS{TRACEBACKHOST},$ARGS{TRACEBACKPORT},"make_withdrawal ACCOUNT=$ARGS{TRACEBACKACCT} USER=$ARGS{TRACEBACKUSER} AMOUNT=" . $withdrawamt*$xrate . " REASON=\"Traceback withdrawal from $ARGS{ACCOUNT}\@${machine}\" PROCS=$ARGS{PROCS} WCTIME=$ARGS{WCTIME} JOBID=$ARGS{JOBID} MACHINE=$ARGS{ACCOUNT}\@${machine}");
	$tracebackstatus=$STATUS;
	if (!$tracebackstatus) {
		&bank_log(4,"Failed debiting Traceback subaccount $ARGS{TRACEBACKUSER}\@$ARGS{TRACEBACKACCT} at $ARGS{TRACEBACKHOST}:$ARGS{TRACEBACKPORT} for ".($withdrawamt*$xrate).". Reason : @QUERYRESULT");
		push(@WITHDRAWMESSAGES,"\nWARNING: Failure debiting traceback subaccount $ARGS{TRACEBACKUSER}\@$ARGS{TRACEBACKACCT} at $ARGS{TRACEBACKHOST}:$ARGS{TRACEBACKPORT}: @QUERYRESULT");
	}
	else {
		push(@WITHDRAWMESSAGES,"\nTraceback account @QUERYRESULT");
		&db("insert into txnlog (txntype,accountname,username,amount,timestamp,authorization,machine,key,processors,wallclock,reason) values ('remote_withdrawal','$ARGS{TRACEBACKACCT}','$ARGS{TRACEBACKUSER}',".($withdrawamt*$xrate).",'now','$me','$machine','$ARGS{JOBID}',$ARGS{PROCS},$ARGS{WCTIME},'Remote debit to $ARGS{TRACEBACKHOST}:$ARGS{TRACEBACKPORT}')") || return &fail($dberror);
		}
	}

# Debit exchange account if account type is forwarding
if ($accttype eq "forwarding")
{
	# If an exchangeuser is not defined then use USER
	unless ($xuser) { $xuser=$ARGS{USER}; }
	@QUERYRESULT=&qbank($xhost,$xport,"make_withdrawal USER=$xuser ACCOUNT=$xacct AMOUNT=$withdrawamt REASON=\"Forwarding withdrawal from $ARGS{ACCOUNT}\@${machine}\" PROCS=$ARGS{PROCS} WCTIME=$ARGS{WCTIME} JOBID=$ARGS{JOBID} MACHINE=$machine");
	$forwardingstatus=$STATUS;
	if (!$forwardingstatus) {
		&bank_log(4,"Withdrawal aborted: Failed debiting Forwarding subaccount $xuser\@$xacct at $xhost:$xport for $withdrawamt");
	}
	else {
		push(@WITHDRAWMESSAGES,"\nForwarding account @QUERYRESULT");
		&db("insert into txnlog (txntype,accountname,username,amount,timestamp,authorization,machine,key,processors,wallclock,reason) values ('remote_withdrawal','$ARGS{ACCOUNT}','$xuser',$withdrawamt,'now','$me','$machine','$ARGS{JOBID}',$ARGS{PROCS},$ARGS{WCTIME},'Debited $xacct at $xhost:$xport')") || return &fail($dberror);
		}
	}

# Credit exchange account if account type is seesaw
if ($accttype eq "seesaw")
{
	# If an exchangeuser is defined then use it, otherwise use depositmask
	if ($xuser) { $USER=" USER=$xuser"; } else { $USER=""; }
	@QUERYRESULT=&qbank($xhost,$xport,"make_deposit$USER ACCOUNT=$xacct AMOUNT=" . $withdrawamt*$xrate . " REASON=\"Remote deposit from $ARGS{ACCOUNT}\@${machine}\" for $ARGS{JOBID}");
	$depositstatus=$STATUS;
	if (!$depositstatus) {
		&bank_log(4,"Failed crediting $xuser\@$xacct at $tbhost:$tbport for ".$withdrawamt*$xrate);
		push(@WITHDRAWMESSAGES,"\nWARNING: Failure crediting seesaw subaccount $xuser\@$xacct at $xhost:$xport: @QUERYRESULT. Therefore will double debit $ARGS{ACCOUNT}.");
	}
	else {
		push(@WITHDRAWMESSAGES,"\nSeesaw account @QUERYRESULT");
		&db("insert into txnlog (txntype,accountname,username,amount,timestamp,authorization,machine,key,reason) values ('remote_deposit','$ARGS{ACCOUNT}','$xuser',".($withdrawamt*$xrate).",'now','$me','$machine','$ARGS{JOBID}','Credited $xacct at $tbhost:$tbport')") || return &fail($dberror);
		}
	}

# If seesaw account was not credited, then double debit this account
if ($accttype eq "seesaw" && !$depositstatus)
{
	$withdrawamt*=2;
}

# Check for newly (de)activated allocations
&update_allocations;

# Lock suballocations before other withdrawal attempts check for sufficient
# funds. If a failure occurs anywhere between here and the COMMIT, the fail
# routine will perform a rollback.
$WORK=1;
&db("lock table suballocation");

#
# Obtain Available balances for user and KITTY
# This includes available credit balances
#

# User portion
if ($ARGS{USER} ne 'KITTY' || $ARGS{USER} ne 'ANY' || $ARGS{USER} ne 'UNKNOWN' || $ARGS{USER} ne 'RESERVE') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($userbalance)=@QUERYRESULT;
	}

# KITTY portion
if ( $enabled && $ARGS{USER} ne 'ANY' && $ARGS{USER} ne 'RESERVE') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='KITTY' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($kittybalance)=@QUERYRESULT;
	}

# ANY portion
if ($ARGS{USER} ne 'RESERVE') {
	&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='ANY' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:')") || return &fail($dberror);
	($anybalance)=@QUERYRESULT;
	}

# Bail out if user has insufficient funds
if (($userbalance+$kittybalance+$anybalance)<$withdrawamt)
	{ return &fail("Insufficient funds to debit $ARGS{USER} by $withdrawamt"); }

# Debit as much as possible from user, KITTY then ANY
$useramt=$kittyamt=$anyamt=0;
$remaining=$withdrawamt;

if ($remaining > 0) {
	if ($remaining<$userbalance) { $useramt=$remaining }
	else { $useramt=$userbalance }
	$remaining-=$useramt;
	}

if ($remaining > 0) {
	if ($remaining<$kittybalance) { $kittyamt=$remaining }
	else { $kittyamt=$kittybalance }
	$remaining-=$kittyamt;
	}

if ($remaining > 0) {
	if ($remaining<$anybalance) { $anyamt=$remaining }
	else { $anyamt=$anybalance }
	$remaining-=$anyamt;
	}

if ($remaining != 0) { return &fail("Something went wrong: Withdrawal amount was not correctly divided among subaccounts"); }

if ($useramt)
{ unless (&status(&debit_user("COMMAND=debit_user","ACCOUNT=$ARGS{ACCOUNT}","USER=$ARGS{USER}","AMOUNT=$useramt","MACHINE=$machine")))
	{ return &fail("Failed running debit_user for $ARGS{USER}"); }
# Log the user subwithdrawal in the txnlog
&db("insert into txnlog (txntype,accountname,username,key,amount,processors,wallclock,timestamp,authorization,machine) values ('make_subwithdrawal','$ARGS{ACCOUNT}','$ARGS{USER}','$ARGS{JOBID}',$useramt,$ARGS{PROCS},$ARGS{WCTIME},'now','$me','$machine')") || return &fail($dberror);
}
if ($kittyamt)
{ unless (&status(&debit_user("COMMAND=debit_user","ACCOUNT=$ARGS{ACCOUNT}","USER=KITTY","AMOUNT=$kittyamt","MACHINE=$machine")))
	{ return &fail("Failed running debit_user for KITTY"); }
# Log the KITTY subwithdrawal in the txnlog
&db("insert into txnlog (txntype,accountname,username,key,amount,processors,wallclock,timestamp,authorization,machine) values ('make_subwithdrawal','$ARGS{ACCOUNT}','KITTY','$ARGS{JOBID}',$kittyamt,$ARGS{PROCS},$ARGS{WCTIME},'now','$me','$machine')") || return &fail($dberror);
}
if ($anyamt)
{ unless (&status(&debit_user("COMMAND=debit_user","ACCOUNT=$ARGS{ACCOUNT}","USER=ANY","AMOUNT=$anyamt","MACHINE=$machine")))
	{ return &fail("Failed running debit_user for ANY"); }
# Log the ANY subwithdrawal in the txnlog
&db("insert into txnlog (txntype,accountname,username,key,amount,processors,wallclock,timestamp,authorization,machine) values ('make_subwithdrawal','$ARGS{ACCOUNT}','ANY','$ARGS{JOBID}',$anyamt,$ARGS{PROCS},$ARGS{WCTIME},'now','$me','$machine')") || return &fail($dberror);
}

# Log the overall withdrawal in the txnlog
if ($accttype eq "seesaw" && !$depositstatus) { $ARGS{REASON}.="Doubling b/c failed credit to $xacct at $tbhost:$tbport"; }
$NAMES="txntype,accountname,username,key,amount,timestamp,authorization,nodetype,qos,machine,class,type,reason";
$VALUES="'make_withdrawal','$ARGS{ACCOUNT}','$ARGS{USER}','$ARGS{JOBID}',$withdrawamt,'now','$me','$ARGS{NODETYPE}','$ARGS{QOS}','$machine','$ARGS{CLASS}','$ARGS{JOBTYPE}','$ARGS{REASON}'";
if ($ARGS{DISK}) { $NAMES.=",disk"; $VALUES.=",$ARGS{DISK}"; }
if ($ARGS{MEMORY}) { $NAMES.=",memory"; $VALUES.=",$ARGS{MEMORY}"; }
if ($ARGS{PROCS}) { $NAMES.=",processors"; $VALUES.=",$ARGS{PROCS}"; }
if ($ARGS{NODES}) { $NAMES.=",nodes"; $VALUES.=",$ARGS{NODES}"; }
if ($ARGS{WCTIME}) { $NAMES.=",wallclock"; $VALUES.=",$ARGS{WCTIME}"; }
&db("insert into txnlog ($NAMES) values ($VALUES)") || return &fail($dberror);
&bank_log(4,"@_");

# Commit the transaction
undef $WORK;
$dbh->commit;

# Remove reservation
&remove_reservation("COMMAND=remove_reservation","JOBID=$ARGS{JOBID}");

# Return success
return &success("User $ARGS{USER} successfully withdrew $withdrawamt from account $ARGS{ACCOUNT}.@WITHDRAWMESSAGES");
}

=pod

=head1 make_withdrawal

=over 4

=item B<NAME>

make_withdrawal - debits accounts.

=item B<SYNOPSIS>

make_withdrawal [B<ACCOUNT>=I<account>] [B<USER>=I<touser>]
[B<WCTIME>=I<wallclockseconds>] [B<PROCS>=I<numprocessors>] [B<JOBID>=I<jobid>]
[B<MACHINE>=I<machine>] [B<NODES>=I<numnodes>] [B<MEMORY>=I<memory>]
[B<DISK>=I<disk>] [B<CLASS>=I<class>] [B<QOS>=I<qos>]
[B<NODETYPE>=I<nodetype>] [B<JOBTYPE>=I<jobtype>] [B<REASON>=I<"reason">]
[B<AMOUNT>=I<amount>] [B<QUOTEID>=I<quoteid>]
[B<TRACEBACKHOST>=I<host>] [B<TRACEBACKPORT>=I<port>]
[B<TRACEBACKACCT>=I<account>] [B<TRACEBACKUSER>=I<user>]

=item B<DESCRIPTION>

Debits the specified account taking into account the B<KITTY> pool. Tries
to use default account if no B<ACCOUNT> is specified. Verifies ACCOUNT is
allowed to debit for MACHINE. Maps B<USER> to UNKNOWN if no B<USER> specified.
Will debit maximally from B<USER>, B<KITTY>, then B<ANY> in that order
in order to satisfy withdrawal request.
Uses AMOUNT if specified, otherwise calculates amount using charge function.
The external charge function factors in charge multipliers based on
specified parameters computing withdrawal amount. An attempt is made
to remove the associated reservation.

=item B<OPTIONS>

ACCOUNT	is used to specify the account to charge. If no account is specified,
			QBank will use the default account for the user if defined.

QUOTEID	when specified will use the rate guaranteed by the provided quoteid
			so long as the quote has not expired. The associated rate will be
			multiplied by the number of processors and the wallclock time to
			calculate the withdrawal amount.

AMOUNT	specifies the amount to be debited. It is expected that AMOUNT will
			not normally be explicitly specified except in the case of manual
			administrative adjustments. If AMOUNT is not specified, QBank will
			invoke the charge function to calculate the amount based on the
			specified parameters (PROCS, WCTIME, QOS, etc.)

CLASS		is intended to be used to distinguish queues or batch vs. interactive
			jobs and may be used as a basis for different billing rates.

JOBTYPE	may be used to specify whether a withdrawal was for a job or a
			block reservation, or alternately to store the scheduler which
			interacted with the bank for the job, i.e. maui, LoadLeveler, PBS ...

USER		indicates the user subaccount to charge for the withdrawal. If a user
			subaccount is specified, debits will be made first to the user
			subaccount and then to the KITTY after the user's own funds are
			depleted.  If no USER is specified, the KITTY account will be used. 

TRACEBACKACCT	specifies the account local to the submitting user that is
					used to enforce his site's local policy and track remote usage.
					This is only used if the specified account is not of type local.

TRACEBACKUSER	used in conjunction with TRACEBACKACCT to specify the subaccount
					used to enforce the submitting site's local policy and provide
					accounting information.

TRACEBACKHOST 	The QBank server host managing the submitting user's traceback
					account.

TRACEBACKPORT 	The QBank server port managing the submitting user's traceback
					account.

=item B<RETURN VALUES>

Returns a message indicating success or failure
A warning is sent if the corresponding reservation was not removed

=back

=cut


###############################################################
# account_balance ACCOUNT=<account> [MACHINE=<machine>]
# Credit balances are taken into effect (contribute negatively)
# Returns unexpired account balance in nodeseconds
###############################################################
sub account_balance {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<account> [MACHINE=<machine>]"); }

&update_allocations;
$select="select sum(amount-overdraft) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and suballocation.allockey=allocation.allockey and active='t'";
if ($ARGS{MACHINE}) { $select.=" and (allowmachines~':$ARGS{MACHINE}:' or allowmachines=':ANY:')"; }
&db($select) || return &fail($dberror);
($balance)=@QUERYRESULT;

return &success($balance);
}

=pod

=head1 account_balance

=over 4

=item B<NAME>

account_balance - check account balance

=item B<SYNOPSIS>

account_balance B<ACCOUNT>=I<account> [B<MACHINE>=I<machine>]

=item B<DESCRIPTION>

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns unexpired account balance in nodeseconds.

=back

=cut

		
###############################################################
# user_balance ACCOUNT=<account> USER=<user> [MACHINE=<machine>]
# Credit balances are taken into effect (contribute negatively)
# Returns currently unexpired user balance in nodeseconds
###############################################################
sub user_balance {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

# Usage
if ($ARGS{USAGE}||!$ARGS{ACCOUNT}||!$ARGS{USER})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<account> USER=<user> [MACHINE=<machine>]"); }

# Not Valid for KITTY, ANY or RESERVE users
if ($ARGS{USER} eq 'KITTY' || $ARGS{USER} eq 'RESERVE' ||  $ARGS{USER} eq 'ANY')
        { return &deny("user_balance is not valid for reserved user $ARGS{USER}"); }

# Verify account membership
&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and (userstatus='enabled' or userstatus='admin')") || return &fail($dberror);
$enabled=$ROWS;
#unless ($ROWS)
#	{return &deny("$ARGS{USER} is not a member of account $ARGS{ACCOUNT}");}

&update_allocations;

if ($enabled) {
	# Get USER+KITTY+ANY balance for enabled users
	$select="select sum(amount-overdraft) from suballocation,allocation where suballocation.allockey=allocation.allockey and accountname='$ARGS{ACCOUNT}' and active='t' and (username='$ARGS{USER}' or username='KITTY' or username='ANY')";
	}
else {
	# Get ANY balance for non-enabled users
	$select="select sum(amount-overdraft) from suballocation,allocation where suballocation.allockey=allocation.allockey and accountname='$ARGS{ACCOUNT}' and active='t' and username='ANY'";
	}
if ($ARGS{MACHINE}) { $select.=" and (allowmachines~':$ARGS{MACHINE}:' or allowmachines=':ANY:')"; }
&db($select) || return &fail($dberror);
($balance)=@QUERYRESULT;

return &success(1*$balance);
}

=pod

=head1 user_balance

=over 4

=item B<NAME>

user_balance - checks user balance.

=item B<SYNOPSIS>

user_balance B<ACCOUNT>=I<account> B<USERB>=I<user> [B<MACHINE>=I<machine>]

=item B<DESCRIPTION>

This balance represents the total funds for a subaccount (user) within
a particular account, including those funds for which reservations
exist. The result includes the relevant B<KITTY> (common) or B<ANY> funds
available to the user.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns currently unexpired user balance in nodeseconds.

=back

=cut

		
###############################################################
# user_available ACCOUNT=<account> USER=<user> [MACHINE=<machine>]
# Credit balances are taken into effect (contribute positively)
# Returns balance available for withdrawal (with pending jobs subtracted)
###############################################################
sub user_available {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

# Usage
if ($ARGS{USAGE}||!$ARGS{ACCOUNT}||!$ARGS{USER})
        { return &usage("$ARGS{COMMAND} ACCOUNT=<account> USER=<user> [MACHINE=<machine>]"); }

# Not Valid for KITTY, ANY or RESERVE users
if ($ARGS{USER} eq 'KITTY' || $ARGS{USER} eq 'RESERVE' || $ARGS{USER} eq 'ANY')
        { return &deny("user_available is not valid for reserved user $ARGS{USER}"); }

# Verify account exists
&db("select accountname from account where accountname='$ARGS{ACCOUNT}'") || return &fail($dberror);
unless ($ROWS)
	{return &deny("$ARGS{ACCOUNT} is not a valid account");}

# Verify account membership
&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}'") || return &fail($dberror);
$enabled=$ROWS;
#unless ($ROWS)
#	{return &deny("$ARGS{USER} is not a member of account $ARGS{ACCOUNT}");}

&update_allocations;

if ($enabled) {
	# Get USER+KITTY+ANY balance if enabled user
	$select="select sum(amount) from suballocation,allocation where suballocation.allockey=allocation.allockey and accountname='$ARGS{ACCOUNT}' and active='t' and (username='$ARGS{USER}' or username='KITTY' or username='ANY')";
	}
else {
	# Get ANY balance if non-enabled user
	$select="select sum(amount) from suballocation,allocation where suballocation.allockey=allocation.allockey and accountname='$ARGS{ACCOUNT}' and active='t' and username='ANY'";
	}
if ($ARGS{MACHINE}) { $select.=" and (allowmachines~':$ARGS{MACHINE}:' or allowmachines=':ANY:')"; }
&db($select) || return &fail($dberror);
($balance)=@QUERYRESULT;

if ($enabled) {
	# Get USER+KITTY+ANY pending if enabled user
	$select="select sum(amount) from subreservation,reservation where subreservation.jobid=reservation.jobid and accountname='$ARGS{ACCOUNT}' and expirationdate>'now' and (username='$ARGS{USER}' or username='KITTY' or username='ANY')";
	}
else {
	# Get ANY pending if non-enabled user
	$select="select sum(amount) from subreservation,reservation where subreservation.jobid=reservation.jobid and accountname='$ARGS{ACCOUNT}' and expirationdate>'now' and username='ANY'";
	}
if ($ARGS{MACHINE}) { $select.=" and machine='$ARGS{MACHINE}'"; }
&db($select) || return &fail($dberror);
($pending)=@QUERYRESULT;

return &success(1*($balance-$pending));
}

=pod

=head1 user_available

=over 4

=item B<NAME>

user_available - check balance available

=item B<SYNOPSIS>

user_available B<ACCOUNT>=I<account> B<USER>=I<user> [B<MACHINE>=I<machine>]

=item B<DESCRIPTION>

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns balance available for withdrawal (includes I<KITTY> and I<ANY>
contributions with corresponding pending jobs subtracted)

=back

=cut


###############################################################
# make_transfer ACCOUNT=<account> AMOUNT=<amount> FROMUSER=<user> 
#               TOUSER=<touser> [METHOD=<EARLIEST|LATEST|EVEN>] 
#               [MACHINES=<mach1:mach2:...>|ANY]
# A high level routine to transfer credits from one user to another
# METHOD can be specified to transfer the EARLIEST or LATEST expiring cycles
# Defaults to an EVEN transfer distribution
# MACHINES can be specified to indicate which machines' funds to be transfered
# This is an inclusive matching so MACHINES=MPP1 would transfer funds within
# an allocation which had an allowmachines list of MPP1:ECS1:TEST or ANY.
# If MACHINES is specified as ANY, only allocations with allowmachines set
# to ANY will be included in the transfer -- If you want ALL nonexpired
# allocations to be candidates in the transfer, leave MACHINES unspecified.
# All transfers occur within individual allocations (not between).
# Currently, transfers can only be made among debit allocations.
# Returns a message indicating amount transferred
###############################################################
sub make_transfer {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($balance=0);

unless (&authorize('acct',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT}||!$ARGS{FROMUSER}||!$ARGS{TOUSER}||!$ARGS{AMOUNT})
	{ return &usage("$ARGS{COMMAND} ACCOUNT=<account> AMOUNT=<amount> FROMUSER=<fromuser> TOUSER=<touser> [METHOD=<EARLIEST|LATEST|EVEN>] [MACHINES=<mach1:mach2:...>|ANY]"); }

# Verify specified subaccounts are valid
&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{FROMUSER}'") || return &fail($dberror);
unless ($ROWS)
        { return &deny("$ARGS{FROMUSER} is not a valid member of account $ARGS{ACCOUNT}"); }
&db("select username from subaccount where accountname='$ARGS{ACCOUNT}' and username='$ARGS{TOUSER}'") || return &fail($dberror);
unless ($ROWS)
        { return &deny("$ARGS{TOUSER} is not a valid member of account $ARGS{ACCOUNT}"); }


# Lock suballocations before other transfer or withdrawal attempts check for
# sufficient funds. If a failure occurs anywhere between here and the COMMIT,
# the fail routine will perform a rollback.
$WORK=1;
&db("lock table suballocation");

# Verify that fromuser has sufficient funds
&update_allocations;
$select="select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='$ARGS{FROMUSER}' and suballocation.allockey=allocation.allockey and alloctype='debit' and active='t'";
# Make sure funds are valid for specified machine(s)
if ($ARGS{MACHINES}) { $select.=" and (allowmachines~':$ARGS{MACHINES}:' or allowmachines=':ANY:')"; }
&db($select) || return &fail($dberror);
($frombalance)=@QUERYRESULT;
if ($frombalance-$ARGS{AMOUNT}<0)
	{ return &fail("$ARGS{FROMUSER} has insufficient funds to effect the transfer"); }

# Obtain list of allocation keys from fromuser that are positive and not expired
$select="select allocation.allockey,amount,expirationdate from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='$ARGS{FROMUSER}' and suballocation.allockey=allocation.allockey and expirationdate>='now' and amount>0 and alloctype='debit'";
# Only transfer among allocations valid for specified machine(s)
if ($ARGS{MACHINES}) { $select.=" and (allowmachines~':$ARGS{MACHINES}:' or allowmachines=':ANY:')"; }
# These will be ordered by expiration date
$select.=" order by expirationdate";
&db($select) || return &fail($dberror);
@ALLOCATION=@QUERYRESULT;

# Depending on the method chosen, iterate through the transfers

# METHOD=EARLIEST
# This method will transfer earliest expiring cycles first
if ($ARGS{METHOD} eq "EARLIEST") {
	$remaining=$ARGS{AMOUNT};
	$transferred=0;
	foreach $allocation (@ALLOCATION) {
		($key,$amount,$expires)=split(/\|/,$allocation);
		# Transfer all from this suballocation or the remainder if less
		$transferamt=$remaining<$amount?$remaining:$amount;
		last unless $remaining; # Bail if no remainder
		&db("update suballocation set amount=amount-$transferamt where allockey=$key and username='$ARGS{FROMUSER}'") || return &fail($dberror);
       	 	&db("update suballocation set amount=amount+$transferamt where allockey=$key and username='$ARGS{TOUSER}'") || return &fail($dberror);
		unless ($ROWS) { # Create suballocation if nonexistent
			&db("insert into suballocation (allockey,username,amount) values ($key,'$ARGS{TOUSER}','$transferamt')") || return &fail($dberror);
			}
		$remaining-=$transferamt;
		$transferred+=$transferamt;
		}
	# Log transfer to txnlog
	&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization) values ('make_transfer','$ARGS{ACCOUNT}','$ARGS{FROMUSER}','$ARGS{METHOD}',-$ARGS{AMOUNT},'now','$me')") || return &fail($dberror);
	&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization) values ('make_transfer','$ARGS{ACCOUNT}','$ARGS{TOUSER}','$ARGS{METHOD}',$ARGS{AMOUNT},'now','$me')") || return &fail($dberror);
       	 }

# METHOD=LATEST
# This method will transfer earliest expiring cycles first
elsif ($ARGS{METHOD} eq "LATEST") {
	$remaining=$ARGS{AMOUNT};
	$transferred=0;
	foreach $allocation (reverse @ALLOCATION) {
		($key,$amount,$expires)=split(/\|/,$allocation);
		# Transfer all from this suballocation or the remainder if less
		$transferamt=$remaining<$amount?$remaining:$amount;
		last unless $remaining; # Bail if no remainder
		&db("update suballocation set amount=amount-$transferamt where allockey=$key and username='$ARGS{FROMUSER}'") || return &fail($dberror);
       	 	&db("update suballocation set amount=amount+$transferamt where allockey=$key and username='$ARGS{TOUSER}'") || return &fail($dberror);
		unless ($ROWS) { # Create suballocation if nonexistent
			&db("insert into suballocation (allockey,username,amount) values ($key,'$ARGS{TOUSER}','$transferamt')") || return &fail($dberror);
			}
		$remaining-=$transferamt;
		$transferred+=$transferamt;
       		}
	# Log transfer to txnlog
	&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization) values ('make_transfer','$ARGS{ACCOUNT}','$ARGS{FROMUSER}','$ARGS{METHOD}',-$ARGS{AMOUNT},'now','$me')") || return &fail($dberror);
	&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization) values ('make_transfer','$ARGS{ACCOUNT}','$ARGS{TOUSER}','$ARGS{METHOD}',$ARGS{AMOUNT},'now','$me')") || return &fail($dberror);
	}
	
# METHOD=EVEN
# Distribute transferred cycles evenly among allocations
# This is the default transfer method
else {
	$ARGS{METHOD}="EVEN";
	$transferred=0;
	foreach $allocation (@ALLOCATION) {
		($key,$amount,$expires)=split(/\|/,$allocation);
		# Normalize the transfer amount and use int((2*n+1)/2) to round
		$transferamt=int(((2*($amount*$ARGS{AMOUNT}/$frombalance))+1)/2);
		&db("update suballocation set amount=amount-$transferamt where allockey=$key and username='$ARGS{FROMUSER}'") || return &fail($dberror);
		&db("update suballocation set amount=amount+$transferamt where allockey=$key and username='$ARGS{TOUSER}'") || return &fail($dberror);
		unless ($ROWS) { # Create suballocation if nonexistent
			&db("insert into suballocation (allockey,username,amount) values ($key,'$ARGS{TOUSER}','$transferamt')") || return &fail($dberror);
			}
		$transferred+=$transferamt;
		}
	# Log transfer to txnlog
	&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization) values ('make_transfer','$ARGS{ACCOUNT}','$ARGS{FROMUSER}','$ARGS{METHOD}',-$ARGS{AMOUNT},'now','$me')") || return &fail($dberror);
	&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization) values ('make_transfer','$ARGS{ACCOUNT}','$ARGS{TOUSER}','$ARGS{METHOD}',$ARGS{AMOUNT},'now','$me')") || return &fail($dberror);
	}

# Commit the transaction
undef $WORK;
$dbh->commit;

&bank_log(4,"@_");
return &success("Successfully transferred $transferred credits");
}

=pod

=head1 make_transfer

=over 4

=item B<NAME>

make_transfer

=item B<SYNOPSIS>

make_transfer B<ACCOUNT>=I<account> B<AMOUNT>=I<amount> B<FROMUSER>=I<user>
B<TOUSER>=I<touser> [B<METHOD>=I<EARLIEST|LATEST|EVEN>]
[B<MACHINES>=I<mach1:mach2:...|ANY>]

=item B<DESCRIPTION>

Used to transfer credits from one subaccount (user) to
another. B<METHOD> can be specified to transfer the I<EARLIEST> or
I<LATEST> expiring cycles, or distribute from all allocations
evenly. It defaults to an I<EVEN> transfer distribution.

Currently, transfers can only be made among debit allocations.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating amount transfered.

=back

=cut


##############################################################
# make_refund [JOBID=<jobid>] [ACCOUNT=<account>] [USER=<user>] 
#             [AMOUNT=<nodeseconds>] [FORCE=1|0] 
#             [WCTIME=<wallclock(seconds)>] [PROCS=<numprocessors>] 
#             [MACHINE=<machine>] [WDDATE=<MM/DD[/YY][_hh:mm:ss]>] 
#             [WDIVAL=<searchinterval(days)>] [REASON="<reason>"]
# In general it will not be sufficient to specify only the jobid since
# the jobid is not guaranteed to be unique within the txnlog.
# make_refund will attempt to do a lookup on the corresponding withdrawal
# If more than one matches are found, it will fail and return a list of jobs 
# that matched the input specifications. No matches is also a failure.
# If the FORCE flag is used, the refund will take place even when no matches
# or multiple matches are found, provided ACCOUNT, USER & AMOUNT are specified.
# A smaller amount than the withdrawal amount can be specified for a refund -
# which will be deducted first from the KITTY up to the kittywithdrawalamount
# unless the FORCE flag is used which causes the entire deduction from the user
# If WDDATE (WithDrawalDATE) is specified, then the search will be confined
# to times within an interval number of days before and after the WDDATE.
# If MACHINE is specified, make_refund will use it to find withdrawal matches.
# If a unique match is found, the associated machine will be used.
# If a force is used, the default system MACHINE will be used unless
# MACHINE was specified.
# This interval defaults to 3 days unless WDIVAL is specified.
# Makes a refund to a user(&KITTY) for a particular job or event
# Reimbursements will go strictly into the earliest expiring allocation
# whether of type debit or credit.
# Returns amount refunded if successful, else a list of matching jobs
##############################################################
sub make_refund {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($nodeseconds,$useravail,$kittyavail,$useramt,$kittyamt);

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [ACCOUNT=<account>] [AMOUNT=<nodeseconds>] [FORCE=1|0] [JOBID=<jobid>] [MACHINE=<machine>] [PROCS=<numprocessors>] [REASON=\"<reason>\"] [USER=<user>] [WCTIME=<seconds>] [WDDATE=<MM/DD[/YY][_hh:mm:ss]>] [WDIVAL=<searchinterval(days)>]"); }

# Set machine to system default if unspecified
if ($ARGS{MACHINE}) { $machine=$ARGS{MACHINE}; }
else { $machine=$MACHINE; }

# Build generic selection capable of selecting both make_withdrawals
# and make_subwithdrawals
$select="select accountname,username,amount,machine from txnlog where 1=1";

# Include selection parameters to narrow search
if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{JOBID}) {$select.=" and key='$ARGS{JOBID}'"}
if ($ARGS{MACHINE}) {$select.=" and machine='$ARGS{MACHINE}'"}
if ($ARGS{PROCS}) {$select.=" and processors='$ARGS{PROCS}'"}
if ($ARGS{WCTIME}) {$select.=" and wallclock='$ARGS{WALLCLOCK}'"}

# Parse and format time to an acceptable database format
if ($ARGS{WDDATE}) {
	$_=$ARGS{WDDATE};	# Parse time expression
	($month,$day,$slash,$year,$underscore,$time,$hour,$minute,$second)=/(\d\d)\/(\d\d)(\/(\d\d))?(_((\d\d):(\d\d):(\d\d)))?/;
	# Generate usage if incorrect
unless ($month) { return &usage("$ARGS{COMMAND} JOBID=<jobid> [ACCOUNT=<account>] [USER=<user>] [AMOUNT=<nodeseconds>] [FORCE=1|0] [WCTIME=<seconds>] [PROCS=<numprocessors>] [WDDATE=<MM/DD[/YY][_hh:mm:ss]>] [WDIVAL=<searchinterval(days)>]"); }
	# Format time to a database acceptable format
	$wddate="$month/$day/".($year?$year:(localtime)[5]).($time?" $time":"");
	# Set search interval to 3 days unless specified
	if ($ARGS{WDIVAL}) { $wdival=$ARGS{WDIVAL}; }
	else { $wdival=3; }
	$wdlo=$wddate-$wdival;
	$wdhi=$wddate+$wdival;
	# Try to match within a window of $wdival days before and after
	# If it is too wide then we may run into non-uniqueness.
	# If too strict, then educated stabs will fail
	$select.=" and timestamp BETWEEN '$wdlo' and '$wdhi'";
	}

$wdselect=$select."and txntype='make_withdrawal'";
$subwdselect=$select."and txntype='make_subwithdrawal'";

# Optionally specify username for withdrawal lookup
if ($ARGS{USER}) {$wdselect.=" and username='$ARGS{USER}'"}

&db($wdselect) || return &fail($dberror);
$matches=$ROWS;
$useramt=$kittyamt=0;

# If multiple matches 
if ($matches>1) {
	# If force flag is set
	if ($ARGS{FORCE}) {
		# and if we have essential attribute values
		if ($ARGS{ACCOUNT}&&$ARGS{USER}&&$ARGS{AMOUNT}) {
			# Refund Entire amount to user
			$refacct=$ARGS{ACCOUNT};
			$refuser=$ARGS{USER};
			$useramt=$ARGS{AMOUNT};
			}
		# otherwise fail
		else {
			return &deny("JobId $ARGS{JOBID} not unique so ACCOUNT,USER, and AMOUNT must be specified");
			} 
		}
	# otherwise fail returning list of matches
	else {
		return &deny("Your refund specifications resulted in the following (non-unique) matches.\nPlease respecify with more restrictive parameters\n".join("\n",@QUERYRESULT));
		}
	}

# If a single unique match
if ($matches==1) {
	($refacct,$refuser,$refamt,$machine)=split(/\|/,$QUERYRESULT[0]);
	# If force flag is set
	if ($ARGS{FORCE}) {
		# and if AMOUNT is specified 
		if ($ARGS{AMOUNT}) {
			# Refund specified amount to user
			# $refacct and $refuser already set from initial query
			$useramt=$ARGS{AMOUNT};
			}
		# otherwise refund appropriate subamounts to user and KITTY
		else {
			# Refund subamounts per make_subwithdrawal
			# $refacct and $refuser already set from initial query
			# Assume to be unique since withdrawal search was unique
			&db($subwdselect." and username='$refuser'") || return &fail($dberror);
			($acct,$user,$useramt,$machine)=split(/\|/,$QUERYRESULT[0]);
			&db($subwdselect." and username='KITTY'") || return &deny($dberror);
			($acct,$user,$kittyamt,$machine)=split(/\|/,$QUERYRESULT[0]);
			} 
		}
	# otherwise refund appropriate subamounts to user and KITTY
	else {
		&db("select * from txnlog where txntype='make_refund' and key='$ARGS{JOBID}'") || return &fail($dberror);
		if ($ROWS) {
			return &deny("A refund already exists for job $ARGS{JOBID}\n@QUERYRESULT\nUse FORCE to override");
			}

		# and if AMOUNT is specified 
		if ($ARGS{AMOUNT}) {
			# Fail if refund amount is larger than withrawal
			if ($ARGS{AMOUNT}>$refamt)
				{ return &deny("Amount specified for refund is greater than amount withdrawn - Use FORCE to override"); }
			# Use AMOUNT as base refunding KITTY first
			&db($subwdselect." and username='KITTY'") || return &fail($dberror);
			($acct,$user,$amount)=split(/\|/,$QUERYRESULT[0]);
			# If amount requested is less than kitty subwithdrawal
			# then refund amt requested otherwise full kitty subwd
			$kittyamt=$ARGS{AMOUNT}<$amount?$ARGS{AMOUNT}:$amount;
			&db($subwdselect." and username='$refuser'") || return &
fail($dberror);
         ($acct,$user,$amount)=split(/\|/,$QUERYRESULT[0]);
			# Refund the remainder to user
			$useramt=$ARGS{AMOUNT}-$kittyamt;
			}
		# otherwise refund appropriate subamounts to user and KITTY
		else {
			# Refund subamounts per make_subwithdrawal
			# $refacct and $refuser already set from initial query
			# Assume to be unique since withdrawal search was unique
			&db($subwdselect." and username='$refuser'") || return &fail($dberror);
			($acct,$user,$useramt)=split(/\|/,$QUERYRESULT[0]);
			&db($subwdselect." and username='KITTY'") || return &fail($dberror);
			($acct,$user,$kittyamt)=split(/\|/,$QUERYRESULT[0]);
			} 
		}
	}

# If no matches
if ($matches==0) {
	# If force flag is set
        if ($ARGS{FORCE}) {
                # and if we have essential attribute values
                if ($ARGS{ACCOUNT}&&$ARGS{USER}&&$ARGS{AMOUNT}) {
                        # Refund Entire amount to user
			$refacct=$ARGS{ACCOUNT};
			$refuser=$ARGS{USER};
			$useramt=$ARGS{AMOUNT};
                        }
                # otherwise fail
                else {
                        return &deny("JobId $ARGS{JOBID} not found so ACCOUNT,US
ER, and AMOUNT must be specified");
                        }
		}
        # otherwise fail recommending force
        else {
                return &deny("There is no withdrawal matching the given parameters -- Respecify with FORCE=1");
                }
	}

# Reimbursements will go strictly into the earliest expiring allocation
&db("select allockey,active,expirationdate from allocation where accountname='$refacct' and active='t' and (allowmachines~':${machine}:' or allowmachines=':ANY:') order by expirationdate") || return &fail($dberror);
unless ($ROWS) { return &deny("No active allocation found valid for refund which allows machine $machine"); }
# Grep out the first field to create an array of keys sorted by expirationdate
@unexpired=grep(($_)=/^(\d+)/,@QUERYRESULT);
# First key will be the earliest expiring (unexpired) allocation key
$earlykey=$unexpired[0];

# Make refunds for user and KITTY as needed
if ($useramt) {
	&db("update suballocation set amount=amount+$useramt where allockey=$earlykey and username='$refuser'") || return &fail($dberror);
	unless ($ROWS) { # Create suballocation if nonexistent
		&db("insert into suballocation (allockey,username,amount) values ($earlykey,'$refuser','$useramt')") || return &fail($dberror);
		}
	# Log the user subrefund in the txnlog
	&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization,machine) values ('make_subrefund','$refacct','$refuser','$ARGS{JOBID}',$useramt,'now','$me','$machine')") || return &fail($dberror);
	}
if ($kittyamt) {
	&db("update suballocation set amount=amount+$kittyamt where allockey=$earlykey and username='KITTY'") || return &fail($dberror);
	unless ($ROWS) { # Create suballocation if nonexistent
		&db("insert into suballocation (allockey,username,amount) values ($earlykey,'KITTY','$kittyamt')") || return &fail($dberror);
		}
	# Log the KITTY subrefund in the txnlog
	&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization,machine) values ('make_subrefund','$refacct','KITTY','$ARGS{JOBID}',$kittyamt,'now','$me','$machine')") || return &fail($dberror);
	}
		
# Log the overall refund in the txnlog
$refundamt=$useramt+$kittyamt;
&db("insert into txnlog (txntype,accountname,username,key,amount,timestamp,authorization,reason,machine) values ('make_refund','$refacct','$refuser','$ARGS{JOBID}',$refundamt,'now','$me','$ARGS{REASON}','$machine')") || return &fail($dberror);
&bank_log(4,"@_");

return &success("Successfully refunded $refundamt credits for $refuser in account $refacct for machine $machine");
}

=pod

=head1 make_refund

=over 4

=item B<NAME>

make_refund - rerund account credit.

=item B<SYNOPSIS>

make_refund [B<JOBID>=I<jobid>] [B<ACCOUNT>=I<account>] [B<USER>=I<user>]
[B<AMOUNT>=I<nodeseconds>] [B<FORCE>=I<1|0>] [B<WCTIMEI>=<wallclock(seconds)>]
[B<PROCS>=I<numprocessors>] [B<MACHINE>=I<machine>]
[B<WDDATE>=I<MM/DD[/YY][_hh:mm:ss]>] [B<WDIVAL>=I<searchinterval(days)>]
[B<REASON>=I<"reason">]

=item B<DESCRIPTION>

Makes a refund to a user (and B<KITTY>) for a particular job or event
In general it will not be sufficient to specify only the jobid since
the jobid is not guaranteed to be unique within the
txnlog. Make_refund will attempt to do a lookup on the corresponding
withdrawal. If more than one match is found, it will fail and return a
list of jobs that matched the input specifications. No matches is also
a failure. If the FORCE flag is used, the refund will take place even
when no matches or multiple matches are found, provided ACCOUNT, USER
& AMOUNT are specified. A smaller amount than the withdrawal amount
can be specified for a refund - which will be deducted first from the
B<KITTY> up to the B<kittywithdrawalamount> unless the B<FORCE> flag
is used which causes the entire deduction from the user If a unique
match is found and a refund already exists for the job, the refund
will fail unless overridden by the B<FORCE> flag. If B<WDDATE>
(B<WithDrawalDATE>) is specified, then the search will be confined to
times within an interval number of days before and after the
B<WDDATE>. This interval defaults to 3 days unless B<WDIVAL> is
specified. Reimbursements will go strictly into the earliest expiring
allocation whether of type debit or credit.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns amount refunded if successful, else a list of matching jobs.

=back

=cut


###############################################################
# change_suballocation KEY=<allockey> USER=<user> OVERDRAFT=<amount>
# Subroutine whose sole utility so far is to modify overdraft amounts
# Returns 1 if successful, else 0
###############################################################
sub change_suballocation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{KEY}||!$ARGS{USER}||!$ARGS{OVERDRAFT})
	{ return &usage("$ARGS{COMMAND} KEY=<allockey> USER=<user> OVERDRAFT=<amount>"); }

if ($ARGS{OVERDRAFT}<0)
	{ return &deny("Overdraft amount cannot be negative"); }

&db("update suballocation set overdraft=$ARGS{OVERDRAFT} where allockey=$ARGS{KEY} and username='$ARGS{USER}'") || return &fail($dberror);

&bank_log(0,"@_");
return &success($ROWS);
}

=pod

=head1 change_suballocation

=over 4

=item B<NAME>

change_suballocation

=item B<SYNOPSIS>

change_suballocation B<KEY>=I<allockey> B<USER>=I<user> B<OVERDRAFT>=I<amount> 

=item B<DESCRIPTION>

Subroutine whose sole utility so far is to modify overdraft amounts

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if successful, else 0.

=back

=cut


###############################################################
# debit_suballocation KEY=<allockey> USER=<user> AMOUNT=<amount>
# This is a lowlevel command formerly called by debit_user
# Probably should not be accessible unless care is taken to lock it
# Returns 1 if successful, else 0
###############################################################
sub debit_suballocation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{KEY}||!$ARGS{USER})
	{ return &usage("$ARGS{COMMAND} KEY=<allockey> USER=<user> AMOUNT=<amount>"); }

&db("select amount from suballocation where allockey=$ARGS{KEY} and username='$ARGS{USER}'") || return &fail($dberror);
($quantum)=@QUERYRESULT;
if ($quantum-$ARGS{AMOUNT}<0)
	{ return &fail("$ARGS{USER} would result in a negative balance"); }

&db("update suballocation set amount=amount-$ARGS{AMOUNT} where allockey=$ARGS{KEY} and username='$ARGS{USER}'") || return &fail($dberror);

# No need to remove suballocation if zero balance -- expiration will clean up

&bank_log(0,"@_");
return &success(1);
}

=pod

=head1 debit_suballocation

=over 4

=item B<NAME>

debit_suballocation

=item B<SYNOPSIS>

debit_suballocation B<KEY>=I<allockey> B<USER>=I<user> B<AMOUNT>=I<amount> 

=item B<DESCRIPTION>

This is a lowlevel command formerly called by debit_user. This
probably shouldn't be accessible unless care is taken to lock it.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if successfull, else 0

=back

=cut


###############################################################
# credit_suballocation KEY=<allockey> USER=<user> AMOUNT=<amount>
# This is a lowlevel command without an internal use thus far
# Returns 1 if successful, else 0
###############################################################
sub credit_suballocation {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{KEY}||!$ARGS{USER})
	{ return &usage("$ARGS{COMMAND} KEY=<allockey> USER=<user> AMOUNT=<amount>"); }

&db("update suballocation set amount=amount+$ARGS{AMOUNT} where allockey=$ARGS{KEY} and username='$ARGS{USER}'") || return &fail($dberror);

unless ($ROWS) { # Create suballocation if nonexistent
	&db("insert into suballocation (allockey,username,amount) values ($ARGS{KEY},'$ARGS{USER}','$ARGS{AMOUNT}')") || return &fail($dberror);
	}

&bank_log(0,"@_");
return &success(1);
}

=pod

=head1 credit_suballocation

=over 4

=item B<NAME>

credit_suballocation

=item B<SYNOPSIS>

credit_suballocation B<KEY>=I<allockey> B<USER>=I<user> B<AMOUNT>=I<amount> 

=item B<DESCRIPTION>

this is a lowlevel command without an internal use thus far.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if successful, else 0.

=back

=cut


###############################################################
# debit_user ACCOUNT=<account> USER=<user> AMOUNT=<nodeseconds> [MACHINE=<machine>]
# Locking should probably be implemented before making this external
# Current problem is that this is called by make_withdrawal which
# already does locking and committing which would cause a conflict
# Debits soonest expiring credits first from debit allocations followed
# by credit allocations
# Returns 1 if success else 0
###############################################################
sub debit_user {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}
local($funds=0,$remaining=$ARGS{AMOUNT},$debitamt,@soonerkeys,@laterkeys,%upper,%lower);

&bank_log(0,"Entering @_");	# For debug purposes

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{ACCOUNT}||!$ARGS{AMOUNT}||!$ARGS{USER})
	{ return &usage("$ARGS{COMMAND} ACCOUNT=<account> USER=<user> AMOUNT=<nodeseconds>"); }

# Make sure user has sufficient funds in either debit or credit suballocations
&db("select sum(amount) from suballocation,allocation where accountname='$ARGS{ACCOUNT}' and username='$ARGS{USER}' and suballocation.allockey=allocation.allockey and active='t' and (allowmachines~':$ARGS{MACHINE}:' or allowmachines=':ANY:')") || return &fail($dberror);
($userbalance)=@QUERYRESULT;
if ($userbalance<$ARGS{AMOUNT})
	{ return &fail("$ARGS{USER} has insufficient funds to make the debit");}

# Obtain list of debit keys
&db("select allockey,expirationdate from allocation where accountname='$ARGS{ACCOUNT}' and active='t' and (allowmachines~':$ARGS{MACHINE}:' or allowmachines=':ANY:') and alloctype='debit' order by expirationdate") || return &fail($dberror);
# Grep out the first field to create an array of keys
@debitkeys=grep(($_)=/^(\d+)/,@QUERYRESULT);

# Obtain list of credit keys
&db("select allockey,expirationdate from allocation where accountname='$ARGS{ACCOUNT}' and active='t' and (allowmachines~':$ARGS{MACHINE}:' or allowmachines=':ANY:') and alloctype='credit' order by expirationdate") || return &fail($dberror);
# Grep out the first field to create an array of keys
@creditkeys=grep(($_)=/^(\d+)/,@QUERYRESULT);

# First take from debit allocations followed by credit allocations
foreach $keyref (\@debitkeys,\@creditkeys) {
	@laterkeys=@soonerkeys=@$keyref;
	next unless $remaining>0;

	# Here we have to make interallocation transfers so that the user always
	# burns up the soonest expiring credits in the account first
	foreach $soonerkey (@soonerkeys) {
		&bank_log(0,"\tsoonerkey=$soonerkey");

		# funds accumulated from this allocation
		&db("select amount from suballocation where allockey=$soonerkey and username='$ARGS{USER}'") || return &fail($dberror);
		($funds)=@QUERYRESULT;
		$funds*=1;
		&bank_log(0,"\tfunds_for_this_iteration=$funds");

		# If transferred funds can satisfy withdrawal...
		# Fall through and debit full amount
		if ($funds>=$remaining) { $debitamt=$remaining; }

		else {	# Round robin interallocation transfers until quanta are zero
			shift(@laterkeys); # to iterate thru later-expiring allocations
			ITERATION: foreach $laterkey (@laterkeys) {
				&bank_log(0,"\tlaterkey=$laterkey");
	
				# Determine amount spinable from this allocation
				# Only consider allocations of the same type
				&db("select amount from suballocation where allockey=$laterkey and username='$ARGS{USER}'") || return &fail($dberror);
				($transferable)=@QUERYRESULT;

				# Loop until suballocation amounts (quanta) are zero
				do { 
					# rebuild hash of next earliest expiring quanta
					&db("select username,amount from suballocation where allockey=$soonerkey and username!='$ARGS{USER}' and amount>0") || return &fail($dberror);
					undef %upper;
					foreach (@QUERYRESULT) { ($user,$amount)=split('\|',$_);$upper{$user}=$amount; }

					# Calc nzmin/max of quanta and how many postv
					$min=1000000000;$max=0;$nzquanta=0;
					foreach $quantum (keys %upper) {
						$min=$upper{$quantum}<$min?$upper{$quantum}:$min;
						$max=$upper{$quantum}>$max?$upper{$quantum}:$max;
						$nzquanta++;
						}
					&bank_log(0,"\tmin=$min max=$max nzquanta=$nzquanta");
					# Go to next suballocation if all quanta empty
					next unless $nzquanta;
	
					# The amount to be transferred is calculated
					# as the minimum of:
					# 1. the smallest nonzero suballocation amount
					# 2. the amount transferable from lower alloc /
					#    the number of nonzero suballocation amounts
					# 3. ceiling of (the amount still needed /
					#    the number of nonzero suballocation amounts)
					#    where ceil(x/n)=int((x+n-1)/n)
					$spinamt=&min(int(($remaining-$funds+$nzquanta-1)/$nzquanta),&min($min,int($transferable/$nzquanta)));
					&bank_log(0,"\ttransferable=int($transferable/$nzquanta)=".int($transferable/$nzquanta));
					&bank_log(0,"\tneeded=int(($remaining-$funds+1)/$nzquanta)=".(int(($remaining-$funds+1)/$nzquanta)));
					&bank_log(0,"spinamt=$spinamt");

					# Round Robin through the nonzero balance users					# in the active allocations in the order of
					# soonest expiring to latest expiring
					foreach $nzuser (keys %upper) {
						&bank_log(0,"\tspin $spinamt from $nzuser");
						# Earlier expiring credits are changed
						# into later expiring credits for the
						# nzuser, while later expiring credits
						# are changed into earliest expiring
						# credits for the requestor. This
						# allows for all withdrawals to expend
						# the earliest expiring account credits,					# while keeping the books balanced.
						# The suballocation amounts are
						# maintained in order to ensure fairness
						# when allocations expire.
	
						# Make sure debits will not result in negative quanta
						# Is this check really necessary?
						&db("select amount from suballocation where allockey=$soonerkey and username='$nzuser'") || return &fail($dberror);
						($fromquantum)=@QUERYRESULT;
						if ($fromquantum-$spinamt<0)
							{ return &fail("$nzuser would result in a negative balance"); }

						# Swap from nzuser to USER in sooner allocation
						&db("update suballocation set amount=amount-$spinamt where allockey=$soonerkey and username='$nzuser'") || return &fail($dberror);
						&db("update suballocation set amount=amount+$spinamt where allockey=$soonerkey and username='$ARGS{USER}'") || return &fail($dberror);
						unless ($ROWS) { # Create suballocation if nonexistent
							&db("insert into suballocation (allockey,username,amount) values ($soonerkey,'$ARGS{USER}','$spinamt')") || return &fail($dberror);
							}

						# Swap from USER to nzuser in later allocation
						&db("update suballocation set amount=amount-$spinamt where allockey=$laterkey and username='$ARGS{USER}'") || return &fail($dberror);
						&db("update suballocation set amount=amount+$spinamt where allockey=$laterkey and username='$nzuser'") || return &fail($dberror);
						unless ($ROWS) { # Create suballocation if nonexistent
							&db("insert into suballocation (allockey,username,amount) values ($laterkey,'$nzuser','$spinamt')") || return &fail($dberror);
							}

						$funds+=$spinamt;
						$transferable-=$spinamt;
						}
					&bank_log(0,"\tfunds_for_this_iteration=$funds");
					if ($funds>=$remaining) {
						last ITERATION;
           			}
					} until ($transferable<$nzquanta || $nzquanta==0) # Loop until quanta are zero
				}
			$debitamt=&min($funds,$remaining);
			}

	&db("update suballocation set amount=amount-$debitamt where allockey=$soonerkey and username='$ARGS{USER}'") || return &fail($dberror);
		&bank_log(0, "\tdebited $debitamt from KEY $soonerkey");
		$remaining-=$debitamt;
		last if $remaining<=0; 
		}
	}

&bank_log(4,"@_");
return &success(1);
}

=pod

=head1 debit_user

=over 4

=item B<NAME>

debit_user

=item B<SYNOPSIS>

debit_user B<ACCOUNT>=I<account> B<USER>=I<user> B<AMOUNT>=I<nodeseconds>
      [B<MACHINE>=I<machine>]

=item B<DESCRIPTION>

Iterates through the allocations, using a round-robin technique to
make interallocation transfers so that the user always burns up the
soonest expiring credits first from debit allocations followed
by credit allocations.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if success else 0.

=back

=cut


##############################################################
# add_chargerate MACHINE=<machine> TYPE=<chargetype> NAME=<chargename> RATE=<chargerate>
# where TYPE is a type of charge factor such as QOS, NODETYPE, TIME, CLASS
# and NAME is a particular instance of the chargetype
# and RATE is expressed as a positive decimal number (like 2 or 1.5)
# TYPE is effectively a hash or associative array of names and rates,
# i.e. %NODETYPE={ P2SC => 1, SMP => 2 } would be represented as:
#
# 	TYPE=NODETYPE NAME=P2SC RATE=1
# 	TYPE=NODETYPE NAME=SMP RATE=2
#
# The SIMPLE type is a special charge factor in which each have a single rate
# 
#	TYPE=SIMPLE NAME=PROCS RATE=1
#
# The following chargerates are predefined in a special DEFAULT machine to be
# used when no other machine-specific chargerates can be located.
#
#	MACHINE=DEFAULT TYPE=SIMPLE NAME=PROCS RATE=1
#	MACHINE=DEFAULT TYPE=SIMPLE NAME=MEMORY RATE=0
#	MACHINE=DEFAULT TYPE=SIMPLE NAME=DISK RATE=0
#	MACHINE=DEFAULT TYPE=QOS NAME=0 RATE=1
#	MACHINE=DEFAULT TYPE=QOS NAME=1 RATE=1
#	MACHINE=DEFAULT TYPE=NODE NAME=DEFAULT RATE=1
#	MACHINE=DEFAULT TYPE=TIME NAME=PRIME RATE=1
#	MACHINE=DEFAULT TYPE=CLASS NAME=batch RATE=1
#	MACHINE=DEFAULT TYPE=CLASS NAME=interactive RATE=1
#
# Returns success or failure message
##############################################################
sub add_chargerate {
$rv=QBank::ChargeRate->add();
return "STATUSCODE=$ARGS{STATUS} RESULT=$rv MESSAGE=\"$ARGS{MESSAGE}\"\n";
}

=pod

=head1 add_chargerate

=over 4

=item B<NAME>

add_chargerate - define new chargerate

=item B<SYNOPSIS>

add_chargerate B<MACHINE>=I<machine> [B<TYPE>=I<chargetype>] 
[B<NAME>=I<chargename>] [B<RATE>=I<chargerate>] 

=item B<DESCRIPTION>

Defines a new charge rate. The charge object stores machine-specific 
charge rates such as processor billing multipliers, QOS charge rates, etc.

TYPE is a type of charge factor such as QOS, NODETYPE, TIME, CLASS

NAME is a particular instance of the chargetype

RATE is expressed as a positive decimal number (like 2 or 1.5)

TYPE is effectively a hash or associative array of names and rates,
i.e. %NODETYPE={ P2SC => 1, SMP => 2 } would be represented as:

 	TYPE=NODETYPE NAME=P2SC RATE=1
 	TYPE=NODETYPE NAME=SMP RATE=2

The SIMPLE type is a special charge factor in which each have a single rate

	TYPE=SIMPLE NAME=PROCS RATE=1

The following chargerates are predefined in a special DEFAULT machine to be
used when no other machine-specific chargerates can be located.

	MACHINE=DEFAULT TYPE=SIMPLE NAME=PROCS RATE=1
	MACHINE=DEFAULT TYPE=SIMPLE NAME=MEMORY RATE=0
	MACHINE=DEFAULT TYPE=SIMPLE NAME=DISK RATE=0
	MACHINE=DEFAULT TYPE=QOS NAME=0 RATE=1
	MACHINE=DEFAULT TYPE=QOS NAME=1 RATE=1
	MACHINE=DEFAULT TYPE=NODE NAME=DEFAULT RATE=1
	MACHINE=DEFAULT TYPE=TIME NAME=PRIME RATE=1
	MACHINE=DEFAULT TYPE=CLASS NAME=batch RATE=1
	MACHINE=DEFAULT TYPE=CLASS NAME=interactive RATE=1

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a success or failure message

=back

=cut


##############################################################
# remove_chargerate MACHINE=<machine> [TYPE=<chargetype>] [NAME=<chargename>]
# Returns success or failure message
##############################################################
sub remove_chargerate {
$rv=QBank::ChargeRate->remove();
return "STATUSCODE=$ARGS{STATUS} RESULT=$ARGS{MESSAGE}\n";
}

=pod

=head1 remove_chargerate

=over 4

=item B<NAME>

remove_chargerate - deletes a chargerate

=item B<SYNOPSIS>

remove_chargerate B<MACHINE>=I<machine> [B<TYPE>=I<chargetype>] [B<NAME>=I<chargename>]

=item B<DESCRIPTION>

Removes the selected charge rate
If MACHINE, TYPE and  NAME are all specified, at most that unique entry
will be changed. If TYPE and/or NAME are left unspecified, all entries matching
the specified attributes will be removed. MACHINE must be specified.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a success or failure message.

=back

=cut


##############################################################
# change_chargerate MACHINE=<machine> [TYPE=<chargetype>] [NAME=<chargename>] RATE=<chargerate>
# This command is used to change the billing rates associated with a particular
# machine
# Returns success or failure message
##############################################################
sub change_chargerate {
$rv=QBank::ChargeRate->change();
return "STATUSCODE=$ARGS{STATUS} RESULT=$ARGS{MESSAGE}\n";
}

=pod

=head1 change_chargerate

=over 4

=item B<NAME>

change_chargerate - change machine-specific billing rates

=item B<SYNOPSIS>

change_chargerate B<MACHINE>=I<machine> [B<TYPE>=I<chargetype>] 
[B<NAME>=I<chargename>] B<RATE>=I<chargerate> 

=item B<DESCRIPTION>

This command is used to change the billing rates associated with a particular 
machine such as processor billing multipliers, QOS charge rates, etc.
If MACHINE, TYPE, NAME and RATE are all specified, at most that unique entry
will be changed. If TYPE and/or NAME are left unspecified, all entries matching
the specified attributes will be changed to have the designated rate.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns success or failure message.

=back

=cut

################################################################################
# get_chargerates [MACHINE=<machine>] [TYPE=<chargetype>] [NAME=<chargename>] [SHOW=<attributelist>]
# Returns an array of "machinename,chargetype,chargename,chargerate"
################################################################################
sub get_chargerates {
$rv=QBank::ChargeRate->get();
return "STATUSCODE=$ARGS{STATUS} RESULT=$rv MESSAGE=\"$ARGS{MESSAGE}\"\n";
}

=pod

=head1 get_chargerates

=over 4

=item B<NAME>

get_chargerates - display machine specific charge rates

=item B<SYNOPSIS>

get_chargerates [B<MACHINE>=I<machine>] [B<CHECKPOINT>=I<ckptkey>]
[B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

Dumps a line for each chargerate specified.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns pipe-delimited arrays of:
"machinename,chargetype,chargename,chargerate"
unless overridden by the B<SHOW> parameter.

=back

=cut


################################################################################
# get_txns [TYPE=<txntype>] [ACCOUNT=<account>] [USER=<user>] [KEY=<key|jobid>] 
#          [START=MM/DD[/YY][_hh:mm:ss]] [END=MM/DD[/YY][_hh:mm:ss]] 
#          [MACHINE=<machine>] [SHOW=<attributelist>]
# Returns an array of "txntype,accountname,username,amount,key,timestamp,expirationdate,authorization,processors,wallclock,type,nodetype,qos,reason,active,activationdate,machine,memory,disk,class,nodes"
################################################################################
sub get_txns {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('user',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [TYPE=<txntype>] [ACCOUNT=<account>] [USER=<user>] [KEY=<key|jobid>] [START=MM/DD[/YY][_hh:mm:ss]] [END=MM/DD[/YY][_hh:mm:ss]] [MACHINE=<machine>] [SHOW=<attributelist>]"); }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="txntype,accountname,username,amount,key,timestamp,expirationdate,authorization,processors,wallclock,type,nodetype,qos,reason,active,activationdate,machine,memory,disk,class,nodes" };

$select="select $ARGS{SHOW} from txnlog where txntype!=''";

if ($ARGS{TYPE}) {$select.=" and txntype='$ARGS{TYPE}'"}
if ($ARGS{ACCOUNT}) {$select.=" and accountname='$ARGS{ACCOUNT}'"}
if ($ARGS{USER}) {$select.=" and username='$ARGS{USER}'"}
if ($ARGS{KEY}) {$select.=" and key='$ARGS{KEY}'"}
if ($ARGS{MACHINE}) {$select.=" and machine~'$ARGS{MACHINE}'"}

if ($ARGS{START}) {
	$_=$ARGS{START};	# Parse time expression
	($month,$day,$slash,$year,$underscore,$time,$hour,$minute,$second)=/(\d\d)\/(\d\d)(\/(\d\d))?(_((\d\d):(\d\d):(\d\d)))?/;
	# Generate usage if incorrect
	unless ($month) {
		return &usage("$ARGS{COMMAND} [TYPE=<txntype>] [ACCOUNT=<account>] [USER=<user>] [START=MM/DD[/YY][_hh:mm:ss]] [END=MM/DD[/YY][_hh:mm:ss]]");
		}
	# Format time to a database acceptable format
	$stime="$month/$day/".($year?$year:(localtime)[5]).($time?" $time":"");
	$select.=" and timestamp>'$stime'";
	}

if ($ARGS{END}) {
	$_=$ARGS{END};	# Parse time expression
	($month,$day,$slash,$year,$underscore,$time,$hour,$minute,$second)=/(\d\d)\/(\d\d)(\/(\d\d))?(_((\d\d):(\d\d):(\d\d)))?/;
	# Generate usage if incorrect
	unless ($month) {
		return &usage("$ARGS{COMMAND} [TYPE=<txntype>] [ACCOUNT=<account>] [USER=<user>] [START=MM/DD[/YY][_hh:mm:ss]] [END=MM/DD[/YY][_hh:mm:ss]]");
		}
	# Format time to a database acceptable format
	$ftime="$month/$day/".($year?$year:(localtime)[5]).($time?" $time":" 23:59:59");
	$select.=" and timestamp<'$ftime'";
	}

# Create a unique allocation key
&db($select) || return &fail($dberror);

return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_txns

=over 4

=item B<NAME>

get_txns - display transaction information

=item B<SYNOPSIS>

get_txns [B<TYPE>=I<txntype>] [B<ACCOUNT>=I<account>]
[B<USER>=I<user>] [B<KEY>=I<key|jobid>] [B<SHOW>=I<attributelist>]
[B<START>=I<MM/DD[/YY][_hh:mm:ss]>] [B<END>=I<MM/DD[/YY][_hh:mm:ss]>]
[B<MACHINE>=I<machine>]

=item B<DESCRIPTION>

A utility for querying the transaction logs for reporting purposes
Transaction types include:

=over 4

=item add_account 

=item remove_account 

=item add_subaccount 

=item remove_subaccount

=item make_deposit

=item remove_allocation

=item expire_allocation

=item make_withdrawal

=item make_subwithdrawal

=item make_reservation

=item make_subreservation

=item remove_reservation

=item expire_reservation

=item make_transfer

=item make_refund

=item make_subrefund

=back 

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns an array of "txntype, accountname, username, amount, key,
timestamp, expirationdate, authorization, processors, wallclock,
memory, disk, class, nodes" unless overridden by the B<SHOW> parameter.

=back

=cut


##############################################################
# reconcile_bank
##############################################################
sub reconcile_bank {
local(@STATUS)=(); local(%TXN)=();

# List of allowable commands - reduces log stuffing exposure
@RECONCILEABLECMDS=("make_reservation","remove_reservation","make_withdrawal");

# Fail if database server is unreachable
$rc = $dbh->ping;
unless ($rc) {return &deny("Cannot Reconcile Bank: $DBI::errstr")}

open(RCNLOG,"<$LOG_DIR/$RCNLOG");
while (<RCNLOG>) {
	push(@STATUS,$_);
	foreach $pair (@TXN=split) {
        	($key,$value)=split(/=/,$pair);
        	$TXN{$key}=$value;
        	}
	$cmd=$TXN{COMMAND};
	if (grep($cmd eq $_,@RECONCILEABLECMDS)) { ($REPLY) = do $cmd(@TXN); }
	else {$REPLY = "STATUSCODE=0 RESULT=$cmd is not a valid command\n";}
	push(@STATUS,"$REPLY\n");
	}
close RCNLOG;

# Save a copy of the reconciled log in case something didn't go right
rename("$LOG_DIR/$RCNLOG","$LOG_DIR/$RCNLOG.old");

&bank_log(4,"@_");
return &success(join'',@STATUS);
}

=pod

=head1 reconcile_bank

=over 4

=item B<NAME>

reconcile_bank

=item B<SYNOPSIS>

reconcile_bank

=item B<DESCRIPTION>

If the database server ever becomes unreachable, the bank daemon will
write all essential scheduler transactions (i.e. reservations,
withdrawals) to a flatfile while allowing the transactions to
succeed. At a later point the administrator can run reconcile_bank to
post the transactions to the database as they were initially made.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a list of statuscodes for the commands processed 

=back

=cut


##############################################################
# make_checkpoint [REASON="<reason>"]
# Returns a message indicating success or failure
##############################################################
sub make_checkpoint {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [REASON=\"<reason>\"]"); }

# Create a unique checkpoint key
&db("select max(checkpointkey) from checkpoint") || return &fail($dberror);
($ckptkey)=@QUERYRESULT; $ckptkey++;

@TABLES=("account","subaccount","allocation","suballocation","reservation","subreservation");

&db("insert into checkpoint (checkpointkey,timestamp,reason) values ($ckptkey,'now','$ARGS{REASON}')") || return &fail($dberror);

foreach $table (@TABLES) {
	&db("update $table set checkpointkey=0") || return &fail($dberror);
	&db("insert into x$table select * from $table") || return &fail($dberror);
	&db("update x$table set checkpointkey=$ckptkey where checkpointkey=0") || return &fail($dberror);
	}

return &success("Successfully created checkpoint $ckptkey");
}

=pod

=head1 make_checkpoint

=over 4

=item B<NAME>

make_checkpoint

=item B<SYNOPSIS>

make_checkpoint [B<REASON>=I<"reason">]

=item B<DESCRIPTION>

Creates a checkpoint of the account, subaccount, allocation,
suballocation, reservation,subreservation,defaultaccount tables so
that state may be preserved for that time.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating success or failure 

=back

=cut


##############################################################
# restore_checkpoint KEY=<ckptkey>
# Restores a previously created checkpoint
# Automatically creates a safety checkpoint taken before the restore
# and an accounting checkpoint taken after the restore?
# Returns a message indicating the number of records restored
##############################################################
sub restore_checkpoint {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{KEY})
        { return &usage("$ARGS{COMMAND} KEY=<ckptkey>"); }

# Create a safety checkpoint in case restore is undesired
unless (&status(&make_checkpoint("REASON=Prerestore ($ARGS{KEY}) safety")))
	{ return &deny("safety checkpoint could not be created"); }

@TABLES=("account","subaccount","allocation","suballocation","reservation","subreservation");

$count=0;
foreach $table (@TABLES) {
	&db("delete from $table") || return &fail($dberror);
	&db("insert into $table select * from x$table where checkpointkey=$ARGS{KEY}") || return &fail($dberror);
	$count+=$ROWS;
	&db("update $table set checkpointkey=0") || return &fail($dberror);
	}

return &success("Successfully restored $count records from checkpoint $ARGS{KEY}");
}

=pod

=head1 restore_checkpoint

=over

=item B<NAME>

restore_checkpoint

=item B<SYNOPSIS>

restore_checkpoint B<KEY>=I<ckptkey>

=item B<DESCRIPTION>

Restores a previously created checkpoint, replacing the account,
subaccount, allocation, suballocation, reservation, subreservation,
defaultaccount tables while preserving the txnlog table. Automatically
creates a safety checkpoint first

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating the number of records restored 

=back

=cut


##############################################################
# remove_checkpoint KEY=<ckptkey>
# Returns a message indicating success or failure
##############################################################
sub remove_checkpoint {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

unless (&authorize('bank',@_))
        { return &deny("$me is not authorized to run $ARGS{COMMAND}"); }

if ($ARGS{USAGE}||!$ARGS{KEY})
        { return &usage("$ARGS{COMMAND} KEY=<ckptkey>"); }

@TABLES=("account","subaccount","allocation","suballocation","reservation","subreservation");

&db("delete from checkpoint where checkpointkey=$ARGS{KEY}") || return &fail($dberror);

foreach $table (@TABLES) {
	&db("delete from x$table where checkpointkey=$ARGS{KEY}") || return &fail($dberror);
	$count+=$ROWS;
	}

return &success("Successfully removed checkpoint $ARGS{KEY}");
}

=pod

=head1 remove_checkpoint

=over 4

=item B<NAME>

remove_checkpoint

=item B<SYNOPSIS>

remove_checkpoint B<KEY>=I<ckptkey>

=item B<DESCRIPTION>

Removes a previously created checkpoint

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns a message indicating success or failure.

=back

=cut


##############################################################
# get_checkpoints [SHOW=<attributelist>]
# Returns an array of "checkpointkey,timestamp,reason"
##############################################################
sub get_checkpoints {
local(%ARGS); foreach (@_) {($key,$value)=split(/=/,$_);$ARGS{$key}=$value;}

if ($ARGS{USAGE})
        { return &usage("$ARGS{COMMAND} [SHOW=<attributelist>]"); }

unless ($ARGS{SHOW}) { $ARGS{SHOW}="checkpointkey,timestamp,reason" };

&db("select $ARGS{SHOW} from checkpoint") || return &fail($dberror);

return &success(join("\n",@QUERYRESULT));
}

=pod

=head1 get_checkpoints

=over 4

=item B<NAME>

get_checkpoints

=item B<SYNOPSIS>

get_checkpoints [B<SHOW>=I<attributelist>]

=item B<DESCRIPTION>

List all checkpoints

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns an array of "checkpointkey, timestamp, reason" unless
overridden by the B<SHOW> parameter.

=back

=cut


##############################################################
# ping_bank
# Attempts to determine, in a reasonably efficient way, if
# the database server is still running and the connection
# to it is still working.
##############################################################
sub ping_bank {
$rc = $dbh->ping;

if ($rc) {return &success($rc)}
else {return &fail($DBI::errstr)}
}

=pod

=head1 ping_bank

=over 4

=item B<NAME>

ping_bank

=item B<SYNOPSIS>

ping_bank

=item B<DESCRIPTION>

Attempts to determine, in a reasonably efficient way, if the database
server is still running and the connection to it is still working.

=item B<OPTIONS>

=item B<RETURN VALUES>

Returns 1 if successful, else the error message 

=back

=cut


###############################################################
# post_txn($cmd)
# Posts the immediate failing transaction to a log for reconciliation
###############################################################
sub post_txn {
local($cmd)=@_;

open(RCNLOG,">>$LOG_DIR/$RCNLOG");
print RCNLOG "$cmd\n";
close RCNLOG;
return 1;	# Necessary for syntax/boundary checking condition statements
};

=pod

=head1 post_txn

=over 4

=item B<NAME>

post_txn

=item B<SYNOPSIS>

post_txn(I<$cmd>)

=item B<DESCRIPTION>

Posts the immediate failing transaction to a log for reconciliation
Used only internal to other subroutines

=item B<OPTIONS>

=item B<RETURN VALUES>

=back

=cut


###############################################################
# help
# Prints a list of subroutines available from the server
###############################################################
sub help {
return &success(join("\n",@VALIDCMDS,"Use \"<command> USAGE=1\" to generate usage statement\n"));
}

=pod

=head1 help

=over 4

=item B<NAME>

help - get help

=item B<SYNOPSIS>

help

=item B<DESCRIPTION>

prints a list of subroutines available from the server.

=item B<OPTIONS>

=item B<RETURN VALUES>

=back

=cut


###############################################################
# min($value1,$value2)
# Returns the minimum value
###############################################################
sub min
{ return $_[0]<=$_[1]?$_[0]:$_[1]; }

###############################################################
# max($value1,$value2)
# Not currently being used
# Returns the maximum value
###############################################################
sub max
{ return $_[0]>=$_[1]?$_[0]:$_[1]; }

###############################################################
# bank_log($lvl,$text)
#
# This LOG facility functions as an encoder by logically ANDing
# the specified priority level with the following output keys:
#	0	DBGLOG	Debug (Toggled based on Value of DEBUG)
##	1	ERRLOG  Critical Failures, Fatal Errors
##	2	SECLOG	Authentication Failures
##	4	TXNLOG	Audit Trail, Transactions affecting user balances
#	8	STDERR	Warning Conditions, Syntax/Usage/Boundary Errors
#
# i.e. a priority of 24 will be logged to STDERR, the errlog,
# (and the dbglog if $DEBUG=1)
###############################################################
sub bank_log {
local($lvl,$text)=@_;

$DEBUG && do {
        open(DBGLOG,">>$LOG_DIR/$DBG_LOG");
        print DBGLOG scalar(localtime),": $text\n";
        close DBGLOG;
	};
#1 & $lvl && do {
#	open(ERRLOG,">>$LOG_DIR/$ERRLOG");
#	print ERRLOG scalar(localtime),": $text\n";
#	close ERRLOG
#	};
#2 & $lvl && do {
#	open(SECLOG,">>$LOG_DIR/$SECLOG");
#	print SECLOG scalar(localtime),": $text\n";
#	close SECLOG;
#	};
#4 & $lvl && do {
#	open(TXNLOG,">>$LOG_DIR/$TXNLOG");
#	print TXNLOG scalar(localtime),": $text\n";
#	close TXNLOG;
#	};
8 & $lvl && do { print STDERR scalar(localtime),": $text\n" };
return 1;	# Necessary for syntax/boundary checking condition statements
}

=pod

=head1 bank_log

=over 4

=item B<NAME>

bank_log

=item B<SYNOPSIS>

bank_log(I<$lvl>, I<$text>)

=item B<DESCRIPTION>

This LOG facility functions as an encoder by logically ANDing the
specivied priority level with the following output keys. 

=over 4

=item 0 => DBGLOG

Debug (Toggled based on Value of DEBUG)

#=item 1 => ERRLOG
#
#Critical Failures, Fatal Errors
#
#=item 2 => SECLOG
#
#Authentication Failures
#
#=item 4 => TXNLOG
#
#Audit Trail, Transactions affecting user balances.
#
=item 8 => STDERR

Warning Conditions, Syntax/Usage/Boundary Errors

=back

#i.e. a priority of 24 will be logged to STDERR, the ERRLOG and the DBGLOG if $DEBUG=1 
#
=item B<OPTIONS>

=item B<RETURN VALUES>

=back

=cut


###############################################################
# usage($syntax)
###############################################################
sub usage {
local($syntax)=@_;

$result="Usage: $syntax";
&bank_log(0,$result); 	# Print only to DEBUGLOG
return "STATUSCODE=0 RESULT=$result\n";
}

###############################################################
# success($result)
###############################################################
sub success {
local($result)=@_;

return "STATUSCODE=1 RESULT=$result\n";
}

###############################################################
# fail($error)
# Performs a rollback if within a transaction
###############################################################
sub fail {
local($error)=@_;

# Rollback transaction if failure occurred within
if ($WORK) {
	$dbh->rollback;
	undef $WORK;
	}

&bank_log(1,"$error");
return "STATUSCODE=0 RESULT=$error\n";
}

###############################################################
# deny($msg)
# Does not do a rollback -- perhaps in future will eliminate
###############################################################
sub deny {
local($msg)=@_;

&bank_log(2,"$msg");
return "STATUSCODE=0 RESULT=$msg\n";
}

###############################################################
# result($reply)
###############################################################
sub result {
#local($_)=@_;

#/^.*RESULT=(.*)$/;
#foreach (chomp(@_)) { &bank_log(0,"line->$_<-\n"); }
chomp(@_);
return join("\n",grep(s/^.*RESULT=//,@_));
#return $1;
}

###############################################################
# status($reply)
###############################################################
sub status {
return grep(/STATUSCODE=(\d+)/ && $1,@_);
}

###############################################################
# db($statement)
# This uses perl DBI module (database interface)
# to interface to a database command
# QBank disables autocommit so must explicitly commit all transactions
# except for those within WORK blocks (in which WORK is set)
# Returns ($error,@result) where error is null if command is successful
###############################################################
sub db {
local($statement)=@_;
local($rc);

$dberror="";
@QUERYRESULT=();

&bank_log(0,$statement);
$sth = $dbh->prepare($statement);
if ($DBI::err) { $dberror="DBI Prepare: $DBI::err: $DBI::errstr"; $dbh->rollback; return 0}
$rc = $sth->execute();
if ($DBI::err) { $dberror="DBI Execute: $DBI::err: $DBI::errstr"; $dbh->rollback; return 0}

# Fetch all rows if a select statement
if ($statement =~ /^select/) {
	while ( @row=$sth->fetchrow()) { push(@QUERYRESULT,join('|',@row)); }
	$qr=join("\n",@QUERYRESULT);
	&bank_log(0,"QUERYRESULT=$qr");
	}
$ROWS=$DBI::rows;
$dbh->commit unless $WORK;

return 1;
}

################################################################################
#
# subroutine qbank($HOST,$PORT,$COMMAND)
#
# Sends command to peer bank server and returns the parsed result and status
#
################################################################################
sub qbank
{
  my($HOST,$PORT,$COMMAND,$REQUEST,$result);
  my($sockaddr,$hostname);
  my($name,$aliases,$proto,$port,$type,$len,$thisaddr);
  my($thisport,$thatport,$response,$result);

  $HOST = shift;
  $PORT = shift;
  $COMMAND = shift;

  #
  # Establish socket connection
  #
  $sockaddr = 'S n a4 x8';
  chop ($hostname = `hostname`);
  ($name,$aliases,$proto)=getprotobyname('tcp');
  ($name,$aliases,$type,$len,$thisaddr)=gethostbyname($hostname);
  ($name,$aliases,$type,$len,$thataddr)=gethostbyname($HOST);
  $thisport=pack($sockaddr,PF_INET,0,$thisaddr);
  $thatport=pack($sockaddr,PF_INET,$PORT,$thataddr);

  unless (socket(S,PF_INET,SOCK_STREAM,$proto))
        { $STATUS=0; $MESSAGE=""; return "Cannot create socket: $!"; }
  unless (bind(S,$thisport))
        { $STATUS=0; $MESSAGE=""; return "Cannot bind socket: $!"; }
  unless (connect(S,$thatport))
        { $STATUS=0; $MESSAGE=""; return "Cannot connect socket: $!"; }

  select(S); $| = 1;              # Turn on autoflushing
  select(stdout); $| = 1;         # Select STDOUT as default output

  #
  # Build  and send command
  #
  $REQUEST="COMMAND=$COMMAND AUTH=$AUTH";
  $ESCREQUEST=$REQUEST;
  $ESCREQUEST =~ s/"/\\"/g;     # Escape quotes in checksum
  chomp($CHECKSUM = `$QAUTH "$ESCREQUEST"`);
  $REQUEST .= " CHECKSUM=$CHECKSUM";
  my $command=pack "a8 a1 A*",sprintf("%08d",length($REQUEST))," ",$REQUEST;
  print S "$command";             # Send Command to server
  @REPLY=();
  while (<S>) { push(@REPLY,$_); }        # Listen for Reply
  $reply=join("\n",@REPLY);
  $STATUS=$1 if $reply=~s/STATUSCODE=(\d*)//;
  $MESSAGE=$1 if $reply=~s/MESSAGE="(.*)"//;
  grep(s/.*RESULT=//,@REPLY);             # Parse out the RESULT
  return @REPLY;
}

################################################################################
#
# Version comparison subroutine
#
# vcmp($LHVersion,$RHVersion)
#
# Numerically compares two period-separated version numbers of the form
# 2.10.4.1 (VRMF), returning -1 if LHVersion is less than RHVersion,
# 0 if they are equal, and 1 if LHVersion is greater than RHVersion.
#
################################################################################
sub vcmp {
        my($a,$b)=  map { [ $_, (split /\./)[0,1,2] ] } @_;

        $a->[1] <=> $b->[1]  # Version
               ||
       $a->[2] <=> $b->[2]  # Release
               ||
       $a->[3] <=> $b->[3]  # Mod
               ||
       $a->[4] <=> $b->[4]  # Fix
       }

sub vlt { return vcmp(@_) == -1 }	# Version Less Than
sub veq { return vcmp(@_) == 0 }		# Version EQual
sub vgt { return vcmp(@_) == 1 }		# Version Greater Than
